package sedplugin.errors;

public class CharacterizationException extends SEDException {
	private static final long serialVersionUID = 1L;

	public final static int PARSING_ERROR = 1;
	public final static int READING_ERROR = 2;
	public final static int WRITTING_ERROR = 3;
	public final static int FETCHING_ERROR = 4;

	protected String characPath;
	protected String imgName;
	protected boolean fullCharac;
	protected int errorType;
	protected int numLine;

	public CharacterizationException(String imageName, String characPath, boolean fullCharac, int errorType, String errorDescription) {
		this(imageName, characPath, fullCharac, errorType, -1, errorDescription);
	}

	public CharacterizationException(String imageName, String characPath, boolean fullCharac, int errorType, int numLine, String errorDescription) {
		super("Characterization error", "While "+errorTypeToString(errorType)+((numLine>=0)?" (line: "+numLine+")":"")+" the "+(fullCharac?"full":"manual")+" Characterization"+((imageName!=null)?(" of \""+imageName+"\""):"")+((characPath!=null)?(" ("+characPath+")"):"")+".", errorDescription);
		this.characPath = characPath;
		this.imgName = imageName;
		this.fullCharac = fullCharac;
		this.errorType = errorType;
		this.numLine = numLine;
	}

	public CharacterizationException(String imageName, String characPath, boolean fullCharac, int errorType, Throwable cause) {
		this(imageName, characPath, fullCharac, errorType, -1, cause);
	}

	public CharacterizationException(String imageName, String characPath, boolean fullCharac, int errorType, int numLine, Throwable cause) {
		super(cause, "While "+errorTypeToString(errorType)+((numLine>=0)?" (line: "+numLine+")":"")+" the "+(fullCharac?"full":"manual")+" Characterization"+((imageName!=null)?(" of \""+imageName+"\""):"")+((characPath!=null)?(" ("+characPath+")"):"")+".");
		title = "Characterization error";
		this.characPath = characPath;
		this.imgName = imageName;
		this.fullCharac = fullCharac;
		this.errorType = errorType;
		this.numLine = numLine;
	}

	public static final String errorTypeToString(int errorType){
		switch(errorType){
		case PARSING_ERROR:
			return "parsing";
		case READING_ERROR:
			return "reading";
		case WRITTING_ERROR:
			return "writting";
		case FETCHING_ERROR:
			return "fetching";
		default:
			return "using";
		}
	}

	/**
	 * @return The characPath.
	 */
	public final String getCharacPath() {
		return characPath;
	}

	/**
	 * @return The imgName.
	 */
	public final String getImgName() {
		return imgName;
	}

	/**
	 * @return The fullCharac.
	 */
	public final boolean isFullCharac() {
		return fullCharac;
	}

	/**
	 * @return The errorType.
	 */
	public final int getErrorType() {
		return errorType;
	}

}
