package sedplugin.errors;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import java.net.URL;

import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import sedplugin.plugin.SEDPlugin;
import sedplugin.plugin.UserPreferences;

/**
 * A simple dialog to show all SEDException which has occurred since the plugin starting.
 * 
 * @author Gregory Mantelet
 * @version 04/2012
 */
public class ErrorReport extends JDialog {

	private static final long serialVersionUID = 1L;

	/** Icons to use to specify the type of the given exception: classic error or refresh error. */
	protected static ImageIcon icoError = null;
	protected static ImageIcon icoRefreshError = null;

	protected final ErrorsLog logger;

	protected DefaultListModel exceptionsList;

	protected JList lstExceptions;

	protected JLabel lblTitle;
	protected JTextArea txtOrigin;
	protected JTextArea txtMessage;
	protected JTextArea txtStackTrace;
	protected JScrollPane panStackTrace;

	protected JCheckBox chkSaveErrorsLog;
	protected JCheckBox chkViewOnlyLastErrors;

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public ErrorReport(JFrame parent){
		this(parent, null, false);
	}

	public ErrorReport(JFrame parent, boolean stackTrace){
		this(parent, null, stackTrace);
	}

	public ErrorReport(JFrame parent, ErrorsLog logger){
		this(parent, logger, false);
	}

	public ErrorReport(JFrame parent, ErrorsLog logger, boolean stackTrace){
		super(parent, "Errors report");
		if (logger == null)
			this.logger = ErrorsLog.getCurrentInstance();
		else
			this.logger = logger;

		exceptionsList = new DefaultListModel();

		build();
		panStackTrace.setVisible(stackTrace);
		chkSaveErrorsLog.setVisible(!stackTrace);

		setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		pack();
		setSize(new Dimension(700, 400));
		setLocationRelativeTo(parent);
		setResizable(true);
		setModal(true);
		setVisible(false);
	}

	/* ************* */
	/* BUILD METHODS */
	/* ************* */
	protected final void build(){
		// Get gravity icons:
		URL urlIcon;
		if (icoError == null){
			urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"bigError.gif");
			if (urlIcon != null)
				icoError = new ImageIcon(urlIcon);
		}
		if (icoRefreshError == null){
			urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"bigSynchronizeError.png");
			if (urlIcon != null)
				icoRefreshError = new ImageIcon(urlIcon);
		}

		// List:
		lstExceptions = new JList(exceptionsList);
		lstExceptions.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		lstExceptions.setVisibleRowCount(10);
		lstExceptions.addListSelectionListener(new ListExceptionListener());
		lstExceptions.setCellRenderer(new ExceptionCellRenderer());

		// Exception description panel:
		lblTitle = new JLabel("", null, JLabel.CENTER);
		lblTitle.setFont(lblTitle.getFont().deriveFont(Font.BOLD, 14));
		lblTitle.setHorizontalTextPosition(JLabel.RIGHT);
		lblTitle.setVerticalTextPosition(JLabel.CENTER);

		txtOrigin = new JTextArea(1, 100);
		txtOrigin.setOpaque(false);
		txtOrigin.setEditable(false);
		txtOrigin.setBorder(BorderFactory.createTitledBorder("Origin:"));
		txtOrigin.setCaretPosition(0);

		txtMessage = new JTextArea(10, 100);
		txtMessage.setOpaque(false);
		txtMessage.setEditable(false);
		txtMessage.setTabSize(4);
		txtMessage.setLineWrap(true);
		txtMessage.setWrapStyleWord(true);
		txtMessage.addKeyListener(new KeyListener(){
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_F1 && (e.getModifiers() & KeyEvent.CTRL_MASK) == KeyEvent.CTRL_MASK){
					panStackTrace.setVisible(!panStackTrace.isVisible());
					ErrorReport.this.validate();
				}
			}
			public void keyPressed(KeyEvent arg0) {;}
			public void keyTyped(KeyEvent e) {;}
		});

		JScrollPane panMessage = new JScrollPane(txtMessage);
		panMessage.getViewport().setOpaque(false);
		panMessage.setViewportBorder(BorderFactory.createEmptyBorder());
		panMessage.setBorder(BorderFactory.createTitledBorder("Description:"));

		txtStackTrace = new JTextArea(10, 100);
		txtStackTrace.setOpaque(false);
		txtStackTrace.setEditable(false);
		txtStackTrace.setTabSize(4);
		txtStackTrace.setLineWrap(true);
		txtStackTrace.setWrapStyleWord(true);

		panStackTrace = new JScrollPane(txtStackTrace);
		panStackTrace.getViewport().setOpaque(false);
		panStackTrace.setViewportBorder(BorderFactory.createEmptyBorder());
		panStackTrace.setBorder(BorderFactory.createTitledBorder("Stack trace:"));

		JPanel panWithoutTitle = new JPanel(new BorderLayout());
		panWithoutTitle.add(txtOrigin, BorderLayout.NORTH);
		panWithoutTitle.add(panMessage, BorderLayout.CENTER);
		panWithoutTitle.add(panStackTrace, BorderLayout.SOUTH);

		JPanel panAllDescription = new JPanel(new BorderLayout());
		panAllDescription.add(lblTitle, BorderLayout.NORTH);
		panAllDescription.add(panWithoutTitle, BorderLayout.CENTER);

		// View only last errors:
		chkViewOnlyLastErrors = new JCheckBox("Display only the last errors");
		chkViewOnlyLastErrors.setToolTipText("Display only the errors since the last synchronization with Aladin.");
		chkViewOnlyLastErrors.setSelected(true);
		chkViewOnlyLastErrors.setFont(chkViewOnlyLastErrors.getFont().deriveFont(Font.BOLD));
		chkViewOnlyLastErrors.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateErrorsList();
			}
		});

		// Save errors log:
		chkSaveErrorsLog = new JCheckBox("Save an errors log before closing SEDPlugin");
		if (UserPreferences.getCurrentInstance().saveErrorsLogBeforeClosing()){
			chkSaveErrorsLog.setSelected(true);
			chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.BOLD));
		}else
			chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.PLAIN));
		chkSaveErrorsLog.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().saveErrorsLogBeforeClosing(chkSaveErrorsLog.isSelected());
				if (chkSaveErrorsLog.isSelected())
					chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.BOLD));
				else
					chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.PLAIN));
			}
		});

		// Options panel:
		JPanel panOptions = new JPanel(new GridLayout(2, 1));
		panOptions.add(chkViewOnlyLastErrors);
		panOptions.add(chkSaveErrorsLog);

		// General:
		getContentPane().setLayout(new BorderLayout());
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, new JScrollPane(lstExceptions), panAllDescription);
		splitPane.setDividerLocation(200);
		splitPane.setOneTouchExpandable(true);

		getContentPane().add(splitPane, BorderLayout.CENTER);
		getContentPane().add(panOptions, BorderLayout.SOUTH);
	}

	private final void updateErrorsList(){
		exceptionsList.clear();
		Iterator<SEDException> itErrors = chkViewOnlyLastErrors.isSelected() ? logger.getLastErrors() : logger.getErrors();
		while(itErrors.hasNext())
			exceptionsList.addElement(itErrors.next());
		if (exceptionsList.size() > 0)
			lstExceptions.setSelectedIndex(0);
	}

	/* ********* */
	/* LISTENERS */
	/* ********* */
	private final class ListExceptionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent e) {
			if (!e.getValueIsAdjusting()){
				if (lstExceptions.getSelectedIndex() > -1){
					SEDException ex = (SEDException)lstExceptions.getSelectedValue();
					boolean refreshError = (ex instanceof RefreshException);
					if (refreshError)
						ex = ((RefreshException)ex).getError();
					// Update title:
					Color colGravity = refreshError ? Color.BLACK : Color.RED;
					lblTitle.setIcon(refreshError ? icoRefreshError : icoError);
					lblTitle.setForeground(colGravity);
					lblTitle.setText(ex.getTitle().toUpperCase());
					lblTitle.setToolTipText("<html><p>"+ex.getTitle()+"</p></html>");
					// Update origin:
					txtOrigin.setForeground(colGravity);
					txtOrigin.setText(ex.getOrigin());
					// Update message:
					txtMessage.setForeground(colGravity);
					txtMessage.setText(ex.getMessage());
					txtMessage.setCaretPosition(0);
					// Update stack trace:
					printStackTrace(ex);
					txtStackTrace.setCaretPosition(0);
				}else{
					lblTitle.setText("");
					lblTitle.setIcon(null);
					txtOrigin.setText("");
					txtMessage.setText("");
					txtStackTrace.setText("");
				}
			}
		}
	}

	/* ********* */
	/* RENDERERS */
	/* ********* */
	private final class ExceptionCellRenderer extends JLabel implements ListCellRenderer{
		private static final long serialVersionUID = 1L;

		private final ImageIcon icoLittleError;
		private final ImageIcon icoLittleSynchronizeError;

		public ExceptionCellRenderer(){
			setOpaque(true);
			URL urlIcon = null;
			urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"error.gif");
			icoLittleError = (urlIcon != null) ? new ImageIcon(urlIcon) : null;
			urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"synchronizeError.png");
			icoLittleSynchronizeError = (urlIcon != null) ? new ImageIcon(urlIcon) : null;

		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			SEDException ex = (SEDException)value;
			boolean refreshError = (ex instanceof RefreshException);
			if (refreshError)
				ex = ((RefreshException)ex).getError();
			setText(ex.getTitle());
			setIcon(refreshError ? icoLittleSynchronizeError : icoLittleError);
			setBackground((isSelected)?list.getSelectionBackground():list.getBackground());
			setForeground((isSelected)?list.getSelectionForeground():list.getForeground());
			return this;
		}

	}

	/* ************* */
	/* OTHER METHODS */
	/* ************* */
	protected final void printStackTrace(SEDException ex){
		txtStackTrace.setText(ErrorsLog.getStackTraceString((ex instanceof RefreshException) ? ((RefreshException)ex).getError() : ex));
	}

	@Override
	public void setVisible(boolean visibility) {
		if (visibility)
			updateErrorsList();
		super.setVisible(visibility);
	}

}
