package sedplugin.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import sedplugin.characterization.CharacErrors;
import sedplugin.characterization.CharacManager;
import sedplugin.characterization.Characterization;
import sedplugin.characterization.FitsMapping;
import sedplugin.characterization.HeaderFits;
import sedplugin.characterization.HeaderFits.HeaderItem;
import sedplugin.errors.CharacterizationException;
import sedplugin.errors.ErrorsLog;
import sedplugin.errors.SEDException;
import sedplugin.plugin.AladinImage;
import sedplugin.plugin.SEDPlugin;
import sedplugin.sed.unit.XAxis;
import sedplugin.sed.unit.YAxis;

public class CharacEditor extends JDialog {
	private static final long serialVersionUID = 1L;

	protected JPanel panImages;
	protected JList lstImages;

	protected JPanel panAllCharac;

	protected JPanel panInfoCharac;
	protected JLabel lblErrorKey;
	protected JLabel lblNbErrors;
	protected JLabel lblTypeKey;
	protected JLabel lblCharacType;
	protected JLabel lblFileKey;
	protected JComboBox cboFile;
	protected JButton cmdLoadPath;
	protected JButton cmdURL;
	protected JButton cmdShowFile;
	protected JButton cmdShowError;
	protected JButton cmdLoad;
	protected JButton cmdSaveAs;
	protected JButton cmdUndo;

	protected JPanel panValueCharac;
	protected JCharacField fldWlCalib;
	protected JCharacField fldWlValue;
	protected JCharacField fldWlUnit;
	protected JCharacField fldFluxCalib;
	protected JCharacField fldFluxUnit;
	protected JCharacField fldDate;

	protected boolean loadingCharac;

	protected JPanel panExitOption;
	protected JButton cmdOK;
	protected JButton cmdClose;
	protected JButton cmdHelp;

	protected JFileChooser loadCharacFileChooser;
	protected JFileChooser saveCharacFileChooser;

	protected ImageIcon icoEmpty;
	protected ImageIcon icoFull;
	protected ImageIcon icoManual;

	protected ImagesListModel listModel;
	protected int currentImage = -1;

	protected Vector<String> calibrationList;

	protected final XAxis xUnit;
	protected Vector<Object> xUnits;
	protected final YAxis yUnit;
	protected Vector<Object> yUnits;

	protected boolean anyCharacModified;

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public CharacEditor(Frame owner, XAxis xUnit, YAxis yUnit) {
		super();

		this.xUnit = xUnit;
		this.yUnit = yUnit;

		init();
		initGI();
		setLocationRelativeTo(owner);

		activate(false);
		setVisible(false);
	}

	/* ******* */
	/* METHODS */
	/* ******* */
	private void init(){
		// Set the images list:
		listModel = new ImagesListModel();

		loadCharacFileChooser = new JFileChooser();
		loadCharacFileChooser.setDialogTitle("Load a Characterization...");
		loadCharacFileChooser.setApproveButtonText("Load");
		loadCharacFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		loadCharacFileChooser.setAcceptAllFileFilterUsed(false);
		loadCharacFileChooser.addChoosableFileFilter(new ExportFileFilter("map", "Manual Characterization (*.map)"));
		loadCharacFileChooser.addChoosableFileFilter(new ExportFileFilter("xml", "Full Characterization Files (*.xml)"));
		loadCharacFileChooser.addChoosableFileFilter(new ExportFileFilter(new String[]{"xml","map"}, "All Characterization Files (*.xml, *.map)"));

		saveCharacFileChooser = new JFileChooser();
		saveCharacFileChooser.setDialogTitle("Save a Characterization...");
		saveCharacFileChooser.setApproveButtonText("Save");
		saveCharacFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		saveCharacFileChooser.setAcceptAllFileFilterUsed(false);
		saveCharacFileChooser.addChoosableFileFilter(new ExportFileFilter("map", "Manual Characterization (*.map)"));

		icoEmpty = new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"empty.gif"));
		icoFull = new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"full.gif"));
		icoManual = new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"manual.gif"));

		loadingCharac = false;

		calibrationList = new Vector<String>(2);
		calibrationList.add(FitsMapping.NOT_CALIBRATED);
		calibrationList.add(FitsMapping.CALIBRATED);

		anyCharacModified = false;
	}

	public final boolean isModified(){
		return anyCharacModified;
	}

	public void addImage(AladinImage image){
		listModel.addImage(image);
	}

	public void removeImage(AladinImage image){
		listModel.removeImage(image);
	}

	public void setCurrentImage(AladinImage img){
		if (img == null)
			lstImages.setSelectedIndex(-1);
		else
			lstImages.setSelectedValue(img.getName(), true);
	}

	private void setCurrentImage(int indImage){
		Characterization charac = null;
		HeaderFits headerFits = null;

		// De-activate the editor if there is no image:
		loadingCharac = true;
		if (indImage < 0 || indImage >= listModel.getSize()){
			activate(false);
			loadingCharac = false;
			return;
		}else{
			activate(true);
			charac = listModel.getCharacterization(indImage);

			headerFits = listModel.getHeaderFits(indImage);
			if (headerFits != null){
				fldWlCalib.setHeaderFits(headerFits, calibrationList);
				fldWlValue.setHeaderFits(headerFits);
				fldWlUnit.setHeaderFits(headerFits, xUnits);
				fldFluxCalib.setHeaderFits(headerFits, calibrationList);
				fldFluxUnit.setHeaderFits(headerFits, yUnits);
				fldDate.setHeaderFits(headerFits);
			}
			fldWlCalib.setSelectedIndex(-1);
			fldWlValue.setSelectedIndex(-1);
			fldWlUnit.setSelectedIndex(-1);
			fldFluxCalib.setSelectedIndex(-1);
			fldFluxUnit.setSelectedIndex(-1);
			fldDate.setSelectedIndex(-1);
		}
		cmdUndo.setEnabled(listModel.isModified(indImage));
		cmdShowFile.setEnabled(charac != null && ((charac.getCharacUrl() != null) || (charac.getMappingFile() != null && charac.getMappingFile().getMappingFile() != null)));

		// Load the Characterization:
		updateFile(indImage);
		updateType(charac);
		updateError(charac);
		if (charac != null){
			// Spectral calibration:
			fldWlCalib.setSelectedItem((charac.isFullCharacterization())?(charac.getErrors().isSpectralCalibrated()?FitsMapping.CALIBRATED:FitsMapping.NOT_CALIBRATED):JCharacField.toAliasedKeyword(charac.getMappingFile().getSpectralCalibration()));
			// Wavelength value:
			fldWlValue.setSelectedItem((charac.isFullCharacterization())?(charac.getWavelength()+""):JCharacField.toAliasedKeyword(charac.getMappingFile().getWavelength()));
			// Wavelength unit:
			fldWlUnit.setSelectedItem((charac.isFullCharacterization())?(charac.getWavelengthUnit()):JCharacField.toAliasedKeyword(charac.getMappingFile().getWavelengthUnit()));
			// Flux calibration:
			fldFluxCalib.setSelectedItem((charac.isFullCharacterization())?(charac.getErrors().isFluxCalibrated()?FitsMapping.CALIBRATED:FitsMapping.NOT_CALIBRATED):JCharacField.toAliasedKeyword(charac.getMappingFile().getFluxCalibration()));
			// Flux unit:
			fldFluxUnit.setSelectedItem((charac.isFullCharacterization())?(charac.getFluxUnit()):JCharacField.toAliasedKeyword(charac.getMappingFile().getFluxUnit()));
			// Date:
			fldDate.setSelectedItem((charac.isFullCharacterization())?(charac.getDate()):JCharacField.toAliasedKeyword(charac.getMappingFile().getDate()));
		}else{
			fldWlCalib.setSelectedItem(FitsMapping.CALIBRATED);
			fldFluxCalib.setSelectedItem(FitsMapping.CALIBRATED);
		}

		loadingCharac = false;
	}

	private void updateFile(int indImage) {
		// Get the Characterization:
		Characterization charac = listModel.getCharacterization(indImage);

		// Set the File history of this Characterization:
		cboFile.setModel(listModel.getFileHistory(indImage));

		// CASE: No Charac !
		if (charac == null){
			cboFile.setSelectedIndex(-1);
			cboFile.setToolTipText(null);

		}// CASE: Full Charac (*.xml):
		else if (charac.isFullCharacterization()){
			cboFile.setSelectedItem(charac.getCharacUrl().toString());
			cboFile.setToolTipText(charac.getCharacUrl().toString());

		}// CASE: Manual Charac (*.map) WITH an existing file:
		else if (charac.getMappingFile().getMappingFile() != null){
			cboFile.setSelectedItem(charac.getMappingFile().getMappingFile().getAbsolutePath());
			cboFile.setToolTipText(charac.getMappingFile().getMappingFile().getAbsolutePath());

		}// CASE: Manual Charac (*.map) WITHOUT an existing file:
		else{
			cboFile.setSelectedIndex(-1);
			cboFile.setToolTipText(null);
		}
	}

	private void updateType(Characterization charac){
		// CASE: No Charac !
		if (charac == null){
			lblCharacType.setIcon(icoEmpty);
			lblCharacType.setText("No Characterization");
			cmdSaveAs.setEnabled(false);

			// CASE: Full Charac:
		}else if (charac.isFullCharacterization()){
			lblCharacType.setIcon(icoFull);
			lblCharacType.setText("Full Characterization");

			// OTHERWISE:
		}else{
			lblCharacType.setIcon(icoManual);
			lblCharacType.setText("Manual Characterization");
		}
	}

	private void updateError(Characterization charac){
		// CASE: Charac WITH error:
		if (charac != null && !charac.isCorrect()){
			cmdShowError.setVisible(true);
			lblNbErrors.setText(charac.getErrors().countErrors()+" errors");

			CharacErrors errors = charac.getErrors();
			if (errors != null){
				if (!errors.isSpectralCalibrated()){
					if (errors.wlCalibrationError != null)
						fldWlCalib.setError(errors.wlCalibrationError);
					else
						fldWlCalib.setError("The spectral axis must be calibrated !");
				}else
					fldWlCalib.setError(null);

				fldWlValue.setError(errors.wlValueError);
				fldWlUnit.setError(errors.wlUnitError);

				if (!errors.isFluxCalibrated()){
					if (errors.fluxCalibrationError != null)
						fldFluxCalib.setError(errors.fluxCalibrationError);
					else
						fldFluxCalib.setError("The flux axis must be calibrated !");
				}else
					fldFluxCalib.setError(null);

				fldFluxUnit.setError(errors.fluxUnitError);
				fldDate.setError(errors.dateError);
			}

			// CASE: No error !
		}else{
			cmdShowError.setVisible(false);
			lblNbErrors.setText("None");

			fldWlCalib.setError(null);
			fldWlValue.setError(null);
			fldWlUnit.setError(null);
			fldFluxCalib.setError(null);
			fldFluxUnit.setError(null);
			fldDate.setError(null);
		}
	}

	private Characterization extractManualCharacterization(Characterization charac, int indImg, boolean askConfirmation){
		AladinImage img = listModel.getImage(indImg);

		if (img != null){
			// CASE: No Charac => Create a new FitsMapping.
			if (charac == null){
				charac = new Characterization(img, new FitsMapping());

			}// CASE: Full Charac => Create a new FitsMapping initialized with the full Characterization.
			else if (charac.isFullCharacterization()){
				//				if (askConfirmation){
				//					int response = JOptionPane.showConfirmDialog(this, "<html><p style=\"text-align: justify; text-indent:2em;\">The Characterization of \""+img.getName()+"\" is a full Characterization (*.xml).<br />Modifications of this Characterization can be saved ONLY in a manual Characterization file (*.map).</p><br /><p style=\"text-align: justify; text-indent:2em;\">Do you really want to modify this Characterization ?</p></html>", "Characterization modified", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				//					if (response == JOptionPane.NO_OPTION)
				//						return null;
				//				}
				charac = new Characterization(img, new FitsMapping(charac));

			}// CASE: Manual Charac => Create a copy of the given Characterization.
			else
				charac = new Characterization(img, new FitsMapping(charac.getMappingFile()));
		}

		return charac;
	}

	public void clear(){
		// charac file:
		updateFile(-1);
		// charac type:
		updateType(null);
		// charac error:
		updateError(null);
		// charac fields:
		fldWlCalib.setSelectedItem(null);
		fldWlValue.setSelectedItem(null);
		fldWlUnit.setSelectedItem(null);
		fldFluxCalib.setSelectedItem(null);
		fldFluxUnit.setSelectedItem(null);
		fldDate.setSelectedItem(null);
	}

	public void activate(boolean activateOrNot){
		cmdUndo.setEnabled(activateOrNot);
		cmdLoad.setEnabled(activateOrNot);
		cmdSaveAs.setEnabled(activateOrNot);
		cmdShowFile.setEnabled(activateOrNot);

		lblFileKey.setEnabled(activateOrNot);
		cboFile.setEnabled(activateOrNot);
		cmdLoadPath.setEnabled(activateOrNot);

		lblTypeKey.setEnabled(activateOrNot);
		lblCharacType.setEnabled(activateOrNot);

		lblErrorKey.setEnabled(activateOrNot);
		lblNbErrors.setEnabled(activateOrNot);
		cmdShowError.setVisible(activateOrNot);

		fldWlCalib.setEnabled(activateOrNot);
		fldWlValue.setEnabled(activateOrNot);
		fldWlUnit.setEnabled(activateOrNot);
		fldFluxCalib.setEnabled(activateOrNot);
		fldFluxUnit.setEnabled(activateOrNot);
		fldDate.setEnabled(activateOrNot);

		if (!activateOrNot)
			clear();
	}

	private void undo(){
		listModel.setCharacterization(null, currentImage);
		// Ensures the Apply button is enabled:
		cmdOK.setEnabled(listModel.getNbModification() > 0);
		setCurrentImage(currentImage);
		lstImages.repaint();
	}

	private void loadFile(){
		AladinImage io = listModel.getImage(currentImage);

		if (io == null){
			cmdLoad.setEnabled(false);
			return;
		}

		// Get the current Characterization:
		Characterization charac = listModel.getCharacterization(currentImage);

		// Set the default Characterizations directory:
		loadCharacFileChooser.setSelectedFile(new File(System.getProperty("user.home"), io.getName()+".xml"));

		// Set the corresponding file into the file chooser:
		if (charac != null){
			if (charac.isFullCharacterization()){
				if (charac.getCharacUrl() != null && charac.getCharacUrl().getProtocol().equalsIgnoreCase("file"))
					loadCharacFileChooser.setSelectedFile(new File(charac.getCharacUrl().toString()));
			}else if (charac.getMappingFile().getMappingFile() != null)
				loadCharacFileChooser.setSelectedFile(charac.getMappingFile().getMappingFile());
		}else{
			if (io.getImageUrl() != null && io.getImageUrl().getProtocol().equalsIgnoreCase("file")){
				File f = new File(io.getImageUrl().toString());
				loadCharacFileChooser.setCurrentDirectory(f.getParentFile());
			}
		}

		// Show the dialog and get the selected file:
		if (loadCharacFileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION){
			try{
				// Load the Characterization:
				AladinImage img = listModel.getImage(currentImage);
				boolean fullCharac = loadCharacFileChooser.getSelectedFile().getName().endsWith(".xml");
				if (fullCharac)
					charac = new Characterization(img, loadCharacFileChooser.getSelectedFile().toURI().toURL());
				else{
					FitsMapping mapping = new FitsMapping(loadCharacFileChooser.getSelectedFile());
					charac = new Characterization(img, mapping);
				}

				// Refresh:
				listModel.setCharacterization(charac, currentImage);
				lstImages.repaint();
				// Ensures the Apply button is enabled:
				cmdOK.setEnabled(listModel.getNbModification() > 0);
				if (loadCharacFileChooser.getSelectedFile() != null)
					updateFileHistory((DefaultComboBoxModel)cboFile.getModel(), loadCharacFileChooser.getSelectedFile().getAbsolutePath());
				setCurrentImage(currentImage);

			}catch(CharacterizationException ex){
				JOptionPane.showMessageDialog(this, "<html>"+ex.toHTMLString()+"</html>", "Loading Characterization...", JOptionPane.ERROR_MESSAGE);
			}catch(MalformedURLException mue){
				JOptionPane.showMessageDialog(this, mue.getMessage(), "Loading Characterization...", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private boolean loadFile(String path, boolean displayError){
		// Get the index of the selected image:
		if (currentImage < 0 || currentImage >= listModel.getSize()){
			cmdLoad.setEnabled(false);
			return false;
		}

		boolean loaded = false;

		// Get the current Characterization and its image:
		Characterization charac = listModel.getCharacterization(currentImage);
		AladinImage img = listModel.getImage(currentImage);

		try{
			// Load the Characterization:
			boolean fullCharac = path.startsWith("http:") || path.endsWith(".xml");
			if (fullCharac){
				if (!path.startsWith("http:") && !path.startsWith("file:"))
					path = "file:"+path;
				charac = new Characterization(img, new URL(path));
			}else{
				FitsMapping mapping = new FitsMapping(new File(path));
				charac = new Characterization(img, mapping);
			}

			// Refresh:
			listModel.setCharacterization(charac, currentImage);
			lstImages.repaint();
			// Ensures the Apply button is enabled:
			cmdOK.setEnabled(listModel.getNbModification() > 0);
			setCurrentImage(currentImage);
			loaded = true;

		}catch(CharacterizationException ex){
			if (displayError)
				JOptionPane.showMessageDialog(this, "<html>"+ex.toHTMLString()+"</html>", "Loading Characterization...", JOptionPane.ERROR_MESSAGE);
		}catch(MalformedURLException mue){
			if (displayError)
				JOptionPane.showMessageDialog(this, mue.getMessage(), "Loading Characterization...", JOptionPane.ERROR_MESSAGE);
		}

		return loaded;
	}

	private void updateFileHistory(DefaultComboBoxModel model, String itemToAdd){
		if (itemToAdd == null || model  == null)
			return;

		itemToAdd = itemToAdd.trim();
		if (itemToAdd.length() == 0)
			return;

		if (itemToAdd.startsWith("file:"))
			itemToAdd = itemToAdd.substring(5);

		boolean exists = false;
		for(int i=0; !exists && i<model.getSize(); i++){
			String item = model.getElementAt(i).toString();
			if (item.startsWith("file:"))
				item = item.substring(5);
			exists = item.equals(itemToAdd);
		}

		if (!exists)
			model.insertElementAt(itemToAdd, 0);
	}

	private void loadGivenPath(){
		if (cboFile.getSelectedItem() != null){
			boolean loaded = true;
			if (!loadingCharac)
				// Load the file as a Characterization:
				loaded = loadFile(cboFile.getSelectedItem().toString(), true);

			// Add this path to the history:
			if (loaded){
				if (cboFile.getSelectedItem() != null)
					updateFileHistory((DefaultComboBoxModel)cboFile.getModel(), cboFile.getSelectedItem().toString());
			}else{
				loadingCharac = true;
				updateFile(currentImage);
				loadingCharac = false;
			}
		}
	}

	private Characterization save(Characterization charac, AladinImage io){
		// CASE: No Charac:
		if (charac == null)
			return null;

		// CASE: Full Charac:
		else if (charac.isFullCharacterization()){
			return saveAs(charac, io);

			// CASE: Manual Charac NOT saved:
		}else if (!charac.getMappingFile().isSaved()){
			FitsMapping mapping = charac.getMappingFile();
			try {
				if (mapping.getMappingFile() == null)
					return saveAs(charac, io);
				else{
					if (mapping.save())
						return charac;
					else
						return null;
				}
			} catch (CharacterizationException e) {
				JOptionPane.showMessageDialog(this, "<html>"+e.toHTMLString()+"</html>", "Saving Characterization...", JOptionPane.ERROR_MESSAGE);
				return null;
			}

			// CASE: Manual Charac saved:
		}else
			return saveAs(charac, io);
	}

	private void saveAs(){
		AladinImage io = listModel.getImage(currentImage);

		if (io != null){
			Characterization charac = listModel.getCharacterization(currentImage);
			charac = saveAs(charac, io);
			if (charac != null){
				// Store this Characterization:
				listModel.setCharacterization(charac, currentImage);
				// Refresh:
				lstImages.repaint();
				// Ensures the Apply button is enabled:
				cmdOK.setEnabled(listModel.getNbModification() > 0);
				updateFile(currentImage);
				updateType(charac);
				cmdUndo.setEnabled(true);

				cmdShowFile.setEnabled(true);
			}
		}
	}

	private Characterization saveAs(Characterization charac, AladinImage io){
		if (charac == null || io == null)
			return null;

		// Set the default Characterization file:
		File defaultFile = CharacManager.getDefaultMap(io.getImageUrl());
		saveCharacFileChooser.setSelectedFile((defaultFile!=null)?defaultFile:(new File(new File(System.getProperty("user.home")), io.getName()+".map")));

		// CASE: Full Charac => default characterization directory + imgName + .map:
		if (charac.isFullCharacterization()){
			if (!charac.getCharacUrl().getProtocol().equalsIgnoreCase("file"))
				saveCharacFileChooser.setSelectedFile(new File(new File(System.getProperty("user.home")), io.getName()+".map"));

			// CASE: Manual Charac:
		}else{
			if (charac.getMappingFile().getMappingFile() != null)
				saveCharacFileChooser.setSelectedFile(charac.getMappingFile().getMappingFile());
		}

		// Show the dialog:
		boolean reask;
		do{
			reask = false;
			int response = saveCharacFileChooser.showSaveDialog(this);
			if (response == JFileChooser.APPROVE_OPTION){
				charac = extractManualCharacterization(charac, currentImage, false);
				try{
					File sauv = saveCharacFileChooser.getSelectedFile();
					// Add the extension .map if absent:
					if (!sauv.getName().endsWith(".map"))
						sauv = new File(sauv.getParentFile(), sauv.getName()+".map");
					// Ask confirmation before saving a generic Characterization:
					if (sauv.getName().equals("generic.map")){
						response = JOptionPane.showConfirmDialog(this, "Are you sure you want updating a generic Characterization ?", "Save a generic.map", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
						if (response == JOptionPane.NO_OPTION){
							reask = true;
							continue;
						}
					}
					// Save the Characterization in the specified file:
					if (charac.getMappingFile().save(sauv))
						return charac;
				}catch(CharacterizationException e){
					JOptionPane.showMessageDialog(this, "<html>"+e.toHTMLString()+"</html>", "Saving Characterization...", JOptionPane.ERROR_MESSAGE);
					return null;
				}
			}
		}while(reask);
		return null;
	}

	@SuppressWarnings("deprecation")
	private void showFile(){
		AladinImage img = listModel.getImage(currentImage);
		if (img != null){
			// Get the Characterization:
			Characterization charac = listModel.getCharacterization(currentImage);
			try{
				// CASE: No Charac:
				if (charac == null)
					cmdShowFile.setEnabled(false);
				// CASE: Full Charac:
				else if (charac.isFullCharacterization())
					new CharacFileDialog(img.getName(), charac.getCharacUrl());
				// CASE: Manual Charac WITH a file:
				else if (charac.getMappingFile().getMappingFile() != null)
					new CharacFileDialog(img.getName(), charac.getMappingFile().getMappingFile().toURL());
				// CASE: Manual Charac WITHOUT a file:
				else
					JOptionPane.showMessageDialog(this, "No Characterization file !", "Show file", JOptionPane.INFORMATION_MESSAGE);
			} catch (SEDException e) {
				JOptionPane.showMessageDialog(this, "<html>"+e.toHTMLString()+"</html>", "Showing Characterization...", JOptionPane.ERROR_MESSAGE);
			} catch (MalformedURLException e) {
				JOptionPane.showMessageDialog(this, "Impossible to show the Characterization file because:\n"+e.getMessage(), "Showing Characterization...", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private void showError(){
		// Get the error of the current Characterization:
		Characterization charac = listModel.getCharacterization(currentImage);

		// Show the error:
		if (charac != null && !charac.isCorrect() && charac.getErrors() != null)
			JOptionPane.showMessageDialog(this, charac.getErrors().getAllErrors(true), "Characterization error", JOptionPane.ERROR_MESSAGE);
	}

	private void displayHelp(){
		// TODO Display the help !
	}

	/* *** */
	/* IHM */
	/* *** */
	private void initGI(){
		/* WEST : List of loaded images */
		panImages = new JPanel(new BorderLayout());
		panImages.setPreferredSize(new Dimension(250,100));
		panImages.setBorder(BorderFactory.createTitledBorder("Loaded images"));
		// List - Images list:
		lstImages = new JList();
		lstImages.setModel(listModel);
		lstImages.setCellRenderer(new ImageCellRenderer());
		lstImages.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane scrollPaneImages = new JScrollPane(lstImages);
		lstImages.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()){
					currentImage = lstImages.getSelectedIndex();
					setCurrentImage(currentImage);
				}
			}
		});
		panImages.add(scrollPaneImages, BorderLayout.CENTER);

		/* CENTER : Characterization Info & Value */
		panAllCharac = new JPanel();
		panAllCharac.setLayout(new BoxLayout(panAllCharac, BoxLayout.Y_AXIS));
		panAllCharac.setBorder(BorderFactory.createTitledBorder("Characterization"));

		panAllCharac.add(getCharacCommands());

		// Panel - Characterization Information:
		panInfoCharac = new JPanel();
		panInfoCharac.setLayout(new BoxLayout(panInfoCharac, BoxLayout.Y_AXIS));
		panInfoCharac.setAlignmentX(LEFT_ALIGNMENT);
		JPanel panMetaInfo = new JPanel(new GridLayout(3,1));
		panMetaInfo.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		panMetaInfo.setAlignmentX(LEFT_ALIGNMENT);
		panMetaInfo.add(getFileRow());
		panMetaInfo.add(getCharacType());
		panMetaInfo.add(getErrorRow());
		panInfoCharac.add(Box.createVerticalStrut(10));
		panInfoCharac.add(panMetaInfo);
		panInfoCharac.add(Box.createVerticalStrut(10));
		panAllCharac.add(panInfoCharac);

		// Panel - Characterization Value:
		panValueCharac = getCharacForm();
		panValueCharac.setAlignmentX(LEFT_ALIGNMENT);
		panAllCharac.add(panValueCharac);

		/* SOUTH : Exit options */
		panExitOption = getExitOptions();

		/* GENERAL PANEL */
		Container panGeneral = getContentPane();
		panGeneral.setLayout(new BorderLayout());
		panGeneral.add(panImages, BorderLayout.WEST);
		panGeneral.add(panAllCharac, BorderLayout.CENTER);
		panGeneral.add(panExitOption, BorderLayout.SOUTH);

		/* GENERAL */
		setTitle("Characterization editor");
		pack();
		URL iconTitle = getClass().getResource(SEDPlugin.ICONS_DIR + "sed.gif");
		if (iconTitle != null)
			setIconImage(new ImageIcon(iconTitle).getImage());
		setVisible(false);
		setResizable(false);
		setDefaultCloseOperation(HIDE_ON_CLOSE);
	}

	@Override
	public void setVisible(boolean b) {
		if (b)
			anyCharacModified = false;
		else{
			// Clear the units lists:
			xUnits = null;
			yUnits = null;
		}
		super.setVisible(b);
	}

	private JPanel getCharacCommands(){
		// Panel - Characterization File Commands:
		JPanel panCharacCommands = new JPanel(new GridLayout(1, 4, 2, 0));
		panCharacCommands.setBorder(BorderFactory.createRaisedBevelBorder());
		panCharacCommands.setAlignmentX(LEFT_ALIGNMENT);

		// Button - Undo:
		cmdUndo = new JButton("Default", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"undo.gif")));
		cmdUndo.setToolTipText("Undoes all modifications done on this Characterization.");
		cmdUndo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				undo();
			}
		});
		panCharacCommands.add(cmdUndo);

		// Button - Load:
		cmdLoad = new JButton("Load", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"open.gif")));
		cmdLoad.setToolTipText("Loads a Characterization: either full (*.xml) or manual (*.map).");
		cmdLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadFile();
			}
		});
		panCharacCommands.add(cmdLoad);

		// Button - Save as:
		cmdSaveAs = new JButton("Save",new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"save.gif")));
		cmdSaveAs.setToolTipText("Saves the current Characterization...");
		cmdSaveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveAs();
			}
		});
		panCharacCommands.add(cmdSaveAs);

		// Button - Show file:
		cmdShowFile = new JButton("Show", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"preview.gif")));
		cmdShowFile.setToolTipText("Show the current Characterization file.");
		cmdShowFile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showFile();
			}
		});
		panCharacCommands.add(cmdShowFile);

		return panCharacCommands;
	}

	private JPanel getFileRow(){
		// Panel - File & Show & Errors:
		JPanel panFileShowErrors = new JPanel();
		panFileShowErrors.setLayout(new BorderLayout());

		// Label - File key:
		lblFileKey = new JLabel("File: ");
		lblFileKey.setFont(lblFileKey.getFont().deriveFont(Font.BOLD));
		panFileShowErrors.add(lblFileKey, BorderLayout.WEST);

		// Label - File path:
		cboFile = new JComboBox();
		cboFile.setModel(new DefaultComboBoxModel());
		cboFile.setEditable(true);
		cboFile.setEditor(new FileEditor());
		cboFile.setRenderer(new FileRenderer());
		cboFile.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (!loadingCharac && ((FileEditor)cboFile.getEditor()).hasFocus() && e.getStateChange() == ItemEvent.SELECTED && cboFile.getSelectedIndex() > -1){
					loadGivenPath();
				}
			}
		});
		cboFile.setAlignmentX(LEFT_ALIGNMENT);
		cboFile.setFont(cboFile.getFont().deriveFont(Font.PLAIN));
		cboFile.setOpaque(true);
		panFileShowErrors.add(cboFile, BorderLayout.CENTER);

		// Button - Load:
		cmdLoadPath = new JButton(new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"refresh.gif")));
		cmdLoadPath.setToolTipText("Load the file whose the path/URL is given. The [Enter] key has the same effect.");
		cmdLoadPath.setAlignmentX(LEFT_ALIGNMENT);
		cmdLoadPath.setEnabled(false);
		cmdLoadPath.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadGivenPath();
			}
		});
		panFileShowErrors.add(cmdLoadPath, BorderLayout.EAST);

		return panFileShowErrors;
	}

	private JPanel getCharacType(){
		// Panel - Charac Type:
		JPanel panCharacType = new JPanel();
		panCharacType.setLayout(new BorderLayout(5,0));

		// Label - Type key:
		lblTypeKey = new JLabel("Type: ");
		lblTypeKey.setFont(lblTypeKey.getFont().deriveFont(Font.BOLD));
		panCharacType.add(lblTypeKey, BorderLayout.WEST);

		// Label - Characterization Type:
		lblCharacType = new JLabel();
		lblCharacType.setAlignmentX(LEFT_ALIGNMENT);
		lblCharacType.setFont(lblCharacType.getFont().deriveFont(Font.PLAIN));
		panCharacType.add(lblCharacType, BorderLayout.CENTER);

		return panCharacType;
	}

	private JPanel getErrorRow(){
		// Panel - Charac errors:
		JPanel panErrors = new JPanel();
		panErrors.setLayout(new BorderLayout(5,0));

		// Label - Errors key:
		lblErrorKey = new JLabel("Error: ");
		lblErrorKey.setFont(lblErrorKey.getFont().deriveFont(Font.BOLD));
		panErrors.add(lblErrorKey, BorderLayout.WEST);

		// Label - Nb Errors:
		lblNbErrors = new JLabel("None");
		lblNbErrors.setFont(lblNbErrors.getFont().deriveFont(Font.PLAIN));
		panErrors.add(lblNbErrors, BorderLayout.CENTER);

		// Button - Show errors:
		cmdShowError = new JButton("Show all errors", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"tinyError.gif")));
		cmdShowError.setAlignmentX(LEFT_ALIGNMENT);
		cmdShowError.setVisible(true);
		cmdShowError.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showError();
			}
		});
		panErrors.add(cmdShowError, BorderLayout.EAST);

		panErrors.setPreferredSize(new Dimension(panErrors.getPreferredSize().width, cmdShowError.getPreferredSize().height));

		return panErrors;
	}

	private JPanel getCharacForm(){
		JPanel panCharacForm = new JPanel();
		panCharacForm.setLayout(new GridLayout(6,1,0,2));

		// Spectral calibration:
		fldWlCalib = new JCharacField("Spectral calibration", FitsMapping.SPECTRAL_CALIBRATION_UTYPE, HeaderFits.STRING, 170, 100);
		fldWlCalib.addItemListener(new CharacFieldListener(fldWlCalib){
			@Override
			public String getDefaultValue(Characterization charac) {
				if (charac == null)
					return null;
				else if (charac.isFullCharacterization())
					return charac.getSpectralCalibration();
				else
					return charac.getMappingFile().getSpectralCalibration();
			}
			@Override
			public void updateMapping(FitsMapping mapping, String value) {
				mapping.setSpectralCalibration(value);
			}
			@Override
			public void displayError(Characterization charac){
				if (!charac.getErrors().isSpectralCalibrated()){
					if (charac.getErrors().wlCalibrationError != null)
						fldWlCalib.setError(charac.getErrors().wlCalibrationError);
					else
						fldWlCalib.setError("The spectral axis must be calibrated !");
				}else
					fldWlCalib.setError(null);
			}
		});
		panCharacForm.add(fldWlCalib);

		// Wavelength value:
		fldWlValue = new JCharacField("Spectral value", FitsMapping.WAVELENGTH_VALUE_UTYPE, HeaderFits.NUMERIC, fldWlCalib.getNameWidth(), 100);
		fldWlValue.setExample("12");
		fldWlValue.addItemListener(new CharacFieldListener(fldWlValue){
			@Override
			public String getDefaultValue(Characterization charac) {
				if (charac == null)
					return null;
				else if (charac.isFullCharacterization())
					return charac.getWavelength()+"";
				else
					return charac.getMappingFile().getWavelength();
			}
			@Override
			public void updateMapping(FitsMapping mapping, String value) {
				mapping.setWavelength(value);
			}
			@Override
			public void displayError(Characterization charac){
				fldWlValue.setError(charac.getErrors().wlValueError);
			}
		});
		panCharacForm.add(fldWlValue);

		// Wavelength unit:
		fldWlUnit = new JCharacField("Spectral unit", FitsMapping.WAVELENGTH_UNIT_UTYPE, HeaderFits.STRING, fldWlCalib.getNameWidth(), 100);
		fldWlUnit.setExample("um");
		fldWlUnit.addItemListener(new CharacFieldListener(fldWlUnit){
			@Override
			public String getDefaultValue(Characterization charac) {
				if (charac == null)
					return null;
				else if (charac.isFullCharacterization())
					return charac.getWavelengthUnit();
				else
					return charac.getMappingFile().getWavelengthUnit();
			}
			@Override
			public void updateMapping(FitsMapping mapping, String value) {
				mapping.setWavelengthUnit(value);
			}
			@Override
			public void displayError(Characterization charac){
				fldWlUnit.setError(charac.getErrors().wlUnitError);
			}
		});
		panCharacForm.add(fldWlUnit);

		// Flux calibration:
		fldFluxCalib = new JCharacField("Flux calibration", FitsMapping.FLUX_CALIBRATION_UTYPE, HeaderFits.STRING, fldWlCalib.getNameWidth(), 100);
		fldFluxCalib.addItemListener(new CharacFieldListener(fldFluxCalib){
			@Override
			public String getDefaultValue(Characterization charac) {
				if (charac == null)
					return null;
				else if (charac.isFullCharacterization())
					return charac.getFluxCalibration();
				else
					return charac.getMappingFile().getFluxCalibration();
			}
			@Override
			public void updateMapping(FitsMapping mapping, String value) {
				mapping.setFluxCalibration(value);
			}
			@Override
			public void displayError(Characterization charac){
				if (!charac.getErrors().isFluxCalibrated()){
					if (charac.getErrors().fluxCalibrationError != null)
						fldFluxCalib.setError(charac.getErrors().fluxCalibrationError);
					else
						fldFluxCalib.setError("The flux axis must be calibrated !");
				}else
					fldFluxCalib.setError(null);
			}
		});
		panCharacForm.add(fldFluxCalib);

		// Flux unit:
		fldFluxUnit = new JCharacField("Flux unit", FitsMapping.FLUX_UNIT_UTYPE, HeaderFits.STRING, fldWlCalib.getNameWidth(), 100);
		fldFluxUnit.setExample("MJy/sr");
		fldFluxUnit.addItemListener(new CharacFieldListener(fldFluxUnit){
			@Override
			public String getDefaultValue(Characterization charac) {
				if (charac == null)
					return null;
				else if (charac.isFullCharacterization())
					return charac.getFluxUnit();
				else
					return charac.getMappingFile().getFluxUnit();
			}
			@Override
			public void updateMapping(FitsMapping mapping, String value) {
				mapping.setFluxUnit(value);
			}
			@Override
			public void displayError(Characterization charac){
				fldFluxUnit.setError(charac.getErrors().fluxUnitError);
			}
		});
		panCharacForm.add(fldFluxUnit);

		// Date:
		fldDate = new JCharacField("Date (optional)", FitsMapping.DATE_UTYPE, JCharacField.ALL_KEYWORDS, fldWlCalib.getNameWidth(), 100);
		fldDate.setExample("13/10/2010");
		fldDate.addItemListener(new CharacFieldListener(fldDate){
			@Override
			public String getDefaultValue(Characterization charac) {
				if (charac == null)
					return null;
				else if (charac.isFullCharacterization())
					return charac.getDate();
				else
					return charac.getMappingFile().getDate();
			}
			@Override
			public void updateMapping(FitsMapping mapping, String value) {
				mapping.setDate(value);
			}
			@Override
			public void displayError(Characterization charac){
				fldDate.setError(charac.getErrors().dateError);
			}
		});
		panCharacForm.add(fldDate);

		return panCharacForm;
	}

	private JPanel getExitOptions(){
		JPanel panOptions = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		// Button - OK:
		cmdOK = new JButton("Apply", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"apply.gif")));
		cmdOK.setToolTipText("APPLY all modifications and close the editor.");
		cmdOK.setDefaultCapable(true);
		cmdOK.setEnabled(false);
		cmdOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				listModel.applyAll();
				cmdOK.setEnabled(false);
				setCurrentImage(currentImage);
				lstImages.repaint();
			}
		});

		// Button - Close:
		cmdClose = new JButton("Close", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"delete.gif")));
		cmdClose.setToolTipText("Close the Characterization editor");
		cmdClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});

		// Button - Help:
		cmdHelp = new JButton("Help", new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"help.gif")));
		cmdHelp.setEnabled(false);
		cmdHelp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				displayHelp();
			}
		});

		JPanel panTemp = new JPanel(new GridLayout(1,3));
		panTemp.add(cmdOK);
		panTemp.add(cmdClose);
		panTemp.add(cmdHelp);

		panOptions.add(panTemp);

		return panOptions;
	}

	private abstract class CharacFieldListener implements ItemListener {

		private JCharacField characField;

		public CharacFieldListener(JCharacField field){
			characField = field;
		}

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED && characField != null && currentImage >= 0){
				if (!loadingCharac){
					Characterization charac = listModel.getCharacterization(currentImage);

					if (!listModel.isModified(currentImage)){
						Characterization manualCharac = extractManualCharacterization(charac, currentImage, true);
						if (manualCharac == null){
							loadingCharac = true;
							characField.setSelectedItem(JCharacField.toAliasedKeyword(getDefaultValue(charac)));
							loadingCharac = false;
							return;
						}else
							charac = manualCharac;
					}

					if (charac != null){
						if (!charac.isFullCharacterization()){
							// Update the value in the mapping object:
							FitsMapping mapping = charac.getMappingFile();
							String value = JCharacField.toFitsKeyword(characField.getSelectedItem().toString());
							if (characField.getSelectedItem() instanceof HeaderItem)
								value = FitsMapping.getFitsKeyword(((HeaderItem)characField.getSelectedItem()).keyword);
							updateMapping(mapping, value);
							// Update the Characterization:
							charac.reloadMapping();

							if (!listModel.isModified(currentImage)){
								// Set the new Characterization:
								listModel.setCharacterization(charac, currentImage);
								// Refresh the display:
								lstImages.repaint();
								// Ensures the Apply button is enabled:
								cmdOK.setEnabled(listModel.getNbModification() > 0);
								setCurrentImage(currentImage);
								return;
							}
						}
					}

					if (currentImage != lstImages.getSelectedIndex()){
						setCurrentImage(lstImages.getSelectedIndex());
						return;
					}

					// Refresh the displayed Characterization:
					lstImages.repaint();
					updateError(charac);
				}
			}
		}

		public abstract String getDefaultValue(Characterization charac);
		public abstract void updateMapping(FitsMapping mapping, String value);
		public abstract void displayError(Characterization charac);
	}

	private class FileEditor extends JTextField implements ComboBoxEditor {
		private static final long serialVersionUID = 1L;

		public FileEditor(){
			this.addKeyListener(new KeyListener() {
				public void keyTyped(KeyEvent event) {;}
				public void keyPressed(KeyEvent arg0) {;}
				public void keyReleased(KeyEvent event) {
					if (event.getKeyCode() == KeyEvent.VK_ENTER)
						loadGivenPath();
					else if (event.getKeyCode() == KeyEvent.VK_ESCAPE){
						loadingCharac = true;
						updateFile(currentImage);
						loadingCharac = false;
					}
				}
			});
		}

		public Component getEditorComponent() {
			return this;
		}

		public Object getItem() {
			return getText();
		}

		public void setItem(Object anObject) {
			if (anObject == null)
				setText(null);
			else{
				String path = anObject.toString();
				if (path.startsWith("file:"))
					path = path.substring(5);
				setText(path);
				setCaretPosition(0);
			}
		}

	}

	private class FileRenderer extends JLabel implements ListCellRenderer {
		private static final long serialVersionUID = 1L;

		public FileRenderer(){
			super();
			setOpaque(true);
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			if (value == null){
				setText("");
				setToolTipText(null);
			}else{
				setText(value.toString());
				setToolTipText(value.toString());
			}

			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			return this;
		}

	}

	private class ImagesListModel extends AbstractListModel {
		private static final long serialVersionUID = 1L;

		private ArrayList<AladinImage> images;
		private ArrayList<Characterization> characterizations;
		private ArrayList<DefaultComboBoxModel> fileHistorics;

		public ImagesListModel(){
			images = new ArrayList<AladinImage>();
			characterizations = new ArrayList<Characterization>();
			fileHistorics = new ArrayList<DefaultComboBoxModel>();
		}

		public synchronized int addImage(final AladinImage img){
			if (img == null || images.contains(img))
				return -1;

			// Add the image:
			images.add(img);
			// Initialize the Characterizations array:
			characterizations.add(null);
			// Initialize each file history:
			DefaultComboBoxModel fileHistory = new DefaultComboBoxModel();
			fileHistorics.add(fileHistory);
			// Generic Charac (generic.map):
			File defaultFile = CharacManager.getGenericCharac(img.getImageUrl());
			if (defaultFile != null && defaultFile.exists())
				updateFileHistory(fileHistory, defaultFile.getAbsolutePath());
			// Default Charac (MAP):
			defaultFile = CharacManager.getDefaultMap(img.getImageUrl());
			if (defaultFile != null && defaultFile.exists())
				updateFileHistory(fileHistory, defaultFile.getAbsolutePath());
			// Default Charac (XML):
			defaultFile = CharacManager.getDefaultXML(img.getImageUrl());
			if (defaultFile != null && defaultFile.exists())
				updateFileHistory(fileHistory, defaultFile.getAbsolutePath());
			// Distant Charac (XML):
			try{
				URL characUrl = CharacManager.getDistantCharac(img.getImageUrl(), img.isAllSky());
				if (characUrl != null)
					updateFileHistory(fileHistory, characUrl.toString());
			}catch(CharacterizationException ex){
				ErrorsLog.getCurrentInstance().addException(ex);
			}
			// The used Charac:
			if (img.getCharacterization() != null){
				if (img.getCharacterization().isFullCharacterization())
					updateFileHistory(fileHistory, img.getCharacterization().getCharacUrl().toString());
				else if (img.getCharacterization().getMappingFile().getMappingFile() != null)
					updateFileHistory(fileHistory, img.getCharacterization().getMappingFile().getMappingFile().getAbsolutePath());
			}

			fireIntervalAdded(img, images.size()-1, images.size()-1);

			return images.size()-1;
		}

		public synchronized int removeImage(final AladinImage img){
			int index = images.indexOf(img);
			if (index >= 0 && index < images.size()){
				images.remove(index);
				characterizations.remove(index);
				fileHistorics.remove(index);
				fireIntervalRemoved(img, index, index);
				return index;
			}
			return -1;
		}

		public void applyAll() {
			for(int i=0; i<images.size(); i++){
				if (characterizations.get(i) != null){
					AladinImage img = images.get(i);
					Characterization charac = saveMapping(characterizations.get(i), images.get(i));
					characterizations.set(i, charac);
					img.setCharacterization(charac);
					img.characterizationUpdated();
					characterizations.set(i, null);
					anyCharacModified = true;
				}
			}
		}

		private Characterization saveMapping(Characterization charac, AladinImage io){
			if (io != null && charac != null && !charac.isFullCharacterization() && !charac.getMappingFile().isSaved()){
				FitsMapping mapping = charac.getMappingFile();
				int response = JOptionPane.showConfirmDialog(CharacEditor.this, "Do you want to save the Characterization file of the image \""+io.getName()+"\" ?\n("+((charac.getMappingFile().getMappingFile()!=null)?charac.getMappingFile().getMappingFile().getAbsolutePath():"")+")", "Save file ?", JOptionPane.YES_NO_OPTION);
				if (response == JOptionPane.YES_OPTION){
					try {
						Characterization newCharac = save(charac, io);
						if (newCharac != null)
							charac = newCharac;
					} catch (Exception e) {
						JOptionPane.showMessageDialog(CharacEditor.this, "Error while saving the Characterization file \""+mapping.getMappingFile()+"\":\n"+e.getMessage(), "Saving error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
			return charac;
		}

		public AladinImage getImage(int index){
			if (index < 0 || index >= images.size())
				return null;
			else
				return images.get(index);
		}

		public DefaultComboBoxModel getFileHistory(int index){
			if (index < 0 || index >= images.size())
				return new DefaultComboBoxModel();
			else
				return fileHistorics.get(index);
		}

		public boolean isModified(int index){
			if (index >= 0 && index < characterizations.size()){
				Characterization charac = characterizations.get(index);
				return charac != null;
			}else
				return false;
		}

		public int getNbModification(){
			int nb = 0;
			for(Characterization c : characterizations)
				if (c != null) nb++;
			return nb;
		}

		public Characterization getCharacterization(int index){
			if (index < 0 || index >= characterizations.size())
				return null;
			else if (characterizations.get(index) != null)
				return characterizations.get(index);
			else
				return images.get(index).getCharacterization();
		}

		public void setCharacterization(Characterization charac, int index){
			if (index < 0 || index >= characterizations.size())
				return;

			characterizations.set(index, charac);
		}

		public HeaderFits getHeaderFits(int index){
			AladinImage io = getImage(index);
			if (io != null)
				return io.getHeaderFits();
			else
				return null;
		}

		public Object getElementAt(int index) {
			if (index < 0 || index >= images.size())
				return null;
			else
				return images.get(index).getName();
		}

		public int getSize() {
			return images.size();
		}
	}

	private class ImageCellRenderer extends JLabel implements ListCellRenderer {
		private static final long serialVersionUID = 1L;

		private final Color GREEN = new Color(0, 128, 0);

		public ImageCellRenderer(){
			super();
			setOpaque(true);
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			boolean correct = true, exists = true;
			ImagesListModel model = (ImagesListModel)list.getModel();

			if (index >= 0 && index < model.getSize()){
				if (model.getCharacterization(index) != null){
					setFont(getFont().deriveFont(Font.BOLD));
					correct = model.getCharacterization(index).isCorrect();
				}else{
					setFont(getFont().deriveFont(Font.PLAIN));
					exists = false;
				}
			}

			setText(value+((model.isModified(index))?"*":""));

			if (isSelected) {
				setBackground(list.getSelectionBackground());
				if (exists)
					setForeground(correct?GREEN:Color.RED);
				else
					setForeground(list.getSelectionForeground());
			} else {
				setBackground(Color.WHITE);
				if (exists)
					setForeground(correct?GREEN:Color.RED);
				else
					setForeground(list.getForeground());
			}

			return this;
		}
	}

}
