package sedplugin.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Frame;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import sedplugin.errors.CharacterizationException;
import sedplugin.errors.SEDException;


/**
 * Simple dialog to print (read-only) a distant Characterization file.
 *
 * @author Gregory Mantelet
 * @version 03/2012
 */
public class CharacFileDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	protected String image;
	protected URL characUrl;

	protected JLabel lblURLField;
	protected JTextField txtURL;
	protected JTextArea txtCharacFile;

	public CharacFileDialog(String imageName, URL characFile) throws SEDException {
		this(null, imageName, characFile);
	}

	public CharacFileDialog(Frame owner,String imageName, URL characFile) throws SEDException {
		super(owner);
		if (characFile == null)
			throw new SEDException("No Characterization file", "CharacFileDialog constructor", "No Characterization file has been given !");

		image = imageName;
		characUrl = characFile;

		lblURLField = new JLabel("URL: ");
		lblURLField.setFont(lblURLField.getFont().deriveFont(Font.BOLD));

		txtURL = new JTextField(characUrl.toString(), 45);
		txtURL.setCaretPosition(0);
		txtURL.setEditable(false);

		JPanel panURL = new JPanel(new BorderLayout());
		panURL.add(lblURLField, BorderLayout.WEST);
		panURL.add(txtURL, BorderLayout.CENTER);

		txtCharacFile = new JTextArea(20,50);
		txtCharacFile.setEditable(false);

		JScrollPane panCharacFile = new JScrollPane(txtCharacFile);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(panURL, BorderLayout.NORTH);
		getContentPane().add(panCharacFile, BorderLayout.CENTER);

		int nbLignes = 0;
		try {
			// Load the Characterization file:
			InputStream toto = characUrl.openStream();
			BufferedReader buf = new BufferedReader(new InputStreamReader(toto));
			txtCharacFile.setText("");
			String ligne = "";
			while((ligne=buf.readLine())!=null){
				txtCharacFile.append(ligne+"\n");
				nbLignes++;
			}
			toto.close();
			buf.close();

			txtCharacFile.setCaretPosition(0);
			setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			setTitle("Characterization file"+((image != null)?(" of "+image):""));
			pack();
			setResizable(true);
			setModal(true);
			setVisible(true);
		} catch (IOException e1) {
			throw new CharacterizationException(imageName, characUrl.toString(), characUrl.toString().endsWith(".xml"), CharacterizationException.READING_ERROR, nbLignes, e1);
		}
	}
}
