package sedplugin.interfaceGraphique;


import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SwingWorker.StateValue;
import javax.swing.filechooser.FileFilter;


import sedplugin.errors.ErrorsLog;
import sedplugin.errors.SEDException;
import sedplugin.graphe.Graph;
import sedplugin.plugin.SEDPlugin;
import sedplugin.sed.ExportTask;

/**
 * A dialog to let the user choose in which format and where exporting a figure/spectrum.
 *
 * @author Gregory Mantelet
 * @version 01/2010
 */
public class ExportDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	protected JFrame parent;
	protected Graph myGraph;

	protected int choosenFormat;
	protected String currentDirectory = null;
	protected String fileName = null;

	protected JPanel panFormat;
	protected JRadioButton optJpeg;
	protected JRadioButton optPng;
	protected JRadioButton optBmp;
	protected JRadioButton optEps;
	protected JRadioButton optXml;
	protected JRadioButton optVOTable;
	protected JRadioButton optTsv;
	protected JRadioButton optCsv;

	protected JPanel panSED;
	protected JComboBox cboSEDs;

	protected JPanel panFile;
	protected JTextField txtPath;
	protected JButton cmdChangeFile;
	protected ChooseActionListener filtersListener;
	protected JFileChooser chooseExportFile;

	protected JPanel panSwitchCommandsProgression;
	protected final static String COMMANDS = "Commands";
	protected final static String PROGRESSION = "Progression";

	protected JPanel panEndCommands;
	protected JButton cmdOK;
	protected JButton cmdAbort;

	protected JPanel panProgression;
	protected JProgressBar progExport;

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public ExportDialog(boolean image){
		// FORMAT CHOICE:
		ActionListener optionListener = new OptionActionListener();
		ButtonGroup optionsGroup = new ButtonGroup();
		/* Figure exportation case */
		if (image){
			// Create a panel to choose format:
			panFormat = new JPanel(new GridLayout(4,1));
			panFormat.setBorder(BorderFactory.createTitledBorder("Which format ?"));
			// EPS (default)
			choosenFormat = ExportTask.EPS*10;
			optEps = new JRadioButton("<html>EPS <i>(.eps)</i></html>");
			optEps.setFont(optEps.getFont().deriveFont(Font.PLAIN));
			optEps.addActionListener(optionListener);
			optEps.addActionListener(new FormatActionListener(ExportTask.EPS*10));
			// JPEG
			optJpeg = new JRadioButton("<html>JPEG <i>(.jpeg, .jpg)</i></html>");
			optJpeg.addActionListener(optionListener);
			optJpeg.addActionListener(new FormatActionListener(ExportTask.IMAGE*10));
			// PNG
			optPng = new JRadioButton("<html>PNG <i>(.png)</i></html>");
			optPng.addActionListener(optionListener);
			optPng.addActionListener(new FormatActionListener(ExportTask.IMAGE*10+1));
			// BMP
			optBmp = new JRadioButton("<html>BMP <i>(.bmp)</i></html>");
			optBmp.addActionListener(optionListener);
			optBmp.addActionListener(new FormatActionListener(ExportTask.IMAGE*10+2));
			// Option group:
			optionsGroup.add(optEps); optEps.setSelected(true); optEps.setFont(optEps.getFont().deriveFont(Font.BOLD));
			optionsGroup.add(optJpeg);
			optionsGroup.add(optPng);
			optionsGroup.add(optBmp);
			// Format panel:
			panFormat.add(optEps);
			panFormat.add(optJpeg);
			panFormat.add(optPng);
			panFormat.add(optBmp);
			/* Spectrum exportation case */
		}else{
			// Create a panel to choose format:
			panFormat = new JPanel(new GridLayout(5,1));
			panFormat.setBorder(BorderFactory.createTitledBorder("Which format ?"));
			// VOTable (default)
			choosenFormat = ExportTask.VOTABLE*10;
			optVOTable = new JRadioButton("<html>VOTable <i>[Spectrum-DM] (.xml, .vot)</i></html>");
			optVOTable.setFont(optVOTable.getFont().deriveFont(Font.PLAIN));
			optVOTable.addActionListener(optionListener);
			optVOTable.addActionListener(new FormatActionListener(ExportTask.VOTABLE*10));
			// XML
			optXml = new JRadioButton("XML");
			optXml.setFont(optXml.getFont().deriveFont(Font.PLAIN));
			optXml.addActionListener(optionListener);
			optXml.addActionListener(new FormatActionListener(ExportTask.XML*10));
			// TSV
			optTsv = new JRadioButton("<html>ASCII <i>[Tab-Separated Values] (.tsv)</i></html>");
			optTsv.setFont(optTsv.getFont().deriveFont(Font.PLAIN));
			optTsv.addActionListener(optionListener);
			optTsv.addActionListener(new FormatActionListener(ExportTask.ASCII*10));
			// CSV
			optCsv = new JRadioButton("<html>ASCII <i>[Comma-Separated Values] (.csv)</i></html>");
			optCsv.setFont(optCsv.getFont().deriveFont(Font.PLAIN));
			optCsv.addActionListener(optionListener);
			optCsv.addActionListener(new FormatActionListener(ExportTask.ASCII*10+1));
			// Option group:
			optionsGroup.add(optVOTable); optVOTable.setSelected(true); optVOTable.setFont(optVOTable.getFont().deriveFont(Font.BOLD));
			optionsGroup.add(optXml);
			optionsGroup.add(optTsv);
			optionsGroup.add(optCsv);
			// Format panel:
			panFormat.add(optVOTable);
			panFormat.add(optXml);
			panFormat.add(optTsv);
			panFormat.add(optCsv);
		}

		// SED CHOICE:
		if (!image){
			cboSEDs = new JComboBox();
			cboSEDs.setEditable(false);

			panSED = new JPanel(new BorderLayout());
			panSED.setBorder(BorderFactory.createTitledBorder("Which SED ?"));
			panSED.add(cboSEDs, BorderLayout.CENTER);
		}

		// FILE CHOICE:
		txtPath = new JTextField("");
		txtPath.addKeyListener(new PathKeyListener());

		cmdChangeFile = new JButton("Change");
		cmdChangeFile.setMnemonic('C');
		cmdChangeFile.addActionListener(filtersListener = new ChooseActionListener());
		cmdChangeFile.setEnabled(false);

		panFile = new JPanel(new BorderLayout());
		panFile.setBorder(BorderFactory.createTitledBorder("File"));
		panFile.add(txtPath, BorderLayout.CENTER);
		panFile.add(cmdChangeFile, BorderLayout.EAST);

		// COMMANDS:
		cmdOK = new JButton("Export");
		cmdOK.addActionListener(new ExportActionListener());
		cmdOK.addMouseMotionListener(new MouseMotionListener(){
			public void mouseDragged(MouseEvent arg0) {;}
			public void mouseMoved(MouseEvent e) {
				if (cmdOK.isEnabled()){
					File f = new File(txtPath.getText());
					currentDirectory = f.getParentFile().getAbsolutePath();

					switch(choosenFormat){
					case ExportTask.EPS*10:
						if (!filtersListener.epsFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".eps");
						break;
					case ExportTask.IMAGE:
						if (!filtersListener.jpegFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".jpeg");
						break;
					case ExportTask.IMAGE*10+1:
						if (!filtersListener.pngFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".png");
						break;
					case ExportTask.IMAGE*10+2:
						if (!filtersListener.bmpFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".bmp");
						break;
					case ExportTask.VOTABLE*10:
						if (!filtersListener.votableFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".xml");
						break;
					case ExportTask.XML*10:
						if (!filtersListener.xmlFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".xml");
						break;
					case ExportTask.ASCII*10:
						if (!filtersListener.tsvFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".tsv");
						break;
					case ExportTask.ASCII*10+1:
						if (!filtersListener.csvFilter.accept(f))
							f = new File(f.getParent(), f.getName()+".csv");
						break;
					}
					fileName = f.getAbsolutePath();

					if (!txtPath.getText().trim().equals(fileName))
						txtPath.setText((fileName != null)?fileName:"");
				}
			}
		});
		cmdOK.setEnabled(false);

		cmdAbort = new JButton("Cancel");
		cmdAbort.addActionListener(new CancelActionListener());

		panEndCommands = new JPanel();
		panEndCommands.add(cmdOK);
		panEndCommands.add(cmdAbort);

		// PROGRESSION BAR:
		progExport = new JProgressBar(0, 0, 100);
		progExport.setStringPainted(true);

		panProgression = new JPanel(new BorderLayout());
		panProgression.add(progExport, BorderLayout.CENTER);

		// SWITCH BETWEEN COMMANDS AND PROGRESSION:
		panSwitchCommandsProgression = new JPanel(new CardLayout());
		panSwitchCommandsProgression.add(panEndCommands, COMMANDS);
		panSwitchCommandsProgression.add(panProgression, PROGRESSION);

		// GENERAL:
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		if (!image) getContentPane().add(panSED);
		getContentPane().add(panFormat);
		getContentPane().add(panFile);
		getContentPane().add(panSwitchCommandsProgression);

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowActivated(WindowEvent e) {
				super.windowActivated(e);
				panSwitchCommandsProgression.repaint();
			}
		});
		pack();
		URL iconTitle = getClass().getResource(SEDPlugin.ICONS_DIR + "export.gif");
		if (iconTitle != null)
			setIconImage(new ImageIcon(iconTitle).getImage());
		setSize(new Dimension(400, getHeight()));
		setResizable(false);
		setTitle("Export "+(image?"figure":"spectrum"));
		setModal(true);
		setVisible(false);
	}

	/* ************ */
	/* SHOW METHODS */
	/* ************ */
	public final void show(JFrame fra, Graph g){
		parent = fra;
		myGraph = g;

		// Show export commands:
		CardLayout cl = (CardLayout)panSwitchCommandsProgression.getLayout();
		cl.show(panSwitchCommandsProgression, COMMANDS);
		cmdOK.setEnabled(false);
		progExport.setValue(0); progExport.setString("");

		activateDialog(true);

		// Create a new JFileChooser:
		chooseExportFile = new JFileChooser();
		chooseExportFile.setFileSelectionMode(JFileChooser.FILES_ONLY);
		chooseExportFile.setAcceptAllFileFilterUsed(false);
		chooseExportFile.setApproveButtonText("OK");

		// Set the current Directory:
		if (currentDirectory == null)
			currentDirectory = chooseExportFile.getCurrentDirectory().getAbsolutePath();
		else{
			File f = new File(currentDirectory);
			if (f.exists())
				chooseExportFile.setCurrentDirectory(f);
			else
				currentDirectory = chooseExportFile.getCurrentDirectory().getAbsolutePath();
		}

		fileName = null;
		txtPath.setText(currentDirectory+File.separatorChar);

		// Finaly...display the dialog:
		setLocationRelativeTo(parent);
		setVisible(true);
	}

	private void activateDialog(boolean activate){
		cmdAbort.setEnabled(activate);

		// Re-activate format panel:
		panFormat.setEnabled(activate);
		if (optEps != null){
			optEps.setEnabled(activate);
			optJpeg.setEnabled(activate);
			optPng.setEnabled(activate);
			optBmp.setEnabled(activate);
		}else{
			optVOTable.setEnabled(activate);
			optXml.setEnabled(activate);
			optTsv.setEnabled(activate);
			optCsv.setEnabled(activate);
		}

		// Re-activate file panel:
		panFile.setEnabled(activate);
		txtPath.setEnabled(activate);
		cmdChangeFile.setEnabled(activate);

		// Get the list of selected SEDs:
		if (cboSEDs != null)
			cboSEDs.setModel(new DefaultComboBoxModel(myGraph.getAllSeries()));
	}

	/* ************* */
	/* FILE MANAGING */
	/* ************* */
	private boolean isValidFile(String filePath){
		if (filePath != null)
			return isValidFile(new File(filePath.trim()));
		else
			return false;
	}

	private boolean isValidFile(File f){
		if (f == null || (f != null && f.getParentFile() == null))
			return false;
		else
			return (!f.isDirectory() && (!f.exists() || (f.exists() && f.isFile() && f.canWrite())) && f.getParentFile().isDirectory() && f.getParentFile().exists() && f.getParentFile().canWrite());
	}

	/* ********* */
	/* LISTENERS */
	/* ********* */
	private final class ChooseActionListener implements ActionListener {
		public ExportFileFilter jpegFilter;
		public ExportFileFilter pngFilter;
		public ExportFileFilter bmpFilter;
		public ExportFileFilter epsFilter;
		public ExportFileFilter xmlFilter;
		public ExportFileFilter votableFilter;
		public ExportFileFilter tsvFilter;
		public ExportFileFilter csvFilter;

		public ChooseActionListener(){
			jpegFilter = new ExportFileFilter(new String[]{"jpeg","jpg"}, "JPEG Images (.jpeg,.jpg)");
			pngFilter = new ExportFileFilter("png", "PNG Images (.png)");
			bmpFilter = new ExportFileFilter("bmp", "Windows BMP Images (.bmp)");
			epsFilter = new ExportFileFilter("eps", "Encapsulated PostScript Images (.eps)");
			xmlFilter = new ExportFileFilter("xml", "XML Files (.xml)");
			votableFilter = new ExportFileFilter(new String[]{"xml","vot"}, "VOTable (.xml,.vot)");
			tsvFilter = new ExportFileFilter("tsv", "ASCII-Tab-Separated Values (.tsv)");
			csvFilter = new ExportFileFilter("csv", "ASCII-Comma-Separated Values (.csv)");
		}

		public void actionPerformed(ActionEvent e) {
			// Set adapted file filters:
			chooseExportFile.setDialogTitle("Export...");
			switch(choosenFormat){
			case ExportTask.EPS*10:
				chooseExportFile.addChoosableFileFilter(epsFilter);
				chooseExportFile.setDialogTitle("Export as EPS...");
				break;
			case ExportTask.IMAGE:
				chooseExportFile.addChoosableFileFilter(jpegFilter);
				chooseExportFile.setDialogTitle("Export as JPEG...");
				break;
			case ExportTask.IMAGE*10+1:
				chooseExportFile.addChoosableFileFilter(pngFilter);
				chooseExportFile.setDialogTitle("Export as PNG...");
				break;
			case ExportTask.IMAGE*10+2:
				chooseExportFile.addChoosableFileFilter(bmpFilter);
				chooseExportFile.setDialogTitle("Export as BMP...");
				break;
			case ExportTask.VOTABLE*10:
				chooseExportFile.addChoosableFileFilter(votableFilter);
				chooseExportFile.setDialogTitle("Export as VOTable...");
				break;
			case ExportTask.XML*10:
				chooseExportFile.addChoosableFileFilter(xmlFilter);
				chooseExportFile.setDialogTitle("Export as XML...");
				break;
			case ExportTask.ASCII*10:
				chooseExportFile.setAcceptAllFileFilterUsed(true);
				chooseExportFile.addChoosableFileFilter(tsvFilter);
				chooseExportFile.setDialogTitle("Export as ASCII...");
				break;
			case ExportTask.ASCII*10+1:
				chooseExportFile.setAcceptAllFileFilterUsed(true);
				chooseExportFile.addChoosableFileFilter(csvFilter);
				chooseExportFile.setDialogTitle("Export as ASCII...");
				break;
			default:
				chooseExportFile.setAcceptAllFileFilterUsed(true);
				chooseExportFile.setDialogTitle("Export as ?...");
			}

			// Display the export dialog and retrieve user choice:
			boolean choose = true;
			while (choose){
				try{
					if (txtPath.getText() != null && txtPath.getText().trim().length() > 0){
						File f = new File(txtPath.getText().trim());
						if (f.isDirectory()){
							chooseExportFile.setCurrentDirectory(f);
							chooseExportFile.setSelectedFile(null);
						}else if (f.getParentFile().exists()){
							chooseExportFile.setCurrentDirectory(f.getParentFile());
							chooseExportFile.setSelectedFile(new File(f.getName()));
						}
					}
					int decision = chooseExportFile.showSaveDialog(parent);
					if (decision == JFileChooser.APPROVE_OPTION){
						File f = chooseExportFile.getSelectedFile();
						if (!f.isDirectory()){
							FileFilter ff = chooseExportFile.getFileFilter();
							if (ff instanceof ExportFileFilter){
								String selectedExtension = ((ExportFileFilter)ff).getExtension();
								// update image file name:
								String extractedExtension = ExportFileFilter.getExtension(f);
								if (extractedExtension == null || !extractedExtension.equalsIgnoreCase(selectedExtension))
									f = new File(f.getParent(), f.getName()+"."+selectedExtension);
							}
							txtPath.setText(f.getAbsolutePath());
							cmdOK.setEnabled(true);

						}else
							throw new SEDException("Invalid selected file", "Choice of the export file", "The selected element (\""+f.getAbsoluteFile()+"\") is not a valid file.");
					}
					switch(choosenFormat){
					case ExportTask.EPS*10:
						chooseExportFile.removeChoosableFileFilter(epsFilter);
						break;
					case ExportTask.IMAGE:
						chooseExportFile.removeChoosableFileFilter(jpegFilter);
						break;
					case ExportTask.IMAGE*10+1:
						chooseExportFile.removeChoosableFileFilter(pngFilter);
						break;
					case ExportTask.IMAGE*10+2:
						chooseExportFile.removeChoosableFileFilter(bmpFilter);
						break;
					case ExportTask.VOTABLE*10:
						chooseExportFile.removeChoosableFileFilter(votableFilter);
						break;
					case ExportTask.XML*10:
						chooseExportFile.removeChoosableFileFilter(xmlFilter);
						break;
					case ExportTask.ASCII*10:
						chooseExportFile.removeChoosableFileFilter(tsvFilter);
					case ExportTask.ASCII*10+1:
						chooseExportFile.removeChoosableFileFilter(csvFilter);
					}
					chooseExportFile.setAcceptAllFileFilterUsed(false);
					choose = false;
				}catch(SEDException ex){
					int response = JOptionPane.showOptionDialog(ExportDialog.this,
							ex.getMessage()+"\nWould you try with another file ?",
							ex.getTitle(),
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE,
							null,
							null,
							null);

					choose = (response == JOptionPane.YES_OPTION);
				}
			}
		}
	}

	private final class ExportActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (fileName != null){
				try{
					File f = new File(fileName);

					if (isValidFile(f)){
						ExportTask et;
						if (cboSEDs != null)
							et = new ExportTask(myGraph, cboSEDs.getSelectedIndex(), choosenFormat/10, f);
						else
							et = new ExportTask(myGraph, choosenFormat/10, f);
						CardLayout cl = (CardLayout)panSwitchCommandsProgression.getLayout();
						cl.show(panSwitchCommandsProgression, PROGRESSION);
						activateDialog(false);
						et.addPropertyChangeListener(new PropertyChangeListener(){
							public void propertyChange(PropertyChangeEvent evt) {
								String property = evt.getPropertyName();

								if (property.equals("state")){
									switch((StateValue)evt.getNewValue()){
									case STARTED:
										ExportDialog.this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
										progExport.setValue(0);
										progExport.setString("");
										break;
									case DONE:
										ExportDialog.this.setCursor(Cursor.getDefaultCursor());
										ExportTask task = (ExportTask)evt.getSource();
										if (task.isCancelled()){
											progExport.setString("Cancelled");
											if (task.getEndException() != null)
												ErrorsLog.getCurrentInstance().addException(task.getEndException());
										}else{
											progExport.setValue(100);
											progExport.setString("Success !");
											ExportDialog.this.setVisible(false);
										}
										break;
									}
								}else if (property.equals("progress")){
									progExport.setValue(((Integer)evt.getNewValue()).intValue());
									progExport.setString(((ExportTask)evt.getSource()).getProgressionMessage());
								}
							}
						});
						et.execute();
					}
				}catch(SEDException ex){
					ErrorsLog.getCurrentInstance().addException(ex);
				}
			}
		}
	}

	private final class CancelActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			setVisible(false);
		}
	}

	private final class FormatActionListener implements ActionListener {
		protected final int format;
		public FormatActionListener(int f){
			format = f;
		}
		public void actionPerformed(ActionEvent e) {
			if (format != choosenFormat && isValidFile(txtPath.getText())){
				String path = txtPath.getText().trim();
				int dotPos = path.lastIndexOf(".");
				File f = new File(path.substring(0, (dotPos==-1)?path.length():dotPos));
				switch(format){
				case ExportTask.EPS*10:
					if (!filtersListener.epsFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".eps");
					break;
				case ExportTask.IMAGE:
					if (!filtersListener.jpegFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".jpeg");
					break;
				case ExportTask.IMAGE*10+1:
					if (!filtersListener.pngFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".png");
					break;
				case ExportTask.IMAGE*10+2:
					if (!filtersListener.bmpFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".bmp");
					break;
				case ExportTask.VOTABLE*10:
					if (!filtersListener.votableFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".xml");
					break;
				case ExportTask.XML*10:
					if (!filtersListener.xmlFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".xml");
					break;
				case ExportTask.ASCII*10:
					if (!filtersListener.tsvFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".tsv");
					break;
				case ExportTask.ASCII*10+1:
					if (!filtersListener.csvFilter.accept(f))
						f = new File(f.getParent(), f.getName()+".csv");
					break;
				}
				txtPath.setText(f.getAbsolutePath());
			}
			choosenFormat = format;
		}
	}

	private final class OptionActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JRadioButton optSel = (JRadioButton)e.getSource();
			if (optEps != null) optEps.setFont(optEps.getFont().deriveFont(Font.PLAIN));
			if (optJpeg != null) optJpeg.setFont(optJpeg.getFont().deriveFont(Font.PLAIN));
			if (optPng != null) optPng.setFont(optPng.getFont().deriveFont(Font.PLAIN));
			if (optBmp != null) optBmp.setFont(optBmp.getFont().deriveFont(Font.PLAIN));
			if (optVOTable != null) optVOTable.setFont(optVOTable.getFont().deriveFont(Font.PLAIN));
			if (optXml != null) optXml.setFont(optXml.getFont().deriveFont(Font.PLAIN));
			if (optTsv != null) optTsv.setFont(optTsv.getFont().deriveFont(Font.PLAIN));
			if (optCsv != null) optCsv.setFont(optCsv.getFont().deriveFont(Font.PLAIN));
			optSel.setFont(optSel.getFont().deriveFont(Font.BOLD));
		}
	}

	private final class PathKeyListener implements KeyListener {
		public void keyReleased(KeyEvent e) {
			cmdOK.setEnabled(isValidFile(txtPath.getText()));
		}

		public void keyPressed(KeyEvent arg0) {;}
		public void keyTyped(KeyEvent arg0) {;}

	}

	/* **** */
	/* MAIN */
	/* **** */
	public final static void main(String[] args){
		ExportDialog ed = new ExportDialog(true);
		ed.show(null, null);
		ed.dispose();
	}

}
