package sedplugin.interfaceGraphique;

import java.util.Vector;

import sedplugin.graphe.Serie;



/**
 * Interface between a Graph object and the JFrame which contains it.
 * 
 * @see sedplugin.graphe.Graph
 * @see sedplugin.interfaceGraphique.SEDFrame
 * 
 * @author Gregory Mantelet
 * @version 04/2012
 */
public interface IGraphManager {

	/**
	 * Allows to warn the frame that mouse position has changed.
	 * 
	 * @param posX		Abscissa of the new mouse position.
	 * @param posY		Ordinate of the new mouse position.
	 * @param xValue	Wavelength value associated with the new mouse abscissa.
	 * @param yValue	Flux value associated with the new mouse ordinate.
	 */
	public void changeMousePosition(double posX, double posY, double xValue, double yValue);

	/**
	 * Allows the Graphical Interface to draw given series.
	 * 
	 * @param series		Series to draw.
	 * @param showYConfig	<i>true</i> to display all ordinate configuration parameter because the Y unit is known for all displayed SEDs, <i>false</i> otherwise.
	 */
	public void drawingSeries(Vector<Serie> series, boolean showYConfig);

	/**
	 * Tells to the JFrame that it must execute the action (if any) corresponding to
	 * the selection of the Aladin image associated with the given spectrum point.
	 * 
	 * @param s		The graphical representation of the drawn SED whose the point is selected.
	 * @param pointIndex	The selected spectrum point.
	 */
	public void highlightSeriePoint(Serie s, int pointIndex);

	public String getWelcomeMessage();
}
