package sedplugin.interfaceGraphique;


import java.beans.PropertyChangeListener;


import sedplugin.graphe.Serie;
import sedplugin.plugin.AladinImage;
import sedplugin.sed.SED;

/**
 * Interface between the SEDPlugin and its graphical interface.
 *
 * @see sedplugin.plugin.SEDPlugin
 * @see sedplugin.interfaceGraphique.SEDFrame
 *
 * @author Gregory Mantelet
 * @version 04/2012
 */
public interface ISEDPluginGI {

	/**
	 * Close/Dispose the Graphical Interface.
	 */
	public void close();

	/**
	 * Activate/Deactivate the plugin so that if there is no more spectrum
	 * then no command can be done on spectrums list or graph.
	 *
	 * @param enabled	<i>true</i> activate the plugin, <i>false</i> deactivate it.
	 */
	public void activatePlugin(boolean enabled);

	/**
	 * Changes visibility of the Graphical Interface.
	 *
	 * @param visible	<i>true</i> the GI must be visible, <i>false</i> else.
	 */
	public void setVisible(boolean visible);

	public void addSerie(final SED sed);

	public void removeSerie(final SED sed);

	public Serie getSerie(final SED sed);

	public void addImage(final AladinImage img);

	public void removeImage(final AladinImage img);

	public PropertyChangeListener getAladinProgressListener();

	public void enableAladinSync(final boolean enable);
}
