package sedplugin.interfaceGraphique;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.ItemSelectable;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

import sedplugin.characterization.FitsMapping;
import sedplugin.characterization.HeaderFits;
import sedplugin.characterization.HeaderFits.HeaderItem;
import sedplugin.plugin.SEDPlugin;


public final class JCharacField extends JPanel implements ItemSelectable {
	private static final long serialVersionUID = 1L;

	private ImageIcon icoError = new ImageIcon(getClass().getResource(SEDPlugin.ICONS_DIR+"tinyError.gif"));

	private HeaderFits header = null;
	public static final int ALL_KEYWORDS = 4;
	private int fitsKeywordType = ALL_KEYWORDS;

	private String example;
	private final float defaultFontSize;

	public final static String FITS_KEYWORD_ALIAS = "fits:";

	private JLabel lblName;
	private JLabel lblIco;
	private JComboBox cboValue;
	private JLabel lblValue;

	public JCharacField(String name, String utype, int keywordType){
		this(name, utype, -1, -1, keywordType);
	}

	public JCharacField(String name, String utype, int keywordType, int valueWidth){
		this(name, utype, -1, keywordType, valueWidth);
	}

	public JCharacField(String name, String utype, int keywordType, int nameWidth, int valueWidth) {
		super();

		fitsKeywordType = keywordType;
		example = null;

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Label - Field name:
		lblName = new JLabel(name+": ");
		lblName.setAlignmentX(RIGHT_ALIGNMENT);
		if (nameWidth > 0){
			lblName.setPreferredSize(new Dimension(nameWidth, lblName.getPreferredSize().height));
			lblName.setMaximumSize(new Dimension(nameWidth, lblName.getMaximumSize().height));
			lblName.setMinimumSize(new Dimension(0, 0));
		}
		lblName.setFont(lblName.getFont().deriveFont(Font.BOLD));
		lblName.setHorizontalAlignment(JLabel.LEFT);
		lblName.setToolTipText("<html><b>UType: </b>"+utype+"</html>");
		add(lblName);

		// Label - Icon (error, OK, etc...)
		lblIco = new JLabel();
		lblIco.setAlignmentX(RIGHT_ALIGNMENT);
		lblIco.setPreferredSize(new Dimension(20, 20));
		add(lblIco);

		// Label - Value:
		lblValue = new JLabel("?");
		lblValue.setAlignmentX(RIGHT_ALIGNMENT);
		if (valueWidth > 0){
			lblValue.setPreferredSize(new Dimension(valueWidth, lblValue.getPreferredSize().height));
			lblValue.setMaximumSize(new Dimension(valueWidth, lblValue.getMaximumSize().height));
			lblValue.setMinimumSize(new Dimension(0, 0));
		}
		lblValue.setHorizontalAlignment(SwingConstants.CENTER);
		lblValue.setFont(lblValue.getFont().deriveFont(Font.ITALIC));
		defaultFontSize = lblValue.getFont().getSize2D();

		// ComboBox - Value:
		cboValue = new JComboBox();
		cboValue.setAlignmentX(RIGHT_ALIGNMENT);
		cboValue.setPreferredSize(new Dimension((valueWidth<150)?150:valueWidth, cboValue.getPreferredSize().height));
		cboValue.setMinimumSize(new Dimension((valueWidth<=0)?100:valueWidth, cboValue.getMinimumSize().height));
		cboValue.setEditable(true);
		cboValue.setEditor(new KeywordEditor());
		cboValue.setRenderer(new KeywordCellRenderer());
		add(cboValue);
		add(lblValue);
	}

	public int getNameWidth(){
		return lblName.getPreferredSize().width;
	}

	@Override
	public void setEnabled(boolean enabled) {
		lblName.setEnabled(enabled);
		lblIco.setEnabled(enabled);
		cboValue.setEnabled(enabled);
		lblValue.setEnabled(enabled);
		super.setEnabled(enabled);
	}

	public int getSelectedIndex(){
		return cboValue.getSelectedIndex();
	}

	public Object getSelectedItem(){
		return cboValue.getSelectedItem();
	}

	public void setSelectedIndex(int index){
		cboValue.setSelectedIndex(index);
	}

	public void setSelectedItem(Object item){
		cboValue.setSelectedItem(item);
	}

	public void setPopupVisible(boolean visible){
		cboValue.setPopupVisible(visible);
	}

	public HeaderFits getHeaderFits(){
		return header;
	}

	public void setHeaderFits(HeaderFits headerFits, Vector<?> others){
		if (others == null){
			setHeaderFits(headerFits);
			return;
		}

		Vector<Object> allItems = new Vector<Object>(others);
		header = headerFits;

		if (header != null){
			if (fitsKeywordType >= ALL_KEYWORDS || fitsKeywordType < HeaderFits.UNDEFINED)
				allItems.addAll(header.getAllKeywords());
			else
				allItems.addAll(header.getAllKeywords(fitsKeywordType));
		}

		cboValue.setModel(new DefaultComboBoxModel(allItems));
	}

	public void setHeaderFits(HeaderFits headerFits){
		header = headerFits;

		if (header == null)
			cboValue.setModel(new DefaultComboBoxModel());
		else if (fitsKeywordType >= ALL_KEYWORDS || fitsKeywordType < HeaderFits.UNDEFINED)
			cboValue.setModel(new DefaultComboBoxModel(header.getAllKeywords()));
		else
			cboValue.setModel(new DefaultComboBoxModel(header.getAllKeywords(fitsKeywordType)));
	}

	public void setError(String errorMsg){
		if (errorMsg == null){
			lblName.setForeground(null);
			lblIco.setIcon(null);
			lblIco.setToolTipText(null);
			((KeywordEditor)cboValue.getEditor()).setBackground(null);
			cboValue.setBackground(null);
			cboValue.setBorder(null);
			lblValue.setForeground(null);
		}else{
			lblName.setForeground(Color.RED);
			lblIco.setIcon(icoError);
			lblIco.setToolTipText(errorMsg);
			((KeywordEditor)cboValue.getEditor()).setBackground(new Color(241,184,168));
			cboValue.setBackground(new Color(241,184,168));
			cboValue.setBorder(BorderFactory.createLineBorder(Color.RED));
			lblValue.setForeground(Color.RED);
		}
	}

	public String getExample(){
		return example;
	}

	public void setExample(String example){
		if (example != null){
			example = example.trim();
			if (example.length() == 0)
				example = null;
		}
		this.example = example;
	}


	public void addItemListener(ItemListener l) {
		cboValue.addItemListener(l);
	}

	public Object[] getSelectedObjects() {
		return cboValue.getSelectedObjects();
	}

	public void removeItemListener(ItemListener l) {
		cboValue.removeItemListener(l);
	}

	public final static String toAliasedKeyword(String fitsKeyword){
		if (FitsMapping.isFitsKeyword(fitsKeyword))
			return FITS_KEYWORD_ALIAS+fitsKeyword.substring(FitsMapping.FITS_KEYWORD_PREFIX.length());
		else
			return fitsKeyword;
	}

	public final static String toFitsKeyword(String aliasedKeyword){
		if (aliasedKeyword.startsWith(FITS_KEYWORD_ALIAS))
			return FitsMapping.FITS_KEYWORD_PREFIX+aliasedKeyword.substring(FITS_KEYWORD_ALIAS.length());
		else
			return aliasedKeyword;
	}

	protected class KeywordEditor extends JTextField implements ComboBoxEditor {
		private static final long serialVersionUID = 1L;

		public Component getEditorComponent() {
			return this;
		}

		public Object getItem() {
			return getText();
		}

		public void setItem(Object anObject) {
			if (anObject != null){
				lblValue.setFont(lblValue.getFont().deriveFont(defaultFontSize));
				if (anObject instanceof HeaderItem){
					HeaderItem item = (HeaderItem)anObject;
					this.setText(toAliasedKeyword(FitsMapping.getFitsKeyword(item.keyword)));
					lblValue.setText(item.value);
					this.setToolTipText("(="+item.value+") "+item.comment);
				}else{
					String value = anObject.toString().trim();
					this.setText(value);
					if (value.length() == 0){
						lblValue.setFont(lblValue.getFont().deriveFont(8));
						lblValue.setText((example != null)?(" (e.g. "+example+")"):"");
					}else if (value.startsWith(FITS_KEYWORD_ALIAS)){
						value = value.substring(FITS_KEYWORD_ALIAS.length());
						if (header != null){
							HeaderItem item = header.getKeywordItem(value);
							if (item != null){
								setItem(item);
								return;
							}
						}
						lblValue.setFont(lblValue.getFont().deriveFont(8));
						lblValue.setText("???"+((example != null)?(" (e.g. "+example+")"):""));
					}else
						lblValue.setText(value);

					this.setToolTipText(null);
				}
			}else{
				this.setText(null);
				this.setToolTipText(null);
				lblValue.setFont(lblValue.getFont().deriveFont(8));
				lblValue.setText((example != null)?("e.g. "+example):"");
			}
		}
	}

	protected class KeywordCellRenderer extends JLabel implements ListCellRenderer {
		private static final long serialVersionUID = 1L;

		public KeywordCellRenderer(){
			super();
			setOpaque(true);
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			setText(value.toString());
			setToolTipText(null);

			if (value instanceof HeaderItem){
				HeaderItem item = (HeaderItem)value;
				if (item != null){
					setText(FITS_KEYWORD_ALIAS+item.keyword);
					setToolTipText("(="+item.value+") "+item.comment);
				}
			}

			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(Color.WHITE);
				setForeground(list.getForeground());
			}

			return this;
		}
	}

	public static void main(String[] args){
		JFrame fra = new JFrame("Test JCharacField");
		fra.getContentPane().setLayout(new GridLayout(1,1,0,0));
		final JCharacField fldWlCalib = new JCharacField("Spectral calibration", FitsMapping.SPECTRAL_CALIBRATION_UTYPE, 100);
		fldWlCalib.setExample("AnExample");
		fldWlCalib.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				String value = e.getItem().toString();
				if (e.getStateChange() == ItemEvent.SELECTED){
					if (value.trim().length() > 0)
						fldWlCalib.setError(null);
					else
						fldWlCalib.setError("No value !");
				}

			}
		});
		fra.getContentPane().add(fldWlCalib);
		fra.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		fra.pack();


		fldWlCalib.setError("My first error message !");
		HeaderItem item = new HeaderItem("MACHIN", "'truc'", "Un test !");
		Vector<Object> v = new Vector<Object>(1);
		v.add(item);
		fldWlCalib.setHeaderFits(null, v);

		fra.setVisible(true);
	}

}
