package sedplugin.interfaceGraphique;


import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.MediaTracker;
import java.awt.RenderingHints;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.awt.image.BufferedImage;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.io.File;

import java.net.URL;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.Timer;
import javax.swing.WindowConstants;

import javax.swing.SwingWorker.StateValue;

import javax.swing.border.EtchedBorder;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;






import sedplugin.errors.ErrorsLog;
import sedplugin.errors.ErrorsNotifier;
import sedplugin.errors.SEDException;
import sedplugin.graphe.AGraphFrame;
import sedplugin.graphe.CirclePoint;
import sedplugin.graphe.CrossPoint;
import sedplugin.graphe.DefaultPoint;
import sedplugin.graphe.FrameOfReference;
import sedplugin.graphe.Graph;
import sedplugin.graphe.Point;
import sedplugin.graphe.Serie;
import sedplugin.graphe.SquarePoint;
import sedplugin.graphe.StarPoint;
import sedplugin.plugin.AladinImage;
import sedplugin.plugin.AladinProxy;
import sedplugin.plugin.MySwingWorker;
import sedplugin.plugin.SEDPlugin;
import sedplugin.plugin.UserPreferences;
import sedplugin.sed.ExportTask;
import sedplugin.sed.SED;
import sedplugin.sed.SEDPoint;
import sedplugin.sed.source.SEDSource;
import sedplugin.sed.source.aladin.Circle;
import sedplugin.sed.source.aladin.PhotometrySource;
import sedplugin.sed.source.aladin.Polygon;
import sedplugin.sed.source.operation.Operation;
import sedplugin.sed.unit.AxisUnit;
import sedplugin.sed.unit.PhysicalQuantity;
import sedplugin.sed.unit.UnitDef;

/**
 * Main frame of the SEDPlugin.<BR />
 * If must implement {@link ISEDPluginGI}.
 *
 * @author Gregory Mantelet (CDS)
 * @version 04/2012
 */
public class SEDFrame extends JFrame implements IGraphManager, ISEDPluginGI, ErrorsNotifier, WindowListener {
	// ATTRIBUTS:
	private static final long serialVersionUID = 1L;

	// MENU BAR:
	protected JMenuBar menuBar;
	protected JMenu mnuPlugin;
	/*protected JMenuItem mnuPlugin_Open;
	protected JMenuItem mnuPlugin_OpenUrl;*/
	protected JFileChooser openDialog;
	/*protected JMenuItem mnuPlugin_CatalogExplorer;*/
	protected JMenuItem mnuPlugin_Quit;
	protected JMenuItem mnuPlugin_Preferences;
	protected JMenu mnuSpectrum;
	protected JMenuItem mnuSpectrum_Rename;
	protected JMenuItem mnuSpectrum_Refresh;
	protected JMenuItem mnuSpectrum_Delete;
	protected JCheckBoxMenuItem mnuSpectrum_Lock;
	protected JMenuItem mnuSpectrum_Export;
	protected JMenu mnuSpectrum_SendTo;
	protected JMenu mnuTools;
	protected JMenuItem mnuTools_Calculator;
	protected JMenuItem mnuTools_Subtractor;
	protected JMenuItem mnuCharac;
	protected JMenu mnuHelp;
	protected JMenuItem mnuHelp_About;

	// TOOLS PANEL:
	protected JPanel panCards;
	protected JLabel lblCardIcon;
	protected JPanel cards;
	protected final static String UNITS = "Units";
	protected final static String GRAPH_PREFERENCES = "Graph edition";
	protected static ImageIcon icoUnits = null;
	protected static ImageIcon icoGraphPreferences = null;

	// UNITS PANEL:
	public final static String ADD_NEW_UNIT = "Others...";
	protected JPanel cardUnits;
	protected JLabel lblAbscissa;
	protected JLabel lblOrdinate;
	protected JComboBox cboAbscissaKind;
	protected JComboBox cboAbscissaUnits;
	protected JComboBox cboOrdinateKind;
	protected JComboBox cboOrdinateUnits;
	protected JCheckBox chkAbscissaLog;
	protected JCheckBox chkOrdinateLog;
	protected JPanel panOrdinate;

	// SERIE CONFIGURATION:
	protected JPanel cardGraphPreferences;
	protected JTextField txtGraphTitle;
	protected JComboBox cboBackground;
	protected JComboBox cboForeground;
	protected JComboBox cboSeries;
	protected JToggleButton cmdConnectPoints;
	protected JComboBox cboPointStyles;
	protected JComboBox cboColor;

	// SPECTRUM LIST:
	protected JPanel panSpectrumList;
	protected JToolBar toolBarList;
	protected static ImageIcon icoLock = null, icoUnlock = null;
	protected JToggleButton cmdLock;
	protected JToggleButton cmdInfo;
	protected JButton cmdReload;
	protected JButton cmdDelete;
	protected JButton cmdSave;
	protected JButton cmdSendTo;
	protected SEDJList lstSEDs;
	protected SEDListModel series;

	// META-DATA:
	protected JTabbedPane panMetadata;
	// Spectrum meta-data:
	protected JLabel lblNameField;
	protected JLabel lblName;
	protected JButton cmdRename;
	protected JLabel lblTypeField;
	protected JLabel lblType;
	protected JLabel lblRaField;
	protected JLabel lblRa;
	protected JLabel lblDecField;
	protected JLabel lblDec;
	protected JLabel lblRadiusField;
	protected JLabel lblRadius;
	//protected JButton cmdSelectMeasure;
	// Point meta-data:
	protected int selectedPoint;
	protected JLabel lblPointNumberField;
	protected JSpinner spiPointNumber;
	protected JLabel lblNbPoints;
	protected JLabel lblWavelengthField;
	protected JLabel lblWavelength;
	protected JLabel lblFluxField;
	protected JLabel lblFlux;
	protected JLabel lblImageField;
	protected JLabel lblImage;
	//protected JButton cmdSelectImage;
	protected JButton cmdViewCharacFile;

	// GRAPH:
	protected JPanel panGraph;
	protected final Graph myGraph;

	// GRAPH TOOLBAR:
	protected JToolBar graphToolBar;
	protected JToggleButton cmdUnits;
	protected JToggleButton cmdGraphPreferences;
	protected JToggleButton cmdGrid;
	protected JToggleButton cmdLegend;
	protected JToggleButton cmdPositionCursor;
	protected JButton cmdExport;
	protected ActionListener exportActionListener;

	// POSITION:
	protected JPanel panPosition;
	protected JLabel lblXPosition;
	protected JLabel lblYPosition;

	// CALCULATOR DIALOG:
	protected CalculatorDialog myCalculator;

	// EXPORT DIALOGS:
	protected ExportDialog myExportFigureDialog;
	protected ExportDialog myExportSpectrumDialog;

	// PLUGIN TOOL BAR:
	protected JToolBar pluginToolBar;
	protected JCheckBox chkInteracts;
	protected static ImageIcon icoConnect = null;
	protected static ImageIcon icoDisconnect = null;
	protected JButton cmdErrors;
	protected static ImageIcon icoSynchronize = null;
	protected static ImageIcon icoStop = null;
	protected JButton cmdRefreshAll;
	protected JProgressBar progress;

	// DISPLAY ERROR NOTIFICATION:
	protected NotificationLabel lblNotification;

	// NO CHARAC DIALOG:
	protected CharacEditor noCharacDialog;

	// ABOUT DIALOG:
	protected AboutDialog about;

	// PLUGIN:
	protected final SEDPlugin myPlugin;

	// ALADIN PROXY:
	protected final AladinProxy proxy;

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public SEDFrame(SEDPlugin plugin, AGraphFrame g) {
		myPlugin = plugin;
		proxy = myPlugin.getAladinProxy();

		getContentPane().setLayout(new BorderLayout());

		// About dialog:
		about = new AboutDialog(this);

		// Initialize the other dialogs:
		myExportFigureDialog = null;
		myExportSpectrumDialog = null;
		noCharacDialog = new CharacEditor(this, myPlugin.getXAxis(), myPlugin.getYAxis());

		// Build the menu bar:
		buildMenuBar();

		// Build plugin tool bar:
		buildPluginToolBar();

		myGraph = new Graph(this, g);

		// Build the spectrums list:
		buildSpectrumList();

		// Build the graph panel:
		buildGraphPanel();

		// Building of the SplitPane:
		JSplitPane splGraph = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panSpectrumList, panGraph);
		splGraph.setOpaque(true);
		splGraph.setOneTouchExpandable(true);
		splGraph.setDividerLocation(200);
		splGraph.setDividerSize(8);
		getContentPane().add(splGraph, BorderLayout.CENTER);

		// End of building the JFrame:
		setTitle("S.E.D. Plugin - v2.0 Beta");
		URL iconTitle = getClass().getResource(SEDPlugin.ICONS_DIR + "sed.gif");
		if (iconTitle != null)
			setIconImage(new ImageIcon(iconTitle).getImage());

		activatePlugin(false);

		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(this);
		pack();

		lblNotification = new NotificationLabel();
		lblNotification.setBackground(Color.WHITE);
		lblNotification.setForeground(Color.BLACK);
		lblNotification.setNotifyAlignement(JLabel.RIGHT, JLabel.TOP);
		lblNotification.setNotifyLocation(0, pluginToolBar.getHeight()+menuBar.getHeight());

		getLayeredPane().add(lblNotification, new Integer(1));

		setVisible(true);
	}

	/* ***************** */
	/* GETTERS & SETTERS */
	/* ***************** */
	public final Graph getGraph() {
		return myGraph;
	}

	public void setChkInteracts(boolean checked) {
		chkInteracts.setSelected(checked);
	}

	/* **************** */
	/* BUILDING METHODS */
	/* **************** */
	private final void buildMenuBar() {
		// File menu:
		mnuPlugin = new JMenu("Plugin");
		mnuPlugin.setMnemonic('P');

		/*mnuPlugin_Open = new JMenuItem("Load...");
		mnuPlugin_Open.setMnemonic('O');
		URL iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "open.gif");
		if (iconUrl != null)
			mnuPlugin_Open.setIcon(new ImageIcon(iconUrl));
		mnuPlugin_Open.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (openDialog == null){
					openDialog = new JFileChooser();
					openDialog.setAcceptAllFileFilterUsed(false);
					openDialog.setApproveButtonText("Load");
					openDialog.setFileFilter(new ExportFileFilter("xml", "VOTable (Photometry-DM)"));
					openDialog.setDialogTitle("Load Photometry...");
				}
				int response = openDialog.showOpenDialog(SEDFrame.this);
				if (response == JFileChooser.APPROVE_OPTION){
					try{
						SEDParser parser = new SEDParser((WavelengthUnit)myPlugin.getXUnit(), (FluxUnit)myPlugin.getYUnit());
						parser.parse(openDialog.getSelectedFile());
						Iterator it = parser.getSEDs();
						if (!it.hasNext()){
							JOptionPane.showMessageDialog(SEDFrame.this, "No SED found !\n(note: flux unit in mag not yet managed !)", "VOTable loading", JOptionPane.WARNING_MESSAGE);
							return;
						}
						while(it.hasNext())
							myPlugin.addSpectrum(((Spectrum)it.next()));
					}catch(Exception ex){
						ErrorsLog.getCurrentInstance().addException(new SEDException(ex, "While loading a VOTable !"));
						lblNotification.setText(ex.getMessage());
					}
				}
			}
		});
		mnuPlugin.add(mnuPlugin_Open);

		mnuPlugin_OpenUrl = new JMenuItem("Load from URL...");
		mnuPlugin_OpenUrl.setMnemonic('U');
		mnuPlugin_OpenUrl.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String voTableUrl = JOptionPane.showInputDialog(SEDFrame.this, "URL of the VOTable (Photometry-DM) to load ?", "Load SED...", JOptionPane.QUESTION_MESSAGE);
				if (voTableUrl != null && voTableUrl.trim().length() > 0){
					try {
						URL url = new URL(voTableUrl.trim());
						SEDParser parser = new SEDParser((WavelengthUnit)myPlugin.getXUnit(), (FluxUnit)myPlugin.getYUnit());
						parser.parse(url.openStream());
						Iterator it = parser.getSEDs();
						if (!it.hasNext()){
							JOptionPane.showMessageDialog(SEDFrame.this, "No SED found !\n(note: flux unit in mag not yet managed !)", "VOTable loading", JOptionPane.WARNING_MESSAGE);
							return;
						}
						while(it.hasNext())
							myPlugin.addSpectrum(((Spectrum)it.next()));
					} catch (MalformedURLException ex) {
						ErrorsLog.getCurrentInstance().addException(new SEDException(ex, "While building a VOTable URL !"));
						lblNotification.setText(ex.getMessage());
					} catch (Exception ex) {
						ErrorsLog.getCurrentInstance().addException(new SEDException(ex, "While loading a VOTable !"));
						lblNotification.setText(ex.getMessage());
					}

				}
			}
		});
		mnuPlugin.add(mnuPlugin_OpenUrl);

		mnuPlugin.addSeparator();*/

		mnuPlugin_Preferences = new JMenuItem("Preferences...");
		mnuPlugin_Preferences.setMnemonic('P');
		URL iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "settings.gif");
		if (iconUrl != null)
			mnuPlugin_Preferences.setIcon(new ImageIcon(iconUrl));
		mnuPlugin_Preferences.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				(new PreferencesDialog(SEDFrame.this)).setVisible(true);
			}
		});
		mnuPlugin.add(mnuPlugin_Preferences);

		mnuPlugin.addSeparator();

		mnuPlugin_Quit = new JMenuItem("Quit");
		mnuPlugin_Quit.setMnemonic('Q');
		mnuPlugin_Quit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
		mnuPlugin_Quit.addActionListener(new ExitActionListener());
		mnuPlugin.add(mnuPlugin_Quit);

		// Spectrum menu:
		mnuSpectrum = new JMenu("Spectrum");
		mnuSpectrum.setMnemonic('S');

		mnuSpectrum_Rename = new JMenuItem("Rename");
		mnuSpectrum_Rename.setMnemonic('n');
		mnuSpectrum_Rename.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "pencil.gif");
		if (iconUrl != null)
			mnuSpectrum_Rename.setIcon(new ImageIcon(iconUrl));
		mnuSpectrum_Rename.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				lstSEDs.triggerRenameSelectedValue();
			}
		});
		mnuSpectrum.add(mnuSpectrum_Rename);

		mnuSpectrum_Refresh = new JMenuItem("Refresh");
		mnuSpectrum_Refresh.setMnemonic('R');
		mnuSpectrum_Refresh.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "refresh.gif");
		if (iconUrl != null)
			mnuSpectrum_Refresh.setIcon(new ImageIcon(iconUrl));
		mnuSpectrum_Refresh.addActionListener(new ReloadActionListener());
		mnuSpectrum.add(mnuSpectrum_Refresh);

		mnuSpectrum_Delete = new JMenuItem("Delete");
		mnuSpectrum_Delete.setMnemonic('D');
		mnuSpectrum_Delete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "delete.gif");
		if (iconUrl != null)
			mnuSpectrum_Delete.setIcon(new ImageIcon(iconUrl));
		mnuSpectrum_Delete.addActionListener(new DeleteSpectrumActionListener());
		mnuSpectrum.add(mnuSpectrum_Delete);

		mnuSpectrum_Lock = new JCheckBoxMenuItem("Lock");
		mnuSpectrum_Lock.setMnemonic('L');
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "lock.gif");
		if (iconUrl != null)
			mnuSpectrum_Lock.setIcon(new ImageIcon(iconUrl));
		mnuSpectrum_Lock.addActionListener(new LockSpectrumActionListener());
		mnuSpectrum.add(mnuSpectrum_Lock);

		mnuSpectrum.addSeparator();

		mnuSpectrum_Export = new JMenuItem("Export...");
		mnuSpectrum_Export.setMnemonic('E');
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "save.gif");
		if (iconUrl != null)
			mnuSpectrum_Export.setIcon(new ImageIcon(iconUrl));
		mnuSpectrum_Export.addActionListener(new ExportActionListener());
		mnuSpectrum.add(mnuSpectrum_Export);

		mnuSpectrum_SendTo = new JMenu("Send to...");
		mnuSpectrum_SendTo.setMnemonic('S');
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "samp.gif");
		if (iconUrl != null)
			mnuSpectrum_SendTo.setIcon(new ImageIcon(iconUrl));
		mnuSpectrum_SendTo.addMenuListener(new SendToMenuListener());
		mnuSpectrum.add(mnuSpectrum_SendTo);

		// Tools menu:
		mnuTools = new JMenu("Tools");
		mnuTools.setMnemonic('T');

		mnuTools_Subtractor = new JMenuItem("Background subtraction...");
		mnuTools_Subtractor.setMnemonic('B');
		mnuTools_Subtractor.addActionListener(new SubtractActionListener());
		mnuTools.add(mnuTools_Subtractor);

		mnuTools_Calculator = new JMenuItem("Calculator");
		mnuTools_Calculator.setMnemonic('C');
		mnuTools_Calculator.addActionListener(new ComputeActionListener());
		mnuTools.add(mnuTools_Calculator);

		// Characterization menu:
		mnuCharac = new JMenuItem("Characterization editor");
		mnuCharac.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				displayCharacterization(null);
			}
		});
		mnuTools.add(mnuCharac);

		/*mnuPlugin_CatalogExplorer = new JMenuItem("Catalog explorer");
		mnuPlugin_CatalogExplorer.setMnemonic('L');
		mnuPlugin_CatalogExplorer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!CatalogExplorer.isAlreadyInstanciated())
					new CatalogExplorer(SEDFrame.this, myPlugin);
			}
		});
		mnuTools.add(mnuPlugin_CatalogExplorer);*/

		mnuHelp = new JMenu("Help");
		mnuHelp_About = new JMenuItem("About...");
		mnuHelp_About.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				about.setVisible(true);
			}
		});
		mnuHelp.add(mnuHelp_About);

		// Finalize the whole menu:
		menuBar = new JMenuBar();
		menuBar.add(mnuPlugin);
		menuBar.add(mnuSpectrum);
		menuBar.add(mnuTools);
		menuBar.add(mnuHelp);

		setJMenuBar(menuBar);
	}

	private final void buildPluginToolBar() {
		pluginToolBar = new JToolBar();
		pluginToolBar.setFloatable(false);

		URL iconUrl = null;

		// Interacts option:
		if (icoConnect == null){
			iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "connect.png");
			icoConnect = (iconUrl==null)?null:new ImageIcon(iconUrl);
		}
		if (icoDisconnect == null){
			iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "disconnect.png");
			icoDisconnect = (iconUrl==null)?null:new ImageIcon(iconUrl);
		}
		chkInteracts = new JCheckBox("Interacts with Aladin ?", icoConnect);
		chkInteracts.setRolloverEnabled(true);
		chkInteracts.setFont(chkInteracts.getFont().deriveFont(Font.BOLD));
		chkInteracts.setSelected(true /*myPlugin.isInteracting()*/);
		chkInteracts.setToolTipText("Check if you want catch events from Aladin.");
		chkInteracts.addItemListener(new InteractionItemListener());

		// Errors report command:
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "synchronizeError.png");
		cmdErrors = (iconUrl == null) ? new JButton("Errors") : new JButton(new ImageIcon(iconUrl));
		cmdErrors.setText(""+(ErrorsLog.getCurrentInstance().getNbLastErrors()/*+ ErrorsLog.getCurrentInstance().getNbCriticalErrors()*/));
		cmdErrors.setVisible(false);
		cmdErrors.setRolloverEnabled(true);
		cmdErrors.setToolTipText("<html>"
				+ ErrorsLog.getCurrentInstance().getNbLastErrors()
				+ " errors since the last synchronization with Aladin<br />"
				+ ErrorsLog.getCurrentInstance().getNbErrors()
				+ " errors in all<br />"
				+ "Click to display all errors which have occured since the SEDPlugin start.</html>");
		cmdErrors.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				ErrorsLog.getCurrentInstance().displayErrors();
			}
		});

		// Refresh command and progress bar:
		// ...refresh command:
		if (icoSynchronize == null){
			iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "synchronize.png");
			icoSynchronize = (iconUrl==null)?null:new ImageIcon(iconUrl);
		}
		if (icoStop == null){
			iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "stop.gif");
			icoStop = (iconUrl==null)?null:new ImageIcon(iconUrl);
		}
		cmdRefreshAll = (icoSynchronize == null)?new JButton("Synchronize"): new JButton(icoSynchronize);
		cmdRefreshAll.setRolloverEnabled(true);
		cmdRefreshAll.setBorderPainted(true);
		cmdRefreshAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (proxy.isSynchronizing())
					proxy.stopSynchronizing();
				else
					proxy.synchronize();
			}
		});
		cmdRefreshAll.setToolTipText("Synchronize with Aladin.");
		// ...refresh progress bar:
		progress = new JProgressBar(0, 100);
		progress.setValue(100);
		progress.setVisible(false);
		progress.setPreferredSize(new Dimension(300, 30));
		progress.setMinimumSize(new Dimension(300, 30));
		progress.setMaximumSize(new Dimension(300, 30));
		progress.setStringPainted(true);
		progress.setString("Synchronized");

		pluginToolBar.add(chkInteracts);
		pluginToolBar.addSeparator();
		pluginToolBar.add(cmdRefreshAll);
		pluginToolBar.add(progress);
		pluginToolBar.add(Box.createHorizontalGlue());
		pluginToolBar.add(cmdErrors);

		getContentPane().add(pluginToolBar, BorderLayout.NORTH);
	}

	private final void buildSpectrumList() {
		panSpectrumList = new JPanel(new BorderLayout());

		// COMMANDS:
		// Save spectrum:
		URL iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "save.gif");
		if (iconUrl == null)
			cmdSave = new JButton("Save");
		else
			cmdSave = new JButton(new ImageIcon(iconUrl));
		cmdSave.setToolTipText("Save one of the selected spectrums.");
		cmdSave.addActionListener(new ExportActionListener());

		// Send_to with SAMP:
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "samp.gif");
		if (iconUrl != null)
			cmdSendTo = new JButton(new ImageIcon(iconUrl));
		else
			cmdSendTo = new JButton("Send");
		cmdSendTo.setToolTipText("Send the selected spectrum to a SAMP compatible application.");
		cmdSendTo.addMouseListener(new SendToMouseListener());

		// Reload:
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "refresh.gif");
		if (iconUrl == null)
			cmdReload = new JButton("Reload");
		else
			cmdReload = new JButton(new ImageIcon(iconUrl));
		cmdReload.setToolTipText("Reload selected spectrums [F5]");
		cmdReload.addActionListener(new ReloadActionListener());

		// Delete:
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "delete.gif");
		if (iconUrl != null)
			cmdDelete = new JButton(new ImageIcon(iconUrl));
		else
			cmdDelete = new JButton("X");
		cmdDelete.addActionListener(new DeleteSpectrumActionListener());
		cmdDelete.setToolTipText("Delete selected spectrums [Del]");

		// Lock/Unlock:
		if (icoLock == null) {
			iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "lock.gif");
			if (iconUrl != null)
				icoLock = new ImageIcon(iconUrl);
		}
		if (icoUnlock == null) {
			iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "unlock.gif");
			if (iconUrl != null)
				icoUnlock = new ImageIcon(iconUrl);
		}
		cmdLock = new JToggleButton();
		if (icoLock == null || icoUnlock == null)
			cmdLock.setText("Lock");
		else {
			cmdLock.setIcon(icoUnlock);
			cmdLock.setSelectedIcon(icoLock);
		}
		cmdLock.addActionListener(new LockSpectrumActionListener());
		cmdLock.setToolTipText("Click to protect selected spectrums against any Aladin stack modification.");

		// Show/Hide info panel:
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "info.gif");
		if (iconUrl == null)
			cmdInfo = new JToggleButton("Info");
		else
			cmdInfo = new JToggleButton(new ImageIcon(iconUrl));
		cmdInfo.setToolTipText("Print some information about the current spectrum. [F1]");
		cmdInfo.addActionListener(new InfoActionListener());

		// ADD THE COMMANDS TOOL BAR:
		toolBarList = new JToolBar(JToolBar.HORIZONTAL);
		toolBarList.setRollover(true);
		toolBarList.setFloatable(false);
		toolBarList.setMinimumSize(new Dimension(0, 30));
		toolBarList.add(cmdSave);
		toolBarList.add(cmdSendTo);
		toolBarList.addSeparator();
		toolBarList.add(cmdReload);
		toolBarList.add(cmdDelete);
		toolBarList.addSeparator();
		toolBarList.add(cmdLock);
		toolBarList.add(cmdInfo);

		// CREATE META-DATA PANEL:
		buildMetadataPanel();

		// CREATE THE SPECTRUMS LIST:
		series = new SEDListModel(myGraph);
		lstSEDs = new SEDJList(series);
		lstSEDs.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		lstSEDs.addListSelectionListener(new SpectrumListSelectionListener());
		lstSEDs.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent evt) {
				// highlight the SED source:
				if (UserPreferences.getCurrentInstance().moveToOnSpectrumClick()){
					SEDSource src = ((Serie)evt.getSource()).getSED().getSource();
					if (src != null)
						src.highlight();
				}
			}
		});
		lstSEDs.addKeyListener(new SpectrumListKeyListener());

		JScrollPane scpSpectrum = new JScrollPane(lstSEDs, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scpSpectrum.setMinimumSize(new Dimension(0, 200));
		scpSpectrum.setPreferredSize(new Dimension(200, (int) scpSpectrum.getPreferredSize().getHeight()));
		scpSpectrum.setOpaque(true);

		panSpectrumList.add(toolBarList, BorderLayout.NORTH);
		panSpectrumList.add(scpSpectrum, BorderLayout.CENTER);
		panSpectrumList.add(panMetadata, BorderLayout.SOUTH);
	}

	private final void buildMetadataPanel() {
		panMetadata = new JTabbedPane();
		panMetadata.setMinimumSize(new Dimension(0, 100));
		panMetadata.addTab("SED", getSpectrumTab());
		panMetadata.addTab("Points", getPointsTab());
		panMetadata.setVisible(false);
	}

	private final JPanel getSpectrumTab() {
		JPanel panSpectrum = new JPanel(new BorderLayout());

		// Name:
		lblNameField = new JLabel("Name: ");
		lblNameField.setFont(lblNameField.getFont().deriveFont(Font.BOLD));
		lblNameField.setAlignmentX(Component.LEFT_ALIGNMENT);
		lblName = new JLabel("");
		lblName.setAlignmentX(Component.LEFT_ALIGNMENT);

		// Type:
		lblTypeField = new JLabel("Type: ");
		lblTypeField.setFont(lblTypeField.getFont().deriveFont(Font.BOLD));
		lblType = new JLabel("");
		lblType.setFont(lblType.getFont().deriveFont(Font.PLAIN));

		// Ra:
		lblRaField = new JLabel("RA: ");
		lblRaField.setFont(lblRaField.getFont().deriveFont(Font.BOLD));
		lblRa = new JLabel("");
		lblRa.setFont(lblRa.getFont().deriveFont(Font.PLAIN));

		// Dec:
		lblDecField = new JLabel("DEC: ");
		lblDecField.setFont(lblDecField.getFont().deriveFont(Font.BOLD));
		lblDec = new JLabel("");
		lblDec.setFont(lblDec.getFont().deriveFont(Font.PLAIN));

		// Radius:
		lblRadiusField = new JLabel("Radius: ");
		lblRadiusField.setFont(lblRadiusField.getFont().deriveFont(Font.BOLD));
		lblRadius = new JLabel("");
		lblRadius.setFont(lblRadius.getFont().deriveFont(Font.PLAIN));

		JPanel panSpectrumMetadataFields = new JPanel(new GridLayout(5, 1));
		JPanel panSpectrumMetadataValues = new JPanel(new GridLayout(5, 1));
		panSpectrumMetadataFields.add(lblNameField);
		panSpectrumMetadataValues.add(lblName);
		panSpectrumMetadataFields.add(lblTypeField);
		panSpectrumMetadataValues.add(lblType);
		panSpectrumMetadataFields.add(lblRaField);
		panSpectrumMetadataValues.add(lblRa);
		panSpectrumMetadataFields.add(lblDecField);
		panSpectrumMetadataValues.add(lblDec);
		panSpectrumMetadataFields.add(lblRadiusField);
		panSpectrumMetadataValues.add(lblRadius);

		/*JPanel panSpectrumCommands = new JPanel();
		cmdSelectMeasure = new JButton("Select in Aladin");
		cmdSelectMeasure.setToolTipText("Select (or draw if needed) the corresponding plan in Aladin.");
		cmdSelectMeasure.addActionListener(new SelectMeasureActionListener());
		panSpectrumCommands.add(cmdSelectMeasure);*/

		panSpectrum.add(panSpectrumMetadataFields, BorderLayout.WEST);
		panSpectrum.add(panSpectrumMetadataValues, BorderLayout.CENTER);
		//panSpectrum.add(panSpectrumCommands, BorderLayout.SOUTH);

		return panSpectrum;
	}

	private final JPanel getPointsTab() {
		JPanel panPoints = new JPanel(new BorderLayout());

		// Position panel:
		JPanel panPointSelection = new JPanel();
		lblPointNumberField = new JLabel("Selected: ");
		lblPointNumberField.setFont(lblPointNumberField.getFont().deriveFont(
				Font.BOLD));
		spiPointNumber = new JSpinner();
		spiPointNumber.addChangeListener(new PointNumberChangeListener());
		lblNbPoints = new JLabel("");
		panPointSelection.add(lblPointNumberField);
		panPointSelection.add(spiPointNumber);
		panPointSelection.add(lblNbPoints);

		// Wavelength:
		lblWavelengthField = new JLabel("X: ");
		lblWavelengthField.setFont(lblWavelengthField.getFont().deriveFont(
				Font.BOLD));
		lblWavelength = new JLabel("");
		lblWavelength.setFont(lblWavelength.getFont().deriveFont(Font.PLAIN));

		// Flux:
		lblFluxField = new JLabel("Y: ");
		lblFluxField.setFont(lblFluxField.getFont().deriveFont(Font.BOLD));
		lblFlux = new JLabel("");
		lblFlux.setFont(lblFlux.getFont().deriveFont(Font.PLAIN));

		// Image:
		lblImageField = new JLabel("Image: ");
		lblImageField.setFont(lblImageField.getFont().deriveFont(Font.BOLD));
		lblImage = new JLabel("");
		lblImage.setFont(lblImage.getFont().deriveFont(Font.PLAIN));

		JPanel panPointsMetadaFields = new JPanel(new GridLayout(3, 1));
		JPanel panPointsMetadaValues = new JPanel(new GridLayout(3, 1));
		panPointsMetadaFields.add(lblWavelengthField);
		panPointsMetadaValues.add(lblWavelength);
		panPointsMetadaFields.add(lblFluxField);
		panPointsMetadaValues.add(lblFlux);
		panPointsMetadaFields.add(lblImageField);
		panPointsMetadaValues.add(lblImage);

		// Commands panel:
		/*cmdSelectImage = new JButton("Load image");
		cmdSelectImage.setAlignmentX(CENTER_ALIGNMENT);
		cmdSelectImage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// If only one spectrum is selected:
				if (lstSEDs.getSelectedIndices().length == 1 && lstSEDs.getSelectedIndex() > -1) {
					// get the indice of the selected spectrum point:
					int indPoint = ((Integer)spiPointNumber.getValue()).intValue();
					if (indPoint > 0) {
						// get the selected spectrum:
						SED sp = ((Serie)lstSEDs.getSelectedValue()).getSED();
						// load the corresponding image in Aladin:
						if (sp != null)
							//myPlugin.loadImage(sp.get(indPoint - 1));
							// TODO Select/Load image !
							System.err.println("DEBUG: No image selection for now ! => TO DO");
					}
				}
			}
		});*/
		cmdViewCharacFile = new JButton("Characterization");
		cmdViewCharacFile.setAlignmentX(CENTER_ALIGNMENT);
		cmdViewCharacFile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// If only one spectrum is selected:
				if (lstSEDs.getSelectedIndices().length == 1
						&& lstSEDs.getSelectedIndex() > -1) {
					// get the indice of the selected spectrum point:
					int indPoint = ((Integer) spiPointNumber.getValue())
					.intValue();
					if (indPoint > 0) {
						// get the selected spectrum:
						SED sp = ((Serie)lstSEDs.getSelectedValue()).getSED();
						if (sp != null) {
							// get the specified spectrum point:
							SEDPoint spt = sp.get(indPoint - 1);
							if (spt != null) {
								// get its origin:
								AladinImage io = spt.getImage();
								if (io != null)
									displayCharacterization(io);
							}
						}
					}
				}
			}
		});

		JPanel panPointsCommands = new JPanel();
		panPointsCommands.setLayout(new BoxLayout(panPointsCommands, BoxLayout.Y_AXIS));
		//panPointsCommands.add(cmdSelectImage);
		panPointsCommands.add(cmdViewCharacFile);

		panPoints.add(panPointSelection, BorderLayout.NORTH);
		panPoints.add(panPointsMetadaFields, BorderLayout.WEST);
		panPoints.add(panPointsMetadaValues, BorderLayout.CENTER);
		panPoints.add(panPointsCommands, BorderLayout.SOUTH);

		return panPoints;
	}

	private final void buildGraphPanel() {
		panGraph = new JPanel();
		panGraph.setLayout(new BorderLayout());
		panGraph.setMinimumSize(new Dimension(600, 200));
		panGraph.setEnabled(false);

		// Graph:
		panGraph.add(myGraph, BorderLayout.CENTER);

		buildCards();
		panGraph.add(panCards, BorderLayout.NORTH);

		// Position indications:
		panPosition = new JPanel(new GridLayout(1, 2));
		panPosition.setVisible(false);
		panPosition.setBackground(Color.WHITE);
		lblXPosition = new JLabel("Wavelength = - \u03BCm");
		lblXPosition.setForeground(Color.BLACK);
		lblYPosition = new JLabel("Flux = - Jy/sr");
		lblYPosition.setForeground(Color.BLACK);
		panPosition.add(lblXPosition);
		panPosition.add(lblYPosition);
		panGraph.add(panPosition, BorderLayout.SOUTH);

		// Toolbar panel:
		graphToolBar = new JToolBar(JToolBar.VERTICAL);
		graphToolBar.setBackground(Color.WHITE);

		URL iconUrl = getClass()
		.getResource(SEDPlugin.ICONS_DIR + "rulers.gif");
		if (iconUrl == null)
			cmdUnits = new JToggleButton("Units");
		else
			cmdUnits = new JToggleButton(new ImageIcon(iconUrl));
		cmdUnits.setToolTipText("Axis");
		cmdUnits.setSelected(true);
		cmdUnits.addActionListener(new UnitsActionListener());
		cmdUnits.setBackground(Color.WHITE);
		cmdUnits.setAlignmentX(Component.CENTER_ALIGNMENT);
		graphToolBar.add(cmdUnits);

		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "series.gif");
		if (iconUrl == null)
			cmdGraphPreferences = new JToggleButton("Graph");
		else
			cmdGraphPreferences = new JToggleButton(new ImageIcon(iconUrl));
		cmdGraphPreferences.setToolTipText("Graph preferences");
		cmdGraphPreferences
		.addActionListener(new GraphPreferencesActionListener());
		cmdGraphPreferences.setBackground(Color.WHITE);
		cmdGraphPreferences.setAlignmentX(Component.CENTER_ALIGNMENT);
		graphToolBar.add(cmdGraphPreferences);
		graphToolBar.addSeparator();

		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "grid.gif");
		if (iconUrl == null)
			cmdGrid = new JToggleButton("Grid");
		else
			cmdGrid = new JToggleButton(new ImageIcon(iconUrl));
		cmdGrid.setToolTipText("Show grid");
		cmdGrid.addActionListener(new GridActionListener());
		cmdGrid.setBackground(Color.WHITE);
		cmdGrid.setAlignmentX(Component.CENTER_ALIGNMENT);
		graphToolBar.add(cmdGrid);

		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "legend.gif");
		if (iconUrl == null)
			cmdLegend = new JToggleButton("Legend");
		else
			cmdLegend = new JToggleButton(new ImageIcon(iconUrl));
		cmdLegend.setToolTipText("Show legend");
		cmdLegend.addActionListener(new LegendActionListener());
		cmdLegend.setBackground(Color.WHITE);
		cmdLegend.setEnabled(false);
		cmdLegend.setAlignmentX(Component.CENTER_ALIGNMENT);
		graphToolBar.add(cmdLegend);

		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "cross_position.gif");
		if (iconUrl == null)
			cmdPositionCursor = new JToggleButton("Cursor");
		else
			cmdPositionCursor = new JToggleButton(new ImageIcon(iconUrl));
		cmdPositionCursor.setToolTipText("Show position cursor");
		cmdPositionCursor.addActionListener(new PositionCursorActionListener());
		cmdPositionCursor.setBackground(Color.WHITE);
		cmdPositionCursor.setAlignmentX(Component.CENTER_ALIGNMENT);
		graphToolBar.add(cmdPositionCursor);

		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "export.gif");
		if (iconUrl == null)
			cmdExport = new JButton("Export");
		else
			cmdExport = new JButton(new ImageIcon(iconUrl));
		cmdExport.setToolTipText("Export figure as image...[Ctrl+E]");
		cmdExport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!myGraph.isEmpty()){
					if (myExportFigureDialog == null)
						myExportFigureDialog = new ExportDialog(true);
					myExportFigureDialog.show(SEDFrame.this, myGraph);
				}
			}
		});
		cmdExport.setBackground(Color.WHITE);
		cmdExport.setEnabled(false);
		cmdExport.setAlignmentX(Component.CENTER_ALIGNMENT);
		graphToolBar.addSeparator();
		graphToolBar.add(cmdExport);

		panGraph.add(graphToolBar, BorderLayout.EAST);
	}

	private final void buildCards() {
		panCards = new JPanel(new BorderLayout());

		cards = new JPanel(new CardLayout());

		buildUnitsCard();
		cards.add(cardUnits, UNITS);

		buildGraphPreferencesCard();
		cards.add(cardGraphPreferences, GRAPH_PREFERENCES);

		URL iconUrl = getClass().getResource(
				SEDPlugin.ICONS_DIR + "bigUnits.gif");
		icoUnits = (iconUrl == null) ? null : new ImageIcon(iconUrl);
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "bigSeries.gif");
		icoGraphPreferences = (iconUrl == null) ? null : new ImageIcon(iconUrl);

		panCards.add(cards, BorderLayout.CENTER);

		lblCardIcon = new JLabel();
		lblCardIcon.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		panCards.add(lblCardIcon, BorderLayout.WEST);

		panCards.setVisible(false);
	}

	private final void buildUnitsCard() {
		// UNITS PANEL:
		int scaleType = myGraph.getFrameOfReference().getScaleType();
		cardUnits = new JPanel();
		cardUnits.setLayout(new GridLayout(2, 1));

		JPanel panAbscissa = new JPanel(new FlowLayout(FlowLayout.LEADING));
		lblAbscissa = new JLabel("Abscissa: ");
		lblAbscissa.setFont(new Font(lblAbscissa.getFont().getFontName(), Font.BOLD, lblAbscissa.getFont().getSize()));
		panAbscissa.add(lblAbscissa);

		AxisUnit unit = myPlugin.getXAxis();
		cboAbscissaKind = new JComboBox(getHtmlQuantities(unit));
		cboAbscissaKind.setSelectedIndex(unit.getIndQuantity());
		cboAbscissaKind.addActionListener(new AbscissaKindActionListener());
		panAbscissa.add(cboAbscissaKind);

		cboAbscissaUnits = new JComboBox(getUnicodeUnits(unit.getAllUnits()));
		cboAbscissaUnits.setSelectedIndex(unit.getQuantity().getIndUnit());
		cboAbscissaUnits.addActionListener(new AbscissaUnitActionListener());
		panAbscissa.add(cboAbscissaUnits);

		chkAbscissaLog = new JCheckBox("Log ");
		chkAbscissaLog.addActionListener(new LogScaleActionListener());
		chkAbscissaLog.setSelected(scaleType == AGraphFrame.X_LOG || scaleType == AGraphFrame.XY_LOG);
		panAbscissa.add(chkAbscissaLog);

		panOrdinate = new JPanel(new FlowLayout(FlowLayout.LEADING));
		lblOrdinate = new JLabel("Ordinate: ");
		lblOrdinate.setFont(new Font(lblOrdinate.getFont().getFontName(), Font.BOLD, lblOrdinate.getFont().getSize()));
		panOrdinate.add(lblOrdinate);

		unit = myPlugin.getYAxis();
		cboOrdinateKind = new JComboBox(getHtmlQuantities(unit));
		cboOrdinateKind.setSelectedIndex(unit.getIndQuantity());
		cboOrdinateKind.addActionListener(new OrdinateKindActionListener());
		panOrdinate.add(cboOrdinateKind);

		cboOrdinateUnits = new JComboBox(getUnicodeUnits(unit.getAllUnits()));
		cboOrdinateUnits.setSelectedIndex(unit.getQuantity().getIndUnit());
		cboOrdinateUnits.addActionListener(new OrdinateUnitActionListener());
		panOrdinate.add(cboOrdinateUnits);

		chkOrdinateLog = new JCheckBox("Log ");
		chkOrdinateLog.addActionListener(new LogScaleActionListener());
		chkOrdinateLog.setSelected(scaleType == AGraphFrame.Y_LOG || scaleType == AGraphFrame.XY_LOG);
		panOrdinate.add(chkOrdinateLog);

		cardUnits.add(panAbscissa);
		cardUnits.add(panOrdinate);
	}

	private final void buildGraphPreferencesCard() {
		// BUILDING OF GRAPH PREFERENCES:
		cardGraphPreferences = new JPanel();
		cardGraphPreferences.setLayout(new GridLayout(2, 1));

		JPanel panGraph = new JPanel(new FlowLayout(FlowLayout.LEADING));

		JLabel lblGraph = new JLabel("Title: ");
		lblGraph.setFont(new Font(lblGraph.getFont().getFontName(), Font.BOLD, lblGraph.getFont().getSize()));
		panGraph.add(lblGraph);

		txtGraphTitle = new JTextField(myGraph.getFrameOfReference().title);
		txtGraphTitle.setPreferredSize(new Dimension(200, 30));
		txtGraphTitle.addKeyListener(new GraphTitleKeyListener());
		panGraph.add(txtGraphTitle);

		JLabel lblBackrgound = new JLabel("Background: ");
		lblBackrgound.setFont(new Font(lblBackrgound.getFont().getFontName(), Font.BOLD, lblBackrgound.getFont().getSize()));
		panGraph.add(lblBackrgound);

		cboBackground = new JComboBox(Serie.SERIE_COLORS);
		cboBackground.setEditable(false);
		cboBackground.setRenderer(new ColorRenderer(false));
		cboBackground.setSelectedItem(myGraph.getBackground());
		cboBackground.addActionListener(new BackgroundColorActionListener());
		cboBackground.setToolTipText("Change background color");
		panGraph.add(cboBackground);

		JLabel lblForeground = new JLabel("Axis: ");
		lblForeground.setFont(new Font(lblForeground.getFont().getFontName(), Font.BOLD, lblForeground.getFont().getSize()));
		panGraph.add(lblForeground);

		cboForeground = new JComboBox(Serie.SERIE_COLORS);
		cboForeground.setEditable(false);
		cboForeground.setRenderer(new ColorRenderer(false));
		cboForeground.setSelectedItem(myGraph.getFrameOfReference().getColor());
		cboForeground.addActionListener(new ForegroundColorActionListener());
		cboForeground.setToolTipText("Change foreground color");
		panGraph.add(cboForeground);

		JPanel panSeries = new JPanel(new FlowLayout(FlowLayout.LEADING));

		JLabel lblSeries = new JLabel("SED: ");
		lblSeries.setFont(new Font(lblSeries.getFont().getFontName(), Font.BOLD, lblSeries.getFont().getSize()));
		panSeries.add(lblSeries);

		cboSeries = new JComboBox();
		cboSeries.setPreferredSize(new Dimension(200, 30));
		cboSeries.addActionListener(new SeriesListSelectionListener());
		panSeries.add(cboSeries);

		URL iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR + "connection.gif");
		if (iconUrl == null)
			cmdConnectPoints = new JToggleButton("Connections");
		else
			cmdConnectPoints = new JToggleButton(new ImageIcon(iconUrl));
		cmdConnectPoints.addActionListener(new ConnectionsActionListener());
		cmdConnectPoints.setToolTipText("Connect points");
		panSeries.add(cmdConnectPoints);

		ColorRenderer serieColorRenderer = new ColorRenderer(true);
		int[] posTemp = PointRenderer.getPointPosition();
		double[] position = { posTemp[0], posTemp[1] };
		cboPointStyles = new JComboBox(new Point[] {
				new DefaultPoint(position[0], position[1], null),
				new SquarePoint(position[0], position[1], null),
				new CirclePoint(position[0], position[1], null),
				new CrossPoint(position[0], position[1], null),
				new StarPoint(position[0], position[1], null) });
		cboPointStyles.setEditable(false);
		cboPointStyles.setRenderer(new PointRenderer(serieColorRenderer));
		cboPointStyles.addActionListener(new PointStyleActionListener());
		cboPointStyles.setToolTipText("Change the point style");
		cboPointStyles.setMaximumSize(new Dimension(40, 30));
		panSeries.add(cboPointStyles);

		cboColor = new JComboBox();
		DefaultComboBoxModel colorModel =new DefaultComboBoxModel(Serie.SERIE_COLORS);
		colorModel.insertElementAt(null, 0);
		cboColor.setModel(colorModel);
		cboColor.setEditable(false);
		cboColor.setRenderer(serieColorRenderer);
		cboColor.addActionListener(new SerieColorActionListener());
		cboColor.setToolTipText("Change serie color");
		panSeries.add(cboColor);

		cardGraphPreferences.add(panGraph);
		cardGraphPreferences.add(panSeries);
	}

	/* ************** */
	/* STATIC METHODS */
	/* ************** */
	/**
	 * Transform the given units list in a string list whose each item is a unit
	 * symbol.<BR />
	 * <i>Symbols are given in unicode.</i>
	 *
	 * @param units
	 *            The units to export as symbol string list.
	 * @return A list of all available units.
	 *
	 * @see UnitDef
	 */
	private final static String[] getUnicodeUnits(UnitDef[] units) {
		String[] unitsString = new String[units.length + 1];
		for (int i = 0; i < unitsString.length - 1; i++)
			unitsString[i] = units[i].unicodeSymbol;
		unitsString[unitsString.length - 1] = ADD_NEW_UNIT;
		return unitsString;
	}

	/**
	 * Transform the given quantities list in a string list whose each item is a
	 * quantity name.<BR />
	 * <i>Quantity names are given in a HTML structure.</i>
	 *
	 * @param quantities	Physical quantities to export as quantity name list.
	 * @return 				The corresponding list of quantity name.
	 *
	 * @see AUnit.PhysicalQuantity
	 */
	private final static String[] getHtmlQuantities(AxisUnit unit) {
		String[] quantitiesString = new String[unit.getNbQuantities()];
		int i = 0;
		for (PhysicalQuantity q : unit) {
			if (q.getName() == null || q.getName().trim().length() == 0)
				quantitiesString[i] = "<html>" + q.getHtmlSymbol() + "</html>";
			else
				quantitiesString[i] = "<html>" + q.getName() + ((q.getHtmlSymbol() != null && q.getHtmlSymbol().trim().length() > 0) ? (" (" + q.getHtmlSymbol() + ")") : "")+ "</html>";
			i++;
		}
		return quantitiesString;
	}

	/**
	 * Resizes an image using a Graphics2D object backed by a BufferedImage.
	 *
	 * @param srcImg	source image to scale
	 * @param w			desired width
	 * @param h			desired height
	 * @return			the new resized image
	 */
	public static final ImageIcon getScaledImage(ImageIcon srcImg, int w, int h) {
		if (srcImg != null && srcImg.getImageLoadStatus() == MediaTracker.COMPLETE){
			BufferedImage resizedImg = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
			Graphics2D g2 = resizedImg.createGraphics();
			g2.setColor(Color.WHITE);
			g2.fillRect(0, 0, w, h);
			g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
			g2.drawImage(srcImg.getImage(), 0, 0, w, h, null);
			g2.dispose();
			return new ImageIcon(resizedImg);
		}else
			return null;
	}

	/* ************************** */
	/* PLUGIN INTERACTION METHODS */
	/* ************************** */
	public void activatePlugin(boolean activate) {
		mnuSpectrum.setEnabled(activate);
		mnuSpectrum_SendTo.setEnabled(activate && myPlugin.isSampConnected());
		mnuTools_Subtractor.setEnabled(activate);

		toolBarList.setEnabled(activate);
		cmdLock.setEnabled(activate);
		cmdInfo.setEnabled(activate);
		cmdReload.setEnabled(activate);
		cmdDelete.setEnabled(activate);
		cmdSave.setEnabled(activate);
		cmdSendTo.setEnabled(activate && myPlugin.isSampConnected());

		// Spectrum meta-data:
		//cmdSelectMeasure.setEnabled(activate);
		lblNameField.setEnabled(activate);
		if (!activate)
			lblName.setText("");
		lblTypeField.setEnabled(activate);
		if (!activate)
			lblType.setText("");
		lblRaField.setEnabled(activate);
		if (!activate)
			lblRa.setText("");
		lblDecField.setEnabled(activate);
		if (!activate)
			lblDec.setText("");
		lblRadiusField.setEnabled(activate);
		if (!activate)
			lblRadius.setText("");
		panMetadata.setEnabled(activate);

		// Points meta-data:
		lblPointNumberField.setEnabled(activate);
		if (!activate)
			spiPointNumber.setModel(new SpinnerNumberModel(0, 0, 0, 0));
		spiPointNumber.setEnabled(activate);
		lblNbPoints.setEnabled(activate);
		lblWavelengthField.setEnabled(activate);
		if (!activate)
			lblWavelength.setText("");
		lblFluxField.setEnabled(activate);
		if (!activate)
			lblFlux.setText("");
		lblImageField.setEnabled(activate);
		if (!activate)
			lblImage.setText("");
		cmdViewCharacFile.setEnabled(activate);

		lstSEDs.setEnabled(activate);

		if (!activate)
			activateGraph(false);
	}

	public void activateGraph(boolean activate){
		panGraph.setEnabled(activate);
		myGraph.setEnabled(activate);

		cards.setEnabled(activate);

		cardUnits.setEnabled(activate);
		lblAbscissa.setEnabled(activate);
		lblOrdinate.setEnabled(activate);
		cboAbscissaKind.setEnabled(activate);
		cboAbscissaUnits.setEnabled(activate);
		cboOrdinateKind.setEnabled(activate);
		cboOrdinateUnits.setEnabled(activate);
		chkAbscissaLog.setEnabled(activate);
		chkOrdinateLog.setEnabled(activate);

		cardGraphPreferences.setEnabled(activate);
		cboSeries.setEnabled(activate);
		cmdConnectPoints.setEnabled(activate);
		cboPointStyles.setEnabled(activate);
		cboColor.setEnabled(activate);

		if (activate) {
			if (!graphToolBar.isEnabled()) {
				// Select and display the units panel:
				((CardLayout) cards.getLayout()).show(cards, UNITS);
				if (icoUnits == null)
					lblCardIcon.setText(UNITS);
				else
					lblCardIcon.setIcon(icoUnits);
				cmdGraphPreferences.setSelected(false);
				cmdUnits.setSelected(true);
				panCards.setVisible(true);
			}
		} else {
			cmdGraphPreferences.setSelected(false);
			cmdUnits.setSelected(false);
			panCards.setVisible(false);
		}

		graphToolBar.setEnabled(activate);
		cmdUnits.setEnabled(activate);
		cmdGraphPreferences.setEnabled(activate);
		cmdGrid.setEnabled(activate);
		cmdLegend.setEnabled(activate);
		cmdPositionCursor.setEnabled(activate);
		cmdExport.setEnabled(activate);
	}

	public void highlightSeriePoint(Serie s, int pointIndex) {
		if (s == null || pointIndex <= -1 || pointIndex >= s.getNbPoints())
			spiPointNumber.setValue(new Integer(0));
		else {
			lstSEDs.setSelectedValue(s);
			spiPointNumber.setValue(new Integer(pointIndex+1));
		}
	}

	public void changeMousePosition(double posX, double posY, double xValue,
			double yValue) {
		if (cmdPositionCursor.isSelected()) {
			AxisUnit unit = myPlugin.getXAxis();
			lblXPosition.setText(unit.getQuantity().getName() + " = " + FrameOfReference.getXValueString(xValue) + " " + unit.getUnit().unicodeSymbol);
			unit = myPlugin.getYAxis();
			lblYPosition.setText(unit.getQuantity().getName() + " = " + FrameOfReference.getYValueString(yValue) + " " + unit.getUnit().unicodeSymbol);
		}
	}

	public void addSerie(SED sed) {
		series.addSED(sed);
		if (series.getSize() > 0){
			activatePlugin(true);
			mnuTools_Subtractor.setEnabled(series.getSize() > 1);
		}
	}

	public Serie getSerie(final SED sed){
		if (sed == null)
			return null;
		else
			return series.getSerie(sed);
	}

	public void removeSerie(SED sed) {
		series.removeSED(sed);
		if (series.getSize() == 0)
			activatePlugin(false);
		else
			mnuTools_Subtractor.setEnabled(series.getSize() > 1);
	}

	public void addImage(final AladinImage img){
		noCharacDialog.addImage(img);
	}

	public void removeImage(final AladinImage img){
		noCharacDialog.removeImage(img);
	}

	public PropertyChangeListener getAladinProgressListener() {
		return new SynchronizePropertyChangeListener(progress, cmdRefreshAll);
	}

	public void drawingSeries(Vector<Serie> series, boolean showYConfig) {
		if (series != null && !series.isEmpty()) {
			if (!myGraph.isEnabled())
				activateGraph(true);
			cboSeries.setModel(new DefaultComboBoxModel(series));
			if (!myGraph.isEmpty())
				cboSeries.setSelectedIndex(0);
			cboColor.repaint();
			cboPointStyles.repaint();
			panOrdinate.setVisible(showYConfig);
		}else if (myGraph.isEnabled())
			activateGraph(false);
	}

	/* *************** */
	/* DISPLAY METHODS */
	/* *************** */
	public String getWelcomeMessage() {
		return about.aboutMsg;
	}

	private void showSEDMetadata(SED sed) {
		if (sed == null){
			// Clear all metadata (only usable when only one spectrum is selected):
			lblName.setText("");
			lblType.setText("");
			lblRa.setText("");
			lblDec.setText("");
			lblRadius.setText("");
			//cmdSelectMeasure.setEnabled(false);
			spiPointNumber.setModel(new SpinnerNumberModel(0, 0, 0, 0));
			spiPointNumber.setEnabled(false);
			lblNbPoints.setText("");
			lblWavelength.setText("");
			lblFlux.setText("");
			lblImage.setText("");
			cmdViewCharacFile.setEnabled(false);
		}else{
			// Display general metadata:
			lblName.setText(sed.getName());

			// Display spectrum points metadata:
			spiPointNumber.setEnabled(true);
			spiPointNumber.setModel(new SpinnerNumberModel(0, 0, sed.size(), 1));
			lblNbPoints.setText(" on " + sed.size());
			lblWavelength.setText("");
			lblFlux.setText("");
			lblImage.setText("");
			//cmdSelectMeasure.setEnabled(false);
			cmdViewCharacFile.setEnabled(false);

			// Display spectrum origin:
			if (sed.getSource() == null){
				lblType.setText("?");
				lblRa.setText("?");
				lblDec.setText("?");
				lblRadius.setText("?");
			}else{
				SEDSource src = sed.getSource();
				lblRa.setToolTipText("");
				String srcType = src.getType();
				/*if (srcType.equals(Position.POSITION_TYPE)){
					lblType.setText("Position");
					lblRaField.setVisible(true);
					lblRaField.setText("RA: ");
					lblRa.setVisible(true);
					lblRa.setText(sexaCoord.substring(0, sexaCoord.indexOf(" ")));
					lblDecField.setVisible(true);
					lblDec.setVisible(true);
					lblDec.setText(sexaCoord.substring(sexaCoord.indexOf(" ") + 1, sexaCoord.length()));
					lblRadiusField.setVisible(false);
					lblRadius.setVisible(false);
				}else*/ if (srcType.equals(Circle.CIRCLE_TYPE)){
					Circle c = (Circle)src;
					String coords = c.getSexaCoords();
					String ra = (coords != null)?coords.substring(0, coords.indexOf(" ")):null;
					String dec = (coords != null)?coords.substring(coords.indexOf(" ")+1):null;
					lblType.setText("Circle");
					lblRaField.setVisible(true);
					lblRaField.setText("RA: ");
					lblRa.setVisible(true);
					lblRa.setText(ra);
					lblDecField.setVisible(true);
					lblDec.setVisible(true);
					lblDec.setText(dec);
					lblRadiusField.setVisible(true);
					lblRadius.setVisible(true);
					lblRadius.setText(c.getRadius() + "");
				}else if (srcType.equals(Polygon.POLYGON_TYPE)){
					lblType.setText("Polygon");
					lblRaField.setVisible(false);
					lblRa.setVisible(false);
					lblDecField.setVisible(false);
					lblDec.setVisible(false);
					lblDec.setText("");
					lblRadiusField.setVisible(false);
					lblRadius.setVisible(false);
					lblRadius.setText("");
				}else if (srcType.equals(Operation.OPERATION_TYPE)){
					lblType.setText("Operation");
					lblRaField.setVisible(true);
					lblRaField.setText("Expression: ");
					lblRa.setVisible(true);
					lblRa.setText(src.getName());
					lblRa.setToolTipText(src.getName());
					lblDecField.setVisible(false);
					lblDec.setVisible(false);
					lblDec.setText("");
					lblRadiusField.setVisible(false);
					lblRadius.setVisible(false);
					lblRadius.setText("");
				}/*else if (srcType.equals(Catalog.CATALOG_TYPE)){
					lblType.setText("Catalog");
					lblRaField.setVisible(true);
					lblRaField.setText("Catalog/Source: ");
					lblRa.setVisible(true);
					lblRa.setText(src.getSourceID());
					lblDecField.setVisible(false);
					lblDec.setVisible(false);
					lblRadiusField.setVisible(false);
					lblRadius.setVisible(false);
					break;
				}*/
				//cmdSelectImage.setEnabled(false);
				//cmdSelectMeasure.setEnabled(true);
				// TODO Select/Draw in Aladin
				//cmdSelectMeasure.setText((src.hasSED()) ? "Select in Aladin" : "Draw in Aladin");
			}
		}
	}

	protected void displayCharacterization(AladinImage imageToSelect){
		// Select the appropriate image in the Characterization editor:
		noCharacDialog.setCurrentImage(imageToSelect);

		// Show the editor:
		noCharacDialog.setVisible(true);
	}

	public void notifyUser(SEDException ex) {
		// Notify the user of the given error:
		lblNotification.printException(ex);
	}

	public void updateCount(){
		int nbLastErrors = ErrorsLog.getCurrentInstance().getNbLastErrors();
		int nbErrors = ErrorsLog.getCurrentInstance().getNbErrors();
		cmdErrors.setText("" + nbLastErrors);
		cmdErrors.setToolTipText("<html>"
				+ nbLastErrors
				+ " errors since the last synchronization with Aladin<br />"
				+ nbErrors
				+ " errors in all<br />"
				+ "Click to display all errors which have occured since the SEDPlugin start.</html>");
		cmdErrors.setEnabled(nbLastErrors > 0);
		cmdErrors.setVisible(nbLastErrors > 0);
	}

	/* *************** */
	/* GENERAL METHODS */
	/* *************** */
	public void close() {
		if (myCalculator != null)
			myCalculator.dispose();
		dispose();
	}

	public void windowClosing(WindowEvent e) {
		myPlugin.cleanup();
	}

	/**
	 * Refresh all spectrums according to the last modifications of the Aladin
	 * stack.
	 *
	 * @see sedplugin.plugin.SEDPlugin#refresh(int[], JProgressBar)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
		if (UserPreferences.getCurrentInstance().whenAllRefreshing() == UserPreferences.ON_ACTIVATE)
			proxy.synchronize();
	}

	/* ********* */
	/* RENDERERS */
	/* ********* */
	/**
	 * Renders a given Color object as a JPanel whose the background is in the
	 * selected color.<BR />
	 * The last selected color is saved in this renderer: to get it use the
	 * {@link ColorRenderer#getColor()} method.
	 *
	 * @author Gregory Mantelet
	 * @version 02/2010
	 */
	public final class ColorRenderer extends JPanel implements ListCellRenderer {
		private static final long serialVersionUID = 1L;

		protected Color color = Color.BLACK;
		protected final JLabel lblAuto;

		public ColorRenderer(boolean auto){
			setLayout(new BorderLayout());
			if (auto){
				lblAuto = new JLabel("Auto");
				lblAuto.setFont(lblAuto.getFont().deriveFont(Font.BOLD));
				lblAuto.setToolTipText("Auto");
				lblAuto.setOpaque(false);
				add(lblAuto, BorderLayout.CENTER);
				setPreferredSize(new Dimension(lblAuto.getPreferredSize().width+5, 20));
			}else{
				lblAuto = null;
				setPreferredSize(new Dimension(20, 20));
			}
			setOpaque(true);
			setBorder(BorderFactory.createRaisedBevelBorder());
		}

		public final Color getColor() {
			return color;
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean hasFocus) {
			if (list == null)
				return this;

			// Set the background:
			if (lblAuto != null && cboSeries != null && value == null){
				Serie s = (Serie)cboSeries.getSelectedItem();
				if (s != null)
					color = s.getColor();
				else
					color = Color.WHITE;
				lblAuto.setVisible(true);
			}else{
				color = (Color)value;
				if (lblAuto != null)
					lblAuto.setVisible(false);
			}
			setBackground(color);

			// Set the border:
			if (isSelected)
				setBorder(BorderFactory.createLoweredBevelBorder());
			else
				setBorder(BorderFactory.createRaisedBevelBorder());

			return this;
		}
	}

	/* **************** */
	/* ACTION LISTENERS */
	/* **************** */
	// Menu PLUGIN->QUIT
	private final class ExitActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			myPlugin.cleanup();
		}
	}

	// Menu SPECTRUM->EXPORT & Button SAVE:
	private final class ExportActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (!myGraph.isEmpty()){
				if (myExportSpectrumDialog == null)
					myExportSpectrumDialog = new ExportDialog(false);
				myExportSpectrumDialog.show(SEDFrame.this, myGraph);
			}
		}
	}

	// Menu SPECTRUM->SEND_TO->SAMP_Client & PopupMenu SEND_TO
	private final class SampClientActionListener implements ActionListener {
		private final String clientId;
		private final String mtype;

		public SampClientActionListener(String msgType, String id) {
			clientId = id;
			mtype = msgType;
		}

		public void actionPerformed(ActionEvent e) {
			// Display a confirm dialog:
			if (UserPreferences.getCurrentInstance()
					.askConfirmationBeforeSendToSamp()) {
				int choice = JOptionPane.showOptionDialog(SEDFrame.this,
						"Ordinate unit is : "
						+ myPlugin.getYAxis().getUnit().unicodeSymbol
						+ "\nAre you sure you want to continue ?",
						"Send to SAMP", JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, new String[] {
					"Yes", "No" }, null);
				if (choice == JOptionPane.NO_OPTION)
					return;
			}

			try {
				File exportedVOTable = new File(SEDPlugin.getNextSendFileName());
				exportedVOTable.delete();

				// Export in VOTable/Spectrum
				ExportTask task = new ExportTask(myGraph, ExportTask.VOTABLE, exportedVOTable);
				task.addPropertyChangeListener(new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent evt) {
						String property = evt.getPropertyName();

						if (property.equals("state")) {
							StateValue state = ((StateValue) evt.getNewValue());
							switch(state){
							case STARTED:
								SEDFrame.this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
								break;
							case DONE:
								SEDFrame.this.setCursor(Cursor.getDefaultCursor());
								break;
							}
						}
					}
				});
				task.execute();

				// Send with SAMP to Topcat if succeeded:
				try {
					File f = (File) task.get();
					if (f != null)
						myPlugin.sendTo(mtype, clientId, exportedVOTable, ((Serie)lstSEDs.getSelectedValue()).getSED().getName());
				} catch (Exception ex) {
					ErrorsLog.getCurrentInstance().addException(new SEDException(ex, "SAMP registering"));
				}
			} catch (SEDException e1) {
				ErrorsLog.getCurrentInstance().addException(e1);
			}
		}
	}

	// Menu SPECTRUM->RELOAD & Button RELOAD:
	private final class ReloadActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (lstSEDs.getSelectedIndex() != -1){
				Object[] series = lstSEDs.getSelectedValues();
				for(Object serie : series){
					SEDSource src = ((Serie)serie).getSED().getSource();
					if (src != null)
						src.update(true);
				}
			}
		}
	}

	// Menu SPECTRUM->DELETE & Button DELETE:
	private final class DeleteSpectrumActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Object[] selectedValues = lstSEDs.getSelectedValues();
			SED[] seds = new SED[selectedValues.length];
			for(int i=0; i<selectedValues.length; i++)
				seds[i] = ((Serie)selectedValues[i]).getSED();
			myPlugin.hideSEDs(seds);
		}
	}

	// Menu SPECTRUM->LOCK & Button LOCK:
	private final class LockSpectrumActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			// Modify the lock option of selected spectrums only if asked by a
			// user (with a mouse click):
			boolean select;
			// click on the button:
			if (e.getSource() instanceof JToggleButton) {
				select = cmdLock.isSelected();
				mnuSpectrum_Lock.setSelected(select);

				// or click on the menu item:
			} else {
				select = mnuSpectrum_Lock.isSelected();
				cmdLock.setSelected(select);
			}
			// change the lock option of the selected spectrums:
			Object[] objs = lstSEDs.getSelectedValues();
			for (int i = 0; i < objs.length; i++){
				SEDSource src = ((Serie)objs[i]).getSED().getSource();
				if (src != null)
					src.freeze(select);
			}

			cmdReload.setEnabled(!select);
			mnuSpectrum_Refresh.setEnabled(!select);

			// refresh the spectrums list:
			lstSEDs.repaint();

			// Refresh the tool tip text of the button:
			if (cmdLock.isSelected())
				cmdLock.setToolTipText("Selected spectrums are now protected against any Aladin stack modification.");
			else
				cmdLock.setToolTipText("Click to protect selected spectrums against any Aladin stack modification.");
		}
	}

	// Button INFO:
	private final class InfoActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			panMetadata.setVisible(cmdInfo.isSelected());
		}
	}

	// Menu TOOLS->BACKGROUND_SUBTRACTOR:
	private final class SubtractActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (myPlugin.getNbSEDs() > 1)
				new BackgroundSubtractorDialog(SEDFrame.this, myPlugin);
			else{
				ErrorsLog.getCurrentInstance().addException(new SEDException("Subtraction impossible", "Menu click", "More than 1 SED is required to subtract 2 SEDs !"));
				mnuTools_Subtractor.setEnabled(false);
			}
		}
	}

	// Menu TOOLS->CALCULATOR:
	private final class ComputeActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			// Build the calculator dialog if not already existing:
			if (myCalculator == null)
				myCalculator = new CalculatorDialog(SEDFrame.this, myPlugin);
			// Show the calculator:
			myCalculator.setVisible(true);
		}
	}

	// Button DRAW_MEASURES:
	/*private final class SelectMeasureActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				if (lstSEDs.getSelectedIndex() != -1 && lstSEDs.getSelectedIndices().length == 1)
					((Serie)lstSEDs.getSelectedValue()).getSED().getSource().highlight();
			} catch (ClassCastException ex) {
				ErrorsLog.getCurrentInstance().addException(
						new SEDException("Can't draw the measure",
								"Drawing a measures plan",
								"The selected item isn't a spectrum: item n°"
								+ lstSEDs.getSelectedIndex()
								+ " but a "
								+ lstSEDs.getSelectedValue()
								.getClass().getName()
								+ " object"));
			}
		}
	}*/

	// Button UNITS:
	private final class UnitsActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			cmdGraphPreferences.setSelected(false);
			if (cmdUnits.isSelected()) {
				((CardLayout) cards.getLayout()).show(cards, UNITS);
				if (icoUnits == null)
					lblCardIcon.setText(UNITS);
				else
					lblCardIcon.setIcon(icoUnits);
				panCards.setVisible(true);
			} else
				panCards.setVisible(false);
		}
	}

	// Button GRAPH_PREFERENCES:
	private final class GraphPreferencesActionListener implements
	ActionListener {
		public void actionPerformed(ActionEvent e) {
			cmdUnits.setSelected(false);
			if (cmdGraphPreferences.isSelected()) {
				((CardLayout) cards.getLayout()).show(cards,
						GRAPH_PREFERENCES);
				if (icoGraphPreferences == null)
					lblCardIcon.setText(GRAPH_PREFERENCES);
				else
					lblCardIcon.setIcon(icoGraphPreferences);
				panCards.setVisible(true);
			} else
				panCards.setVisible(false);
		}
	}

	// Button GRID:
	private final class GridActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (!myGraph.isEmpty()) {
				myGraph.getFrameOfReference().gridVisible = cmdGrid.isSelected();
				cmdGrid.setToolTipText((!cmdGrid.isSelected()) ? "Show grid" : "Hide grid");
				myGraph.update();
			}
		}
	}

	// Button LEGEND:
	private final class LegendActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (!myGraph.isEmpty()) {
				myGraph.getFrameOfReference().legendVisible = cmdLegend.isSelected();
				cmdLegend.setToolTipText(((cmdLegend.isSelected()) ? "Hide" : "Show") + " legend");
				myGraph.repaint();
			}
		}
	}

	// Button CURSOR:
	private final class PositionCursorActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (!myGraph.isEmpty()) {
				myGraph.getFrameOfReference().mousePositionVisible = cmdPositionCursor.isSelected();
				panPosition.setVisible(cmdPositionCursor.isSelected());
				cmdPositionCursor.setToolTipText(((cmdPositionCursor.isSelected()) ? "Hide" : "Show") + " position cursor");
				myGraph.repaint();
			}
		}
	}

	// ComboList BACKGROUND_COLOR:
	private final class BackgroundColorActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			myGraph.setBackground((Color) cboBackground.getSelectedItem());

			Color col = myGraph.getBackground();
			graphToolBar.setBackground(col);
			panPosition.setBackground(col);
			lblXPosition.setForeground(new Color(255 - col.getRed(), 0, 0));
			lblYPosition.setForeground(new Color(255 - col.getRed(), 0, 0));
			myGraph.update();
		}
	}

	// ComboList FOREGROUND_COLOR:
	private final class ForegroundColorActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			myGraph.getFrameOfReference().setColor((Color)cboForeground.getSelectedItem());
			myGraph.update();
		}
	}

	// ComboList X_QUANTITIES:
	private final class AbscissaKindActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			AxisUnit xUnit = myPlugin.getXAxis();
			xUnit.setQuantity(cboAbscissaKind.getSelectedIndex());
			myGraph.update();

			cboAbscissaUnits.setModel(new DefaultComboBoxModel(getUnicodeUnits(xUnit.getAllUnits())));
			cboAbscissaUnits.setSelectedIndex(xUnit.getQuantity().getIndUnit());
		}
	}

	// ComboList X_UNITS:
	private final class AbscissaUnitActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			AxisUnit xUnit = myPlugin.getXAxis();
			int indUnit = cboAbscissaUnits.getSelectedIndex();
			if (indUnit == cboAbscissaUnits.getItemCount() - 1) {
				new UnitsCustomizeDialog(SEDFrame.this, xUnit);
				cboAbscissaUnits.setModel(new DefaultComboBoxModel(getUnicodeUnits(xUnit.getAllUnits())));
				cboAbscissaUnits.setSelectedIndex(cboAbscissaUnits.getItemCount() - 2);
				xUnit.setUnit(cboAbscissaUnits.getItemCount() - 2);
			} else
				xUnit.setUnit(indUnit);
			myGraph.update();
		}
	}

	// ComboList Y_QUANTITIES:
	private final class OrdinateKindActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			AxisUnit yUnit = myPlugin.getYAxis();
			yUnit.setQuantity(cboOrdinateKind.getSelectedIndex());
			myGraph.update();

			cboOrdinateUnits.setModel(new DefaultComboBoxModel(getUnicodeUnits(yUnit.getAllUnits())));
			cboOrdinateUnits.setSelectedIndex(yUnit.getQuantity().getIndUnit());
		}
	}

	// ComboList Y_UNITS:
	private final class OrdinateUnitActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			AxisUnit yUnit = myPlugin.getYAxis();
			int indUnit = cboOrdinateUnits.getSelectedIndex();
			if (indUnit == cboOrdinateUnits.getItemCount() - 1) {
				new UnitsCustomizeDialog(SEDFrame.this, yUnit);
				cboOrdinateUnits.setModel(new DefaultComboBoxModel(getUnicodeUnits(yUnit.getAllUnits())));
				cboOrdinateUnits.setSelectedIndex(cboOrdinateUnits.getItemCount() - 2);
				yUnit.setUnit(cboOrdinateUnits.getItemCount() - 2);
			} else
				yUnit.setUnit(indUnit);
			myGraph.update();
		}
	}

	// CheckBoxes LOG_SCALE:
	private final class LogScaleActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int scaleType = (chkAbscissaLog.isSelected()) ? AGraphFrame.X_LOG : AGraphFrame.XY_LINEAR;
			scaleType += (chkOrdinateLog.isSelected()) ? AGraphFrame.Y_LOG : AGraphFrame.XY_LINEAR;
			myGraph.getFrameOfReference().setScaleType(scaleType);
			myGraph.update();
		}
	}

	// Button CONNECT_POINTS:
	private final class ConnectionsActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Serie s = ((Serie) cboSeries.getSelectedItem());
			if (s.arePointsConnected() == cmdConnectPoints.isSelected())
				return;
			s.setPointsConnected(cmdConnectPoints.isSelected());
			myGraph.update();
		}
	}

	// ComboList POINT_STYLES:
	private final class PointStyleActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Serie s = ((Serie) cboSeries.getSelectedItem());
			if (s.getPointStyle() == cboPointStyles.getSelectedIndex())
				return;
			s.setPointStyle(cboPointStyles.getSelectedIndex());
			myGraph.update();
		}
	}

	// ComboList SERIE_COLOR:
	private final class SerieColorActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Serie s = (Serie)cboSeries.getSelectedItem();
			if ((s.getColor() == null && cboColor.getSelectedItem() == null) || (s.getColor() != null && cboColor.getSelectedItem() != null && s.getColor().getRGB() == ((Color)cboColor.getSelectedItem()).getRGB()))
				return;
			if (cboColor.getSelectedIndex() <= 0 || cboColor.getSelectedItem() == null)
				s.setColor(null);
			else
				s.setColor(cboColor.getSelectedIndex()-1);
			myGraph.update();
			Component[] components = lstSEDs.getComponents();
			for(Component c : components)
				((SEDJListItem)c).update();
			lstSEDs.repaint();
		}
	}

	// ComboList SERIES:
	private final class SeriesListSelectionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Serie s = (Serie) cboSeries.getSelectedItem();
			cmdConnectPoints.setSelected(s.arePointsConnected());
			cboPointStyles.setSelectedIndex(s.getPointStyle());
			// TODO Display that the color is auto !
			cboColor.setSelectedItem(s.getColor());
		}
	}

	/* ************** */
	/* MENU LISTENERS */
	/* ************** */
	// Menu SPECTRUM->SEND_TO:
	private final class SendToMenuListener implements MenuListener {
		public void menuCanceled(MenuEvent arg0) { ; }
		public void menuDeselected(MenuEvent arg0) { ; }

		/**
		 * Refresh the list of potential SAMP clients/receivers.
		 */
		public void menuSelected(MenuEvent e) {
			// Clear the former clients list:
			mnuSpectrum_SendTo.removeAll();
			// Don't do anything if the plugin isn't registered with SAMP:
			if (myPlugin.isSampConnected()) {
				// get the list of potential SAMP receivers:
				Map<String, HashMap<String, Object[]>> mapMTypes = myPlugin.getSampClients();
				// get only their name:
				Iterator<String> itMType = mapMTypes.keySet().iterator();
				while (itMType.hasNext()) {
					String mtype = itMType.next();
					Map<String, Object[]> mapClients = mapMTypes.get(mtype);
					Iterator<String> itClient = mapClients.keySet().iterator();
					while (itClient.hasNext()) {
						String clientId = itClient.next();
						Object[] metadata = mapClients.get(clientId);
						// build a submenu item for each client:
						JMenuItem clientItem = new JMenuItem((String) metadata[0]);
						if (metadata[1] != null)
							clientItem.setIcon(getScaledImage(new ImageIcon((URL) metadata[1]), 24, 24));
						clientItem.addActionListener(new SampClientActionListener(mtype, clientId));
						mnuSpectrum_SendTo.add(clientItem);
					}
				}
			}
		}
	}

	// Menu INTERACTS:
	private final class InteractionItemListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			proxy.setIsInteracting(chkInteracts.isSelected());
			chkInteracts.setFont(chkInteracts.getFont().deriveFont((chkInteracts.isSelected()) ? Font.BOLD : Font.PLAIN));
			chkInteracts.setIcon(chkInteracts.isSelected() ? icoConnect : icoDisconnect);
		}
	}

	public void enableAladinSync(final boolean enable){
		chkInteracts.setSelected(enable);
	}

	/* ************** */
	/* LIST LISTENERS */
	/* ************** */
	private final class SpectrumListSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent evt) {
			if (evt.getValueIsAdjusting() == false) {
				Serie[] series = lstSEDs.getSelectedValues();

				// NO SELECTED SERIE:
				if (series.length == 0){
					// clear all metadata (no SED to display):
					showSEDMetadata(null);

					// disable the rename command:
					mnuSpectrum_Rename.setEnabled(false);

					// disable the lock commands:
					mnuSpectrum_Lock.setSelected(false);
					mnuSpectrum_Lock.setEnabled(false);
					cmdLock.setSelected(false);
					cmdLock.setEnabled(false);

					// disable the refresh commands:
					mnuSpectrum_Refresh.setEnabled(false);
					cmdReload.setEnabled(false);

				}// ONE SELECTED SERIE:
				else if (series.length == 1){
					// show metadata of the selected spectrum:
					SED sed = series[0].getSED();
					showSEDMetadata(sed);

					// enable the rename command:
					mnuSpectrum_Rename.setEnabled(true);

					// show its lock status:
					boolean enabled = (sed.getSource() != null);
					if (enabled && sed.getSource() instanceof PhotometrySource && ((PhotometrySource)sed.getSource()).isDisconnectedFromAladin())
						enabled = false;
					boolean frozen = (!enabled || sed.getSource().isFrozen());
					mnuSpectrum_Lock.setEnabled(enabled);
					cmdLock.setEnabled(enabled);
					if (enabled){
						mnuSpectrum_Lock.setSelected(frozen);
						cmdLock.setSelected(frozen);
					}
					// ...and the corresponding tool tip text:
					if (cmdLock.isSelected())
						cmdLock.setToolTipText("The spectrum is protected against any modification.");
					else
						cmdLock.setToolTipText("The spectrum can be modified/updated.");

					// show its refresh status:
					mnuSpectrum_Refresh.setEnabled(!frozen);
					cmdReload.setEnabled(!frozen);

					// selects the corresponding serie:
					cboSeries.setSelectedItem(lstSEDs.getSelectedValue());

				}// SEVERAL SELECTED SERIES:
				else{
					// clear all metadata (only visible if only one SED is selected):
					showSEDMetadata(null);

					// disable the rename command:
					mnuSpectrum_Rename.setEnabled(false);

					// Refresh the lock status of all selected Spectrums:
					cmdLock.setEnabled(true);
					cmdLock.setSelected(series[0].getSED().getSource() == null || series[0].getSED().getSource().isFrozen());
					mnuSpectrum_Lock.setEnabled(true);
					mnuSpectrum_Lock.setSelected(cmdLock.isSelected());

					// Enable the reload button only if all selected spectrums are not constant:
					boolean reloadAll = true;
					for(Serie s : series)
						reloadAll = reloadAll && (s.getSED().getSource() != null && !s.getSED().getSource().isFrozen());
					mnuSpectrum_Refresh.setEnabled(reloadAll);
					cmdReload.setEnabled(reloadAll);
				}
			}
		}

	}

	/* **************** */
	/* CHANGE LISTENERS */
	/* **************** */
	/**
	 * A PropertyChangeListener to follow the synchronization of the plugin with Aladin.
	 * @author Gr&eacute;gory Mantelet
	 * @version 04/2012
	 */
	public static final class SynchronizePropertyChangeListener implements PropertyChangeListener {
		protected MySwingWorker task;
		protected final JProgressBar progress;
		protected final JButton cmdSynchronize;
		protected final Timer timVisible;

		public SynchronizePropertyChangeListener(JProgressBar bar, JButton cmd){
			task = null;
			progress = bar;
			cmdSynchronize = cmd;
			timVisible = new Timer(3000, new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					progress.setVisible(false);
					timVisible.stop();
				}
			});
		}

		public final void setTask(MySwingWorker task){
			this.task = task;
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if (task == null)
				return;

			String property = evt.getPropertyName();

			if (property.equals("state")){
				StateValue state = (StateValue)evt.getNewValue();
				switch(state){
				// TASK BEGIN:
				case STARTED:
					if (progress != null){
						timVisible.stop();
						progress.setVisible(true);
						progress.setValue(0);
						progress.setString("waiting for Aladin...");
					}
					if (cmdSynchronize != null){
						cmdSynchronize.setText("");
						cmdSynchronize.setIcon(null);
						cmdSynchronize.setToolTipText("Cancel the synchronization with Aladin.");
						if (icoStop == null)
							cmdSynchronize.setText("Stop");
						else
							cmdSynchronize.setIcon(icoStop);
					}
					break;

					// TASK END
				case DONE:
					if (task.isCancelled()){
						if (progress != null)
							progress.setString("Cancelled");
					}else{
						if (progress != null){
							progress.setValue(100);
							progress.setString("Synchronized");
						}
					}
					if (progress != null)
						timVisible.start();
					if (cmdSynchronize != null){
						cmdSynchronize.setText("");
						cmdSynchronize.setIcon(null);
						cmdSynchronize.setToolTipText("Synchronize with Aladin.");
						if (icoSynchronize == null)
							cmdSynchronize.setText("Synchronize");
						else
							cmdSynchronize.setIcon(icoSynchronize);
					}
					break;
				}

				// TASK PROGRESSION:
			}else if (property.equals("progress")){
				if (progress != null)
					progress.setValue(((Integer)evt.getNewValue()).intValue());
			}else if (property.equals("ProgressMessage"))
				progress.setString(evt.getNewValue().toString());
		}
	}

	// Spinner POINT_NUMBER:
	private final class PointNumberChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			// Get the current value:
			int currentValue = ((Integer)spiPointNumber.getModel().getValue()).intValue();
			// Get the selected spectrum:
			SED sed = ((Serie)lstSEDs.getSelectedValue()).getSED();
			// Get the number of points into the selected spectrum:
			int nbPoints = (sed == null) ? 0 : sed.size();
			// If only one spectrum is selected and the given value is correct:
			if (lstSEDs.getSelectedIndices().length == 1 && currentValue > 0 && currentValue <= nbPoints) {
				// get the corresponding spectrum point:
				SEDPoint spt = sed.get(currentValue - 1);
				// show its abscissa value:
				lblWavelength.setText(FrameOfReference.getXValueString(spt.getOriginalX())+ " " + spt.getOriginalXUnit());
				// show its ordinate value:
				lblFlux.setText(FrameOfReference.getYValueString(spt.getOriginalY())+ " " + spt.getOriginalYUnit());
				// select the specified spectrum point in the graph:
				myGraph.selectPoint(lstSEDs.getSelectedValue(), currentValue-1);
				// if its origin is known...
				if (spt.getImage() != null) {
					// ...show its origin metadata:
					AladinImage io = spt.getImage();
					lblImage.setText(io.getName());
					//cmdSelectImage.setEnabled(true);
					cmdViewCharacFile.setEnabled(true);
					// ...select the corresponding image in Aladin:
					if (UserPreferences.getCurrentInstance().moveToOnSpectrumClick())
						proxy.highlightImage(io);

				}// else clear origin metadata fields:
				else {
					lblImage.setText("");
					//cmdSelectImage.setEnabled(false);
					cmdViewCharacFile.setEnabled(false);
				}

				// Else clear all metadata fields:
			} else {
				lblWavelength.setText("");
				lblFlux.setText("");
				lblImage.setText("");
				//cmdSelectImage.setEnabled(false);
				cmdViewCharacFile.setEnabled(false);
				myGraph.selectPoint(null, -1);
			}
		}
	}

	/* ************* */
	/* KEY LISTENERS */
	/* ************* */
	// List SPECTRUMS:
	private final class SpectrumListKeyListener extends KeyAdapter {
		@Override
		public void keyPressed(KeyEvent e) {
			// Delete key:
			if (e.getKeyCode() == KeyEvent.VK_DELETE){
				Object[] selectedValues = lstSEDs.getSelectedValues();
				SED[] seds = new SED[selectedValues.length];
				for(int i=0; i<selectedValues.length; i++)
					seds[i] = ((Serie)selectedValues[i]).getSED();
				myPlugin.hideSEDs(seds);

			}// F1 key:
			else if (e.getKeyCode() == KeyEvent.VK_F1) {
				cmdInfo.setSelected(!cmdInfo.isSelected());
				panMetadata.setVisible(cmdInfo.isSelected());
			}

			// F5 key:
			else if (e.getKeyCode() == KeyEvent.VK_F5) {
				Serie[] selectedValues = lstSEDs.getSelectedValues();
				for(Serie s : selectedValues)
					s.getSED().getSource().update(true);
			}
		}
	}

	// TextField GRAPH_TITLE:
	private final class GraphTitleKeyListener extends KeyAdapter {
		@Override
		public void keyReleased(KeyEvent e) {
			// Hide the graph title if empty::
			if (txtGraphTitle.getText().length() == 0)
				myGraph.getFrameOfReference().titleVisible = false;

			// Else show and update it:
			else {
				myGraph.getFrameOfReference().titleVisible = true;
				myGraph.getFrameOfReference().title = txtGraphTitle.getText();
			}

			// Refresh the graph:
			myGraph.repaint();

		}
	}

	/* *************** */
	/* MOUSE LISTENERS */
	/* *************** */
	// Button SEND_TO:
	private final class SendToMouseListener extends MouseAdapter {
		@Override
		public void mousePressed(MouseEvent e) {
			if (e.getButton() == MouseEvent.BUTTON1) {
				// Don't do anything if the plugin isn't registered with SAMP:
				if (myPlugin.isSampConnected()) {
					JPopupMenu popup = new JPopupMenu();
					// get the list of potential SAMP receivers:
					Map<String, HashMap<String, Object[]>> mapMTypes = myPlugin.getSampClients();
					// get only their name:
					Iterator<String> itMType = mapMTypes.keySet().iterator();
					int i=0;
					while (itMType.hasNext()) {
						String mtype = itMType.next();
						Map<String, Object[]> mapClients = mapMTypes.get(mtype);
						Iterator<String> itClient = mapClients.keySet().iterator();
						while (itClient.hasNext()) {
							String clientId = itClient.next();
							Object[] metadata = mapClients.get(clientId);
							// build a submenu item for each client:
							JMenuItem clientItem = new JMenuItem((String) metadata[0]);
							if (metadata[1] != null)
								clientItem.setIcon(getScaledImage(new ImageIcon((URL) metadata[1]), 24, 24));
							clientItem.addActionListener(new SampClientActionListener(mtype, clientId));
							popup.add(clientItem);
							i++;
						}
					}
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		}
	}

	/* ****************************** */
	/* UNIMPLEMENTED WINDOW LISTENERS */
	/* ****************************** */
	public void windowClosed(WindowEvent arg0) { ; }
	public void windowDeactivated(WindowEvent arg0) { ; }
	public void windowDeiconified(WindowEvent arg0) { ; }
	public void windowIconified(WindowEvent arg0) { ; }
	public void windowOpened(WindowEvent arg0) { ; }

}
