package sedplugin.interfaceGraphique;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Vector;


import javax.swing.BoxLayout;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.Scrollable;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionListener;

import sedplugin.graphe.Serie;

public class SEDJList extends JPanel implements Scrollable, ListDataListener, ListSelectionModel {
	private static final long serialVersionUID = 1L;

	private final MouseListener mouseListener = new SEDJListItemMouseListener();
	private final KeyListener keyListener = new SEDJListItemKeyListener(this);
	private final ListSelectionModel selectionModel = new DefaultListSelectionModel();
	private SEDListModel lstSed;

	private int current = -1;

	public SEDJList(final SEDListModel lst) {
		super();
		lstSed = lst;
		lstSed.addListDataListener(this);

		selectionModel.setSelectionMode(MULTIPLE_INTERVAL_SELECTION);

		setOpaque(false);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	public final boolean isCurrent(final int index){
		return index == current;
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension((int)super.getPreferredSize().getWidth(), getComponentCount()*SEDJListItem.HEIGHT);
	}

	public final SEDListModel getListModel(){
		return lstSed;
	}

	public final void setListModel(final SEDListModel model){
		if (lstSed != null){
			lstSed.removeListDataListener(this);
			lstSed = null;
		}
		lstSed = model;
		lstSed.addListDataListener(this);
	}

	public int getSelectedIndex(){
		return getMinSelectionIndex();
	}

	public Serie getSelectedValue(){
		if (isSelectionEmpty())
			return null;
		else
			return (Serie)lstSed.getElementAt(getMinSelectionIndex());
	}

	public int[] getSelectedIndices(){
		ArrayList<Integer> lstIndices = new ArrayList<Integer>();
		for(int i=0; i<lstSed.getSize(); i++){
			if (isSelectedIndex(i))
				lstIndices.add(i);
		}
		int[] indices = new int[lstIndices.size()];
		for(int i=0; i<indices.length; i++)
			indices[i] = lstIndices.get(i).intValue();
		return indices;
	}

	public Serie[] getSelectedValues(){
		ArrayList<Serie> lstValues = new ArrayList<Serie>();
		for(int i=0; i<lstSed.getSize(); i++){
			if (isSelectedIndex(i))
				lstValues.add((Serie)lstSed.getElementAt(i));
		}
		Serie[] values = new Serie[lstValues.size()];
		for(int i=0; i<values.length; i++)
			values[i] = lstValues.get(i);
		return values;
	}

	public void triggerRenameSelectedValue(){
		if (getSelectedIndices().length != 1)
			return;
		else
			((SEDJListItem)getComponent(getSelectedIndex())).triggerRename();
	}

	/* ********** */
	/* SCROLLABLE */
	/* ********** */
	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		return SEDJListItem.HEIGHT*3;
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		return SEDJListItem.HEIGHT;
	}

	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	public boolean getScrollableTracksViewportWidth() {
		return true;
	}

	/* ****************** */
	/* LIST DATA LISTENER */
	/* ****************** */
	public void fireContentsChanged(final int index){
		selectionModel.removeSelectionInterval(index, index);
		selectionModel.addSelectionInterval(index, index);
	}

	public void contentsChanged(ListDataEvent evt) {
		;
	}

	public void intervalAdded(ListDataEvent evt) {
		int index = getComponentCount();
		boolean toSelect = (getMaxSelectionIndex()-getMinSelectionIndex() == 0);
		Serie serie = (Serie)evt.getSource();
		SEDJListItem addedItem = (SEDJListItem)add(new SEDJListItem(this, serie, index));
		if (addedItem != null){
			if (serie != null && serie.getSED() != null && serie.getSED().getSource() != null)
				serie.getSED().addSEDLoadListener(addedItem);
			addedItem.addMouseListener(mouseListener);
			addedItem.addKeyListener(keyListener);
			addListSelectionListener(addedItem);
		}
		if (toSelect)
			setSelectionInterval(index, index);
	}

	public void intervalRemoved(ListDataEvent evt) {
		boolean selected = false;
		Serie serie;
		for(int i=evt.getIndex1(); i >= evt.getIndex0(); i--){
			SEDJListItem itemToRemove = (SEDJListItem)getComponent(i);
			itemToRemove.removeMouseListener(mouseListener);
			itemToRemove.removeKeyListener(keyListener);
			removeListSelectionListener(itemToRemove);
			serie = itemToRemove.getSerie();
			serie.releaseAutoColor();
			if (serie != null && serie.getSED() != null && serie.getSED().getSource() != null)
				serie.getSED().removeSEDLoadListener(itemToRemove);
			selected = selected || isSelectedIndex(i);
			remove(i);
			removeSelectionInterval(i, i);
		}
		if (evt.getIndex0() >= 0){
			for(int i=evt.getIndex0(); i<getComponentCount(); i++)
				((SEDJListItem)getComponent(i)).setIndex(i);
		}
		if (selected && getComponentCount() > 0){
			if (evt.getIndex0() < getComponentCount())
				addSelectionInterval(evt.getIndex0(), evt.getIndex0());
			else if (evt.getIndex0() > 0)
				addSelectionInterval(evt.getIndex0()-1, evt.getIndex0()-1);
		}
		repaint();
	}

	/* ******************** */
	/* LIST SELECTION MODEL */
	/* ******************** */
	public final void addListSelectionListener(ListSelectionListener listener) { selectionModel.addListSelectionListener(listener); }
	public final void addSelectionInterval(int index0, int index1) { selectionModel.addSelectionInterval(index0, index1); }
	public final void clearSelection() { current = -1; selectionModel.clearSelection(); }
	public final int getAnchorSelectionIndex() { return selectionModel.getAnchorSelectionIndex(); }
	public final int getLeadSelectionIndex() { return selectionModel.getLeadSelectionIndex(); }
	public final int getMaxSelectionIndex() { return selectionModel.getMaxSelectionIndex(); }
	public final int getMinSelectionIndex() { return selectionModel.getMinSelectionIndex(); }
	public final int getSelectionMode() { return selectionModel.getSelectionMode(); }
	public final boolean getValueIsAdjusting() { return selectionModel.getValueIsAdjusting(); }
	public final void insertIndexInterval(int index, int length, boolean before) { selectionModel.insertIndexInterval(index, length, before); }
	public final boolean isSelectedIndex(int index) { return selectionModel.isSelectedIndex(index); }
	public final boolean isSelectionEmpty() { return selectionModel.isSelectionEmpty(); }
	public final void removeIndexInterval(int index0, int index1) { selectionModel.removeIndexInterval(index0, index1); }
	public final void removeListSelectionListener(ListSelectionListener listener) { selectionModel.removeListSelectionListener(listener); }
	public final void removeSelectionInterval(int index0, int index1) { selectionModel.removeSelectionInterval(index0, index1); }
	public final void setAnchorSelectionIndex(int index) { selectionModel.setAnchorSelectionIndex(index); }
	public final void setLeadSelectionIndex(int index) { selectionModel.setLeadSelectionIndex(index); }
	public final void setSelectionInterval(int index0, int index1) {
		if (index0 == index1 && index0 >= 0 && index0 < getComponentCount())
			current = index0;
		selectionModel.setSelectionInterval(index0, index1);
	}
	public final void setSelectedValue(Serie s){
		int index = lstSed.indexOf(s);
		if (index >= 0 && index < lstSed.getSize())
			setSelectionInterval(index, index);
	}
	public final void setSelectionMode(int selectionMode) { selectionModel.setSelectionMode(selectionMode); }
	public final void setValueIsAdjusting(boolean isAdjusting) { selectionModel.setValueIsAdjusting(isAdjusting); }

	/* ******************* */
	/* ITEM MOUSE LISTENER */
	/* ******************* */
	protected final class SEDJListItemMouseListener extends MouseAdapter {

		private void removeAllSelected(final int index){
			int lowDiff = index-getMinSelectionIndex();
			int upDiff = getMaxSelectionIndex()-index;
			if (lowDiff <= upDiff)
				removeSelectionInterval(getMinSelectionIndex(), index);
			else
				removeSelectionInterval(index, getMaxSelectionIndex());
		}

		@Override
		public void mouseClicked(MouseEvent evt) {
			SEDJListItem item = (SEDJListItem)evt.getSource();
			int index = item.getIndex();
			int oldCurrent = current;
			current = index;

			switch(selectionModel.getSelectionMode()){
			case ListSelectionModel.SINGLE_INTERVAL_SELECTION:
				if (evt.isControlDown()){
					if (isSelectedIndex(index))
						removeAllSelected(index);
					else if (index == getMaxSelectionIndex()+1 || index == getMinSelectionIndex()-1){
						addSelectionInterval(index, index);
					}else
						setSelectionInterval(index, index);
				}else if (evt.isShiftDown()){
					if (oldCurrent <= index)
						setSelectionInterval(oldCurrent, index);
					else
						setSelectionInterval(index, oldCurrent);
				}else
					setSelectionInterval(index, index);

				break;

			case ListSelectionModel.MULTIPLE_INTERVAL_SELECTION:
				if (evt.isControlDown()){
					if (isSelectedIndex(index))
						removeSelectionInterval(index, index);
					else
						addSelectionInterval(index, index);
				}else if (evt.isShiftDown()){
					if (oldCurrent <= index)
						setSelectionInterval(oldCurrent, index);
					else
						setSelectionInterval(index, oldCurrent);
				}else
					setSelectionInterval(index, index);

				break;

			default:
				if (evt.isControlDown() && isSelectedIndex(index))
					clearSelection();
				else
					setSelectionInterval(index, index);
			}
			repaint();
		}
	}

	/* ***************** */
	/* ITEM KEY LISTENER */
	/* ***************** */
	protected final class SEDJListItemKeyListener implements KeyListener {
		private final SEDJList lst;
		public SEDJListItemKeyListener(final SEDJList lst){
			this.lst = lst;
		}
		private KeyEvent modifyEvent(KeyEvent evt){
			evt.setSource(lst);
			return evt;
		}
		public void keyPressed(KeyEvent evt) {
			if (evt.isControlDown() && evt.getKeyCode() == KeyEvent.VK_A){
				current = getComponentCount()-1;
				setSelectionInterval(0, current);
			}else if (!isSelectionEmpty() && !evt.isAltDown() && !evt.isAltGraphDown() && !evt.isMetaDown()){
				int oldCurrent = current;

				if (evt.getKeyCode() == KeyEvent.VK_DOWN)
					current = oldCurrent+1;
				else if (evt.getKeyCode() == KeyEvent.VK_UP)
					current = oldCurrent-1;
				else if (evt.getKeyCode() == KeyEvent.VK_HOME)
					current = 0;
				else if (evt.getKeyCode() == KeyEvent.VK_END)
					current = getComponentCount()-1;
				else if (evt.getKeyCode() == KeyEvent.VK_PAGE_UP){
					current = oldCurrent-3;
					if (current < 0)
						current = 0;
				}else if (evt.getKeyCode() == KeyEvent.VK_PAGE_DOWN){
					current = oldCurrent+3;
					if (current >= getComponentCount())
						current = getComponentCount()-1;
				}

				if (current >= 0 && current < getComponentCount()){
					if (evt.isControlDown() || evt.isShiftDown()){
						int index0 = Math.min(current, oldCurrent);
						int index1 = Math.max(current, oldCurrent);
						addSelectionInterval(index0, index1);
					}else
						setSelectionInterval(current, current);
				}else
					current = oldCurrent;
			}
			processKeyEvent(modifyEvent(evt));
		}
		public void keyReleased(KeyEvent evt) {
			processKeyEvent(modifyEvent(evt));
		}
		public void keyTyped(KeyEvent evt) {
			processKeyEvent(modifyEvent(evt));
		}
	}

	public void drawOnly(Vector<Serie> series) {
		Component[] comps = getComponents();
		for(Component c : comps){
			SEDJListItem item = (SEDJListItem)c;
			if (!series.contains(item.getSerie()))
				item.setChecked(false);
		}
	}

}
