package sedplugin.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;



import sedplugin.errors.ErrorsLog;
import sedplugin.errors.SEDException;
import sedplugin.plugin.SEDPlugin;
import sedplugin.sed.unit.AxisUnit;
import sedplugin.sed.unit.UnitDef;
import sedplugin.sed.unit.XAxis;


public class UnitsCustomizeDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	protected JPanel panModel;
	protected JComboBox cboModel;

	protected JPanel panCustom;
	protected JComponent[] customItems;

	/** Unit in which a new multiple of an available unit may be added. */
	protected AxisUnit unit;

	protected double toRefFactor;

	protected final static String[] BASE_UNITS = new String[]{"W", "m", "Jy", "Hz", "K"};

	protected final static String[] FACTORS_STRING = new String[]{"f", "p", "n", "u", "m", "c", "d", "", "da", "h", "k", "M", "G", "T"};

	protected final static double[] FACTORS = new double[]{1e15, 1e12, 1e9, 1e6, 1e3, 1e2, 1e1, 1.0, 1e-1, 1e-2, 1e-3, 1e-6, 1e-9, 1e-12};

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public UnitsCustomizeDialog(JFrame owner, AxisUnit unitToUpdate){
		super(owner);

		unit = unitToUpdate;

		build();
		fillCustomPanel();

		setTitle("Add a unit");
		URL iconTitle = getClass().getResource(SEDPlugin.ICONS_DIR + "rulers.gif");
		if (iconTitle != null)
			setIconImage(new ImageIcon(iconTitle).getImage());
		pack();
		setResizable(false);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setLocationRelativeTo(owner);
		setModal(true);
		setVisible(true);
	}

	/* **************** */
	/* BUILDING METHODS */
	/* **************** */

	protected void build(){
		// MODEL panel:
		panModel = new JPanel(new BorderLayout());
		panModel.add(new JLabel("Model: "), BorderLayout.WEST);

		cboModel = new JComboBox(unit.getAllUnits());
		cboModel.setEditable(false);
		cboModel.addActionListener(new ModelActionListener());
		panModel.add(cboModel, BorderLayout.CENTER);

		// CUSTOM UNIT panel:
		panCustom = new JPanel();

		// COMMANDS panel:
		JPanel panCommands = new JPanel();

		JButton cmdAdd = new JButton("Add");
		cmdAdd.addActionListener(new AddActionListener());
		panCommands.add(cmdAdd);

		JButton cmdCancel = new JButton("Cancel");
		cmdCancel.addActionListener(new CancelActionListener());
		panCommands.add(cmdCancel);

		// GENERAL panel:
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(panModel, BorderLayout.NORTH);
		getContentPane().add(panCustom, BorderLayout.CENTER);
		getContentPane().add(panCommands, BorderLayout.SOUTH);
	}

	protected void fillCustomPanel(){
		panCustom.removeAll();
		toRefFactor = 1.0;

		// Split the selected unit:
		String[] splitedUnit = ((UnitDef)cboModel.getSelectedItem()).symbol.split("/");

		// Create a component for each intern unit:
		for(int u = 0; u < splitedUnit.length; u++){
			// compute method:
			String unit = splitedUnit[u];
			String computeMethod = "";
			int indComputeMethod = unit.lastIndexOf('[');
			if (indComputeMethod > 0){
				computeMethod = " "+unit.substring(indComputeMethod);
				unit = unit.substring(0, indComputeMethod).trim();
			}

			// square unit ?
			boolean square = (unit.endsWith("2") || unit.endsWith("²"));
			if (square)
				unit = unit.substring(0, unit.length()-1);

			// which base unit ?
			int indUnit = -1, indFact = -1;
			for(int b = 0; b < BASE_UNITS.length; b++){
				if (unit.endsWith(BASE_UNITS[b])){
					indUnit = b;
					indFact = unit.indexOf(BASE_UNITS[b]);
					break;
				}
			}

			// which multiple and which mult-factor ?
			double multFact = 1.0;
			if (indFact > 0){
				String fact = unit.substring(0, indFact);
				indFact = 7;
				for(int f = 0; f < FACTORS_STRING.length; f++){
					if (FACTORS_STRING[f].equals(fact)){
						indFact = f;
						multFact = FACTORS[f];
						break;
					}
				}
			}else
				indFact = 7;
			if (square)
				multFact = Math.pow(multFact, 2);

			// update the global factor:
			toRefFactor = (u==0)?toRefFactor/multFact:toRefFactor*multFact;

			// add a unit separator if not the first unit part:
			if (u > 0)
				panCustom.add(new JLabel("/"));

			// create a ComboBox if its base unit has been found:
			if (indUnit > 0){
				String[] factors = new String[FACTORS.length];
				for(int f = 0; f < FACTORS_STRING.length && f < factors.length; f++){
					String unitPrefix = FACTORS_STRING[f];
					factors[f] = unitPrefix+BASE_UNITS[indUnit]+(square?"²":"")+computeMethod;
				}
				JComboBox cboFactors = new JComboBox(factors);
				cboFactors.setEditable(false);
				cboFactors.setSelectedIndex(indFact);

				panCustom.add(cboFactors);

				// create a JLabel else:
			}else
				panCustom.add(new JLabel(unit+(square?"²":"")+computeMethod));
		}

		pack();
	}

	/* ********* */
	/* LISTENERS */
	/* ********* */
	protected final class ModelActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK){
				fillCustomPanel();
			}
		}
	}

	protected final class AddActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			double multFact = 1.0;
			String symbol = "";
			try{
				for(int i=0; i<panCustom.getComponentCount(); i++){
					Component comp = panCustom.getComponent(i);
					if (comp instanceof JLabel)
						symbol += ((JLabel)comp).getText();
					else if (comp instanceof JComboBox){
						JComboBox cboFactor = (JComboBox)comp;
						symbol += (String)cboFactor.getSelectedItem();
						double temp = FACTORS[cboFactor.getSelectedIndex()];
						if (symbol.endsWith("²"))
							temp = Math.pow(temp, 2);
						multFact = (i==0)?multFact/temp:multFact*temp;
					}else
						throw new SEDException("Can't add the new unit", "Adding a custom unit", "Unknown component: "+comp.getClass());
				}

				UnitDef newUnit = new UnitDef(symbol);
				double convFactor = unit.getQuantity().getConvFactor(((UnitDef)cboModel.getSelectedItem()).symbol);
				unit.getQuantity().add(newUnit, convFactor*(toRefFactor/multFact));

			}catch(SEDException se){
				ErrorsLog.getCurrentInstance().addException(se);
			}
			dispose();
		}
	}

	protected final class CancelActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			dispose();
		}
	}


	/* *********** */
	/* MAIN METHOD */
	/* *********** */
	public static void main(String[] args) throws Exception {
		new UnitsCustomizeDialog(null, new XAxis());
	}

}
