package sedplugin.plugin;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.SwingWorker.StateValue;

import sedplugin.errors.ErrorsLog;
import sedplugin.errors.SEDException;


public abstract class MySwingWorker {

	private Thread task = null;
	private Object result = null;
	private boolean done = false;
	private boolean cancel = false;
	private PropertyChangeSupport support;
	private int progress = 0;
	private StateValue state = StateValue.PENDING;

	private class SwingWorkerTask extends Thread {
		@Override
		public void run() {
			state = StateValue.STARTED;
			firePropertyChange("state", StateValue.PENDING, StateValue.STARTED);

			try {
				result = doInBackground();
			} catch (InterruptedException ex) {
				interrupt();
			} catch (Exception ex) {
				ErrorsLog.getCurrentInstance().addException(new SEDException(ex, "MySwingWorkerTask"));
			}

			done = true;
			state = StateValue.DONE;
			firePropertyChange("state", StateValue.STARTED, StateValue.DONE);
		}
	}

	public MySwingWorker(){
		support = new PropertyChangeSupport(this);
		task = new SwingWorkerTask();
	}

	public final void addPropertyChangeListener(PropertyChangeListener listener){
		support.addPropertyChangeListener(listener);
	}

	public final void removePropertyChangeListener(PropertyChangeListener listener){
		support.removePropertyChangeListener(listener);
	}

	public final void firePropertyChange(String propertyName, Object oldValue, Object newValue){
		support.firePropertyChange(propertyName, oldValue, newValue);
	}

	public final PropertyChangeSupport getPropertyChangeSupport(){
		return support;
	}

	protected final void setProgress(int progress){
		firePropertyChange("progress", new Integer(this.progress), new Integer(progress));
		this.progress = progress;
	}

	public final int getProgress(){
		return progress;
	}

	public final StateValue getState(){
		return state;
	}

	public final boolean isDone(){
		return done;
	}

	public final boolean cancel(boolean mayInterruptIfRunning){
		if (done || cancel)
			return false;

		try{
			cancel = true;
			task.join(1500);
			cancel = done;
		}catch(InterruptedException ie){
			cancel = true;
		}catch(Throwable t){
			cancel = false;
		}

		if (!cancel && mayInterruptIfRunning){
			try{
				task.interrupt();
				cancel = true;
				task.join(500);
				cancel = done;
			}catch(InterruptedException ie){
				cancel = true;
			}catch(Throwable t){
				cancel = false;
			}
		}

		return cancel;
	}

	public final boolean isCancelled(){
		return cancel;
	}

	public synchronized final Object get() throws Exception  {
		while (!done)
			Thread.sleep(500);

		if (isCancelled())
			throw new Exception("CANCELLED");
		else if (task.isInterrupted())
			throw new InterruptedException("INTERRUPTED");
		else
			return result;
	}

	public synchronized final Object get(long timeout) throws Exception {
		long time = 0;
		while (!done && time < timeout){
			time += timeout/3;
			Thread.sleep(time);
		}
		if (!done){
			task.interrupt();
			throw new Exception("The result of this task has been requested in only "+timeout+"s !");
		}

		if (isCancelled())
			throw new Exception("CANCELLED");
		else if (task.isInterrupted())
			throw new InterruptedException("INTERRUPTED");
		else
			return result;
	}

	public final void execute() {
		if (state == StateValue.PENDING && !cancel)
			task.start();
	}

	protected abstract Object doInBackground() throws Exception;
}
