package sedplugin.sed.source.aladin;

import sedplugin.plugin.SEDPlugin;

import cds.aladin.Obj;


public class Circle extends PhotometrySource {

	public final static String CIRCLE_TYPE = "CIRCLE";

	private int nbImages = -1;
	private double radius = Double.NaN;
	protected double ra = Double.NaN;
	protected double dec = Double.NaN;
	protected String sexaCoords = null;

	static int count = 0;

	public Circle(final Obj object, final Drawing drawing, final SEDPlugin plugin){
		super(object, drawing, plugin);
	}

	public final String getType() {
		return CIRCLE_TYPE;
	}

	public final String getSexaCoords(){
		return sexaCoords;
	}

	public final double getRadius(){
		return radius;
	}

	@Override
	public boolean isModified(){
		return (sed == null || drawing.getNbCorrectImages() != nbImages || object.getRadius() != radius || sexaCoords == null || !sexaCoords.equals(object.getSexa()));
	}

	@Override
	public void updateModifiedFields(){
		nbImages = drawing.getNbCorrectImages();
		radius = object.getRadius();
		sexaCoords = object.getSexa();
	}

	/* **************** */
	/* EXPORT FUNCTIONS */
	/* **************** */
	public final String toASCIIFormat(String sep) {
		return null;
	}

	public String toVOTableFormat(String linePrefix) {
		StringBuffer voTable = new StringBuffer();
		voTable.append(linePrefix).append("<GROUP utype=\"spec:Char.SpatialAxis\">\n");
		voTable.append(linePrefix).append("\t<GROUP utype=\"spec:Char.SpatialAxis.Coverage\">\n");
		voTable.append(linePrefix).append("\t\t<GROUP utype=\"spec:SpatialAxis.Coverage.Location\">\n");
		voTable.append(linePrefix).append("\t\t\t<PARAM name=\"SkyPosition\" utype=\"spec:Char.SpatialAxis.Coverage.Location.Value\" ucd=\"pos.eq\" unit=\"deg\" datatype=\"double\" arraysize=\"2\" value=\"").append(object.raj).append(' ').append(object.dej).append("\"/>\n");
		voTable.append(linePrefix).append("\t\t</GROUP>\n");
		voTable.append(linePrefix).append("\t\t<GROUP utype=\"spec:Char.SpatialAxis.Coverage.Bounds\">\n");
		voTable.append(linePrefix).append("\t\t\t<PARAM name=\"SkyExtent\" utype=\"spec:Char.SpatialAxis.Coverage.Bounds.Extent\" ucd=\"instr.fov\" datatype=\"double\" unit=\"arcsec\" value=\"").append(object.getRadius()).append("\" />\n");
		voTable.append(linePrefix).append("\t\t</GROUP>\n");
		voTable.append(linePrefix).append("\t</GROUP>\n");
		voTable.append(linePrefix).append("</GROUP>\n");
		return voTable.toString();
	}

	public String toXMLFormat(String linePrefix) {
		StringBuffer voTable = new StringBuffer();
		voTable.append(linePrefix).append("<SpatialAxis name=\"Sky\" ucd=\"pos.eq\" unit=\"deg\">\n");
		voTable.append(linePrefix).append("\t<CoordSystem idref=\"ID000001\" />\n");
		voTable.append(linePrefix).append("\t<Coverage>\n");
		voTable.append(linePrefix).append("\t\t<Location>\n");
		voTable.append(linePrefix).append("\t\t\t<Value ucd=\"pos.eq.ra\" unit=\"deg\">").append(object.raj).append("</Value>\n");
		voTable.append(linePrefix).append("\t\t\t<Value ucd=\"pos.eq.dec\" unit=\"deg\">").append(object.dej).append("</Value>\n");
		voTable.append(linePrefix).append("\t\t</Location>\n");
		voTable.append(linePrefix).append("\t\t<Bounds>\n");
		voTable.append(linePrefix).append("\t\t\t<Extent ucd=\"instr.fov\" unit=\"arcsec\">").append(object.getRadius()).append("</Extent>\n");
		voTable.append(linePrefix).append("\t\t</Bounds>\n");
		voTable.append(linePrefix).append("\t</Coverage>\n");
		voTable.append(linePrefix).append("</SpatialAxis>\n");
		return voTable.toString();
	}

}
