package sedplugin.sed.source.aladin;

import sedplugin.plugin.SEDPlugin;

import cds.aladin.Obj;


public class Polygon extends PhotometrySource {

	public final static String POLYGON_TYPE = "POLYGON";

	static int count = 0;

	private double ra = Double.NaN;
	private double dec = Double.NaN;
	private String coords = null;
	private int nbImages = -1;


	public Polygon(Obj object, Drawing drawing, SEDPlugin plugin){
		super(object, drawing, plugin);
	}

	public String getType() {
		return POLYGON_TYPE;
	}

	@Override
	public final boolean isModified(){
		String newCoords = drawing.getPolygonCoords(object);
		return (sed == null || drawing.getNbCorrectImages() != nbImages || (coords == null && newCoords != null) || (newCoords == null && coords != null) || (coords != null && newCoords != null && !coords.equals(newCoords)));
	}

	@Override
	protected void updateModifiedFields() {
		nbImages = drawing.getNbCorrectImages();
		coords = drawing.getPolygonCoords(object);
		double[] centerPos = drawing.getPolygonCenter(object);
		ra = centerPos[0]; dec = centerPos[1];
	}

	/* **************** */
	/* EXPORT FUNCTIONS */
	/* **************** */
	public final String toASCIIFormat(String sep) {
		return null;
	}

	public String toVOTableFormat(String linePrefix) {
		StringBuffer voTable = new StringBuffer();
		voTable.append(linePrefix).append("<GROUP utype=\"spec:Char.SpatialAxis\">\n");
		voTable.append(linePrefix).append("\t<GROUP utype=\"spec:Char.SpatialAxis.Coverage\">\n");
		voTable.append(linePrefix).append("\t\t<GROUP utype=\"spec:SpatialAxis.Coverage.Location\">\n");
		voTable.append(linePrefix).append("\t\t\t<PARAM name=\"SkyPosition\" utype=\"spec:Char.SpatialAxis.Coverage.Location.Value\" ucd=\"pos.eq\" unit=\"deg\" datatype=\"double\" arraysize=\"2\" value=\"").append(ra).append(' ').append(dec).append("\"/>\n");
		voTable.append(linePrefix).append("\t\t</GROUP>\n");
		// TODO SUPPORT with a STC-S definition of the polygon
		/*voTable.append(linePrefix).append("\t\t<GROUP utype=\"spec:Char.SpatialAxis.Coverage.Bounds\">\n");
		voTable.append(linePrefix).append("\t\t\t<PARAM name=\"SkyExtent\" utype=\"spec:Char.SpatialAxis.Coverage.Bounds.Extent\" ucd=\"instr.fov\" datatype=\"double\" unit=\"arcsec\" value=\"").append(object.getRadius()).append("\" />\n");
		voTable.append(linePrefix).append("\t\t</GROUP>\n");*/
		voTable.append(linePrefix).append("\t</GROUP>\n");
		voTable.append(linePrefix).append("</GROUP>\n");
		return voTable.toString();
	}

	public String toXMLFormat(String linePrefix) {
		StringBuffer voTable = new StringBuffer();
		voTable.append(linePrefix).append("<SpatialAxis name=\"Sky\" ucd=\"pos.eq\" unit=\"deg\">\n");
		voTable.append(linePrefix).append("\t<CoordSystem idref=\"ID000001\" />\n");
		voTable.append(linePrefix).append("\t<Coverage>\n");
		voTable.append(linePrefix).append("\t\t<Location>\n");
		voTable.append(linePrefix).append("\t\t\t<Value ucd=\"pos.eq.ra\" unit=\"deg\">").append(ra).append("</Value>\n");
		voTable.append(linePrefix).append("\t\t\t<Value ucd=\"pos.eq.dec\" unit=\"deg\">").append(dec).append("</Value>\n");
		voTable.append(linePrefix).append("\t\t</Location>\n");
		// TODO SUPPORT with a STC-S definition of the polygon
		/*voTable.append(linePrefix).append("\t\t<Bounds>\n");
		voTable.append(linePrefix).append("\t\t\t<Extent ucd=\"instr.fov\" unit=\"arcsec\">").append(object.getRadius()).append("</Extent>\n");
		voTable.append(linePrefix).append("\t\t</Bounds>\n");*/
		voTable.append(linePrefix).append("\t</Coverage>\n");
		voTable.append(linePrefix).append("</SpatialAxis>\n");
		return voTable.toString();
	}

}
