package sedplugin.sed.source.operation;

import sedplugin.errors.SEDException;
import sedplugin.sed.SED;
import sedplugin.sed.SEDPoint;

public class ConstantToken extends OperationToken {

	public final double value;

	public ConstantToken(double value, int begin, int end){
		super(value+"", TokenType.CONSTANT, begin, end);
		this.value = value;
	}

	public ConstantToken(String expr, int begin, int end) throws SEDException {
		super(expr, TokenType.CONSTANT, begin, end);
		try{
			value = Double.parseDouble(expr);
		}catch(NumberFormatException nfe){
			throw new SEDException("Incorrect number", "Parsing an operation", "\""+expr+"\" is not a number. Position: between col."+begin+" and col."+end);
		}
	}

	public SED toSED(final SED pattern){
		if (pattern == null)
			return null;

		SED sed = new SED(pattern.getXAxis(), pattern.getYAxis());
		for(SEDPoint p : pattern)
			sed.add(new SEDPoint(p.getX(), p.getXUnit(), value, p.getYUnit()));
		return sed;
	}

	@Override
	public final int getNbPoints(){
		return 1;
	}

}
