package sedplugin.sed.unit;

/**
 * Unit entry.
 * It corresponds only to one symbol.
 * 
 * @author Gr&eacute;gory Mantelet (CDS)
 * @version 03/2012
 */
public class UnitDef {
	/** Unit symbol. */
	public final String symbol;

	/** Unicode version of the unit symbol. */
	public String unicodeSymbol;

	/**
	 * Builds a unit with its symbol.
	 * 
	 * @param unitSymbol	Unit symbol.
	 */
	public UnitDef(String unitSymbol){
		this(unitSymbol, unitSymbol);
	}

	/**
	 * Builds a unit with its symbol and a unicode version of the given symbol.
	 * 
	 * @param unitSymbol		Unit symbol.
	 * @param unicodeUnitSymbol	Unit symbol in unicode.
	 */
	public UnitDef(String unitSymbol, String unicodeUnitSymbol){
		symbol = unitSymbol;
		unicodeSymbol = unicodeUnitSymbol;
	}

	public String toString(){ return symbol; }

	/**
	 * Two UnitDef objects are equals only if their symbol is equal. The unicode symbol is not used there.
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		return (obj instanceof UnitDef) && ((UnitDef)obj).symbol.equals(symbol);
	}
}
